// Editor.cpp : Defines the class behaviors for the application.//
#include "stdafx.h"
#include "Editor.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "EditorDoc.h"
#include "EditorView.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
/////////////////////////////////////////////////////////////////////////////
// CEditorApp
BEGIN_MESSAGE_MAP(CEditorApp, CWinApp)
	//{{AFX_MSG_MAP(CEditorApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_WINDOW_CLOSE_ALL, OnWindowCloseAll)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_SAVE_ALL, OnFileSaveAll)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()
/////////////////////////////////////////////////////////////////////////////
// CEditorApp construction
CEditorApp::CEditorApp()
{
}
/////////////////////////////////////////////////////////////////////////////
// The one and only CEditorApp object
CEditorApp theApp;
/////////////////////////////////////////////////////////////////////////////
// CEditorApp initialization
const int maxsaveddocuments = 10;	// maximum # of documents remembered when reloading
BOOL CEditorApp::InitInstance()
{	
	// Standard initialization	
#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif
	SetRegistryKey("Al Stevens Programs");
	LoadStdProfileSettings(10);
	m_pDocTemplate = new CMultiDocTemplate(
		IDR_EDITORTYPE,
		RUNTIME_CLASS(CEditorDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CEditorView));
	AddDocTemplate(m_pDocTemplate);
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Enable DDE Execute open
	EnableShellOpen();

	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// --- suppress initial FileNew command on startup
	if (cmdInfo.m_nShellCommand != CCommandLineInfo::FileNew)	{
		if (!ProcessShellCommand(cmdInfo))
			return FALSE;
	}
	else	{
		// ----- reload documents from the previous session
		CString strProfile;
		int nDocno = 0;
		CWnd* pFocusWnd = 0;
		for (; nDocno < maxsaveddocuments; nDocno++)	{
			strProfile.Format("Document%d", nDocno+1);
			CString strDocument = GetProfileString("Documents", strProfile, 0);
			if (strDocument.IsEmpty())
				break;

			if (OpenDocumentFile(strDocument))	{
				// --- if the document was maximized on exit, maximize it now
				strProfile.Format("Document%d Maximized", nDocno+1);
				BOOL bIsMax = GetProfileInt("Documents", strProfile, 0);
				if (bIsMax)	{
					CEditorDoc* pDoc = 0;
					POSITION tpos = GetFirstDocTemplatePosition();
					while (tpos != 0)	{
						CDocTemplate* pTem = GetNextDocTemplate(tpos);
						ASSERT(pTem != 0);
						POSITION dpos = pTem->GetFirstDocPosition();
						while (dpos != 0)	{
							pDoc = static_cast<CEditorDoc*>(pTem->GetNextDoc(dpos));
							ASSERT(pDoc != 0);
							CString strDocFile = pDoc->GetPathName();
							if (strDocument.CompareNoCase(strDocFile) == 0)
								break;
						}
					}
					if (pDoc != 0)	{
						POSITION vpos = pDoc->GetFirstViewPosition();
						ASSERT(vpos != 0);
						CEditorView* pView = static_cast<CEditorView*>(pDoc->GetNextView(vpos));
						ASSERT(pView != 0);
						pFocusWnd = pView->GetParent();
						ASSERT(pFocusWnd != NULL);
						WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
						if (pFocusWnd->GetWindowPlacement(&wndpl))	{
							wndpl.showCmd = SW_SHOWMAXIMIZED;
							pFocusWnd->SetWindowPlacement(&wndpl);
						}
					}
				}
			}
		}
		if (pFocusWnd != 0)
			pFocusWnd->SetFocus();
	}
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();
	return TRUE;
}

void CEditorApp::SaveOpenDocuments()
{
	POSITION tpos = GetFirstDocTemplatePosition();
	int nDocno = 0;
	CString strProfile;
	while (tpos != 0 && nDocno < maxsaveddocuments)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CEditorDoc* pDoc = static_cast<CEditorDoc*>(pTem->GetNextDoc(dpos));
			ASSERT(pDoc != 0);
			CString strPath = pDoc->GetPathName();
			strProfile.Format("Document%d", ++nDocno);
			WriteProfileString("Documents", strProfile, strPath);
			POSITION vpos = pDoc->GetFirstViewPosition();
			ASSERT(vpos != 0);
			CEditorView* pView = static_cast<CEditorView*>(pDoc->GetNextView(vpos));
			ASSERT(pView != 0);
			CWnd* pWnd = pView->GetParent();
			ASSERT(pWnd != NULL);
			WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
			if (pWnd->GetWindowPlacement(&wndpl))	{
				strProfile.Format("Document%d Maximized", nDocno);
				WriteProfileInt("Documents", strProfile, 
					wndpl.showCmd == SW_SHOWMAXIMIZED);

			}
		}
	}
	while (nDocno < maxsaveddocuments)	{
		strProfile.Format("Document%d", ++nDocno);
		WriteProfileString("Documents", strProfile, 0);
		strProfile.Format("Document%d Maximized", nDocno);
		WriteProfileString("Documents", strProfile, 0);
	}
}

void CEditorApp::SaveWindowSettings()
{
	CWnd* pWnd = GetViewWindow();
	if (pWnd != 0)	{
		WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
		if (pWnd->GetWindowPlacement(&wndpl))
			WriteProfileInt("Settings", "Maximized", wndpl.showCmd == SW_SHOWMAXIMIZED);
	}
}

void CEditorApp::MaximizeDocuments()
{
	CWnd* pWnd = GetViewWindow();
	if (pWnd != 0)	{
		WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
		if (pWnd->GetWindowPlacement(&wndpl))	{
			wndpl.showCmd = SW_SHOWMAXIMIZED;
			pWnd->SetWindowPlacement(&wndpl);
		}
	}
}

CWnd* CEditorApp::GetViewWindow()
{
	CWnd* pWnd = 0;
	POSITION dpos = m_pDocTemplate->GetFirstDocPosition();
	if (dpos != 0)	{
		CDocument* pDoc = m_pDocTemplate->GetNextDoc(dpos);
		if (pDoc != 0)	{
			POSITION vpos = pDoc->GetFirstViewPosition();
			if (vpos != 0)	{
				CView* pView = pDoc->GetNextView(vpos);
				if (pView != 0)
					pWnd = pView->GetParent();
			}
		}
	}
	return pWnd;
}

/////////////////////////////////////////////////////////////////////////////
// CEditorApp commands

void CEditorApp::OnAppAbout()
{	
	CDialog aboutDlg(IDD_ABOUTBOX);
	aboutDlg.DoModal();
}

void CEditorApp::OnWindowCloseAll() 
{
	SaveAllModified();
	CloseAllDocuments(FALSE);	
}

// ----- overridden to permit CEditorView to intercept command
void CEditorApp::OnFileOpen() 
{
	CWinApp::OnFileOpen();
}

void CEditorApp::OnFileSaveAll() 
{
	POSITION tpos = GetFirstDocTemplatePosition();
	while (tpos != 0)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CDocument* pDoc = pTem->GetNextDoc(dpos);
			ASSERT(pDoc != 0);
			if (pDoc->IsModified())	{
				const CString& strPath = pDoc->GetPathName();
				if (!strPath.IsEmpty())	{
					pDoc->OnSaveDocument(strPath);
					pDoc->SetModifiedFlag(FALSE);
				}
			}
		}
	}
}

