// ----- grep.cpp

#include "stdafx.h"
#include "Quincy.h"
#include "GrepInput.h"
#include "GrepResult.h"

void CQuincyApp::OnUpdateGrep(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bGrepIsInstalled);
}
void CQuincyApp::OnGrep() 
{
	ASSERT(m_pdlgGrepInput != 0);
	// --- create the stdin, stdout, stderr files for grep to use
	CString strin = WorkingFile("quincy.in");
	CString strout = WorkingFile("quincy.out");
	CString strerr = WorkingFile("quincy.err");

	BuildStdin(strin);
	BuildStdout(strout);
	BuildStderr(strerr);

	if (m_pdlgGrepInput->DoModal() == IDOK)	{
		CString strCmd = m_strQuincyBinPath + "\\grep.exe -n ";
		if (!m_pdlgGrepInput->m_bMatchCase)
			strCmd += "-i ";
		if (m_pdlgGrepInput->m_bMatchWord)
			strCmd += "-w ";
		strCmd += "\"" + m_pdlgGrepInput->m_strGrepInput + "\"";
		if (RunGrep(strCmd + " *.h"))
			if (RunGrep(strCmd + " *.c"))
				if (RunGrep(strCmd + " *.cpp"))
					RunGrep(strCmd + " *.rc");
	}
	CloseConsoleHandles();

	ASSERT(m_pdlgGrepResult != 0);
	if (m_bGrepResultCreated != TRUE)	{
		m_pdlgGrepResult->Create(IDD_GREPS);
		m_bGrepResultCreated = TRUE;
	}

	m_pdlgGrepResult->m_ResultList.ResetContent();
	m_pdlgGrepResult->Invalidate();

	m_nGrepCount = 0;

	CStdioFile ifile(strout, CFile::modeRead | CFile::shareDenyNone);
	CString strLine;
	while (ifile.ReadString(strLine) && m_nGrepCount < maxgreps)	{
		// ------ put entry into grep response
		int ndx = strLine.Find("No such file or directory");
		if (ndx == -1)	{
			do
				if ((ndx = strLine.Find('\t')) != -1)
					strLine.SetAt(ndx, ' ');
			while (ndx != -1);
			m_pdlgGrepResult->m_ResultList.AddString(strLine);
			if ((ndx = strLine.Find(':')) == -1)
				continue;
			CString strFile = strLine.Left(ndx);
			int lineno = atoi(strLine.GetBuffer(0) + ndx + 1);
			GrepLog[m_nGrepCount].m_strFileSpec = strFile;
			GrepLog[m_nGrepCount++].m_nLineNumber = lineno;
		}
	}
	if (m_nGrepCount == 0)
		m_pdlgGrepResult->m_ResultList.AddString(
			CString("\"" + m_pdlgGrepInput->m_strGrepInput + "\": No matches were found"));

	m_pdlgGrepResult->ShowWindow(SW_SHOW);

	ifile.Close();

	remove(strin);
	remove(strout);
	remove(strerr);
}

BOOL CQuincyApp::RunGrep(const CString& strCmd)
{
	STARTUPINFO suinfo = {
		sizeof(STARTUPINFO),	// cb
		0, 0,					// reserved
		"Grep",					// lpTitle
		0, 0,					// dwX, dwY
		0, 0,					// dwXSize, dwYSize
		0, 0,					// dwXCountChars, dwYCountChars
		0,						// dwFillAttribute
		STARTF_USESHOWWINDOW |
		STARTF_USESTDHANDLES,	// dwFlags
		SW_SHOWMINIMIZED,		// wShowWindow
		0, 0,					// reserved
		m_hStdin,				// hStdInput;
		m_hStdout,				// hStdOutput;
		m_hStdout,				// hStdError;
	};
	PROCESS_INFORMATION pi;
	char* pCmd = ((CString&)strCmd).GetBuffer(0);
	BOOL rtn = CreateProcess(	0,
								pCmd,
								0,
								0,
								TRUE, 
								DETACHED_PROCESS, 
								0,
								0,
								&suinfo,&pi);
	if (rtn)	{
		while (WaitForSingleObject(pi.hProcess,100) == WAIT_TIMEOUT)
			;
		CloseHandle(pi.hThread);
		CloseHandle(pi.hProcess);
	}
	else
		AfxMessageBox("Cannot execute grep utility program", MB_ICONSTOP);
	return rtn;
}


