// QWindow.cpp : implementation file
//
#include "stdafx.h"
#include "quincy.h"
#include "QWindow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// QWindow

QWindow::QWindow()
{
	m_pClearBrush = new CBrush(RGB(255,255,255));
	m_pFirstEvent = 0;
	m_pLastEvent = 0;
}

QWindow::~QWindow()
{
	DeleteAllEvents();
	delete m_pClearBrush;
}

BEGIN_MESSAGE_MAP(QWindow, CWnd)
	//{{AFX_MSG_MAP(QWindow)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void QWindow::AddEvent(QWEvent* pQWEvent)
{
	if (m_pFirstEvent == 0)
		m_pFirstEvent = pQWEvent;
	if (m_pLastEvent != 0)
		m_pLastEvent->m_pNextEvent = pQWEvent;
	m_pLastEvent = pQWEvent;
}
void QWindow::DeleteAllEvents()
{
	QWEvent* pThisEvent = m_pFirstEvent;
	while (pThisEvent != 0)	{
		QWEvent* pNextEvent = pThisEvent->m_pNextEvent;
		delete pThisEvent;
		pThisEvent = pNextEvent;
	}
	m_pFirstEvent = 0;
	m_pLastEvent = 0;
}
void QWindow::MoveTo(int x, int y)
{
	QWMovement* pMovement = new QWMovement(CPoint(x, y));
	AddEvent(pMovement);
	CDC* pDC = GetDC();
	pMovement->Display(pDC);
	ReleaseDC(pDC);
}
void QWindow::LineTo(int x, int y)
{
	QWVector* pVector = new QWVector(CPoint(x, y));
	AddEvent(pVector);
	CDC* pDC = GetDC();
	pVector->Display(pDC);
	ReleaseDC(pDC);
}
void QWindow::Erase()
{
	DeleteAllEvents();
	Invalidate();
}
void QWindow::TextOut(char* cp)
{
	QWText* pText = new QWText(cp);
	AddEvent(pText);
	CDC* pDC = GetDC();
	pText->Display(pDC);
	ReleaseDC(pDC);
}
void QWindow::Rectangle(int ht, int wd)
{
	QWRectangle* pRect = new QWRectangle(CPoint(wd,ht));
	AddEvent(pRect);
	CDC* pDC = GetDC();
	pRect->Display(pDC);
	ReleaseDC(pDC);
}
void QWindow::Circle(int rad)
{
	QWCircle* pCircle = new QWCircle(rad);
	AddEvent(pCircle);
	CDC* pDC = GetDC();
	pCircle->Display(pDC);
	ReleaseDC(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// QWindow message handlers

void QWindow::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CRect rc;
	GetClientRect(&rc);

	dc.FillRect(&rc, m_pClearBrush);

	QWEvent* pThisEvent = m_pFirstEvent;
	while (pThisEvent != 0)	{
		pThisEvent->Display(&dc);
		pThisEvent = pThisEvent->m_pNextEvent;
	}
}

