// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Quincy.h"
#include "debugger.h"

#include "MainFrm.h"

#include "TraceDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	ON_WM_CLOSE()
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_CLOSE()
	ON_COMMAND(ID_WINDOW_CLOSE_ALL, OnWindowCloseAll)
	ON_COMMAND(ID_FILE_SAVE_ALL, OnFileSaveAll)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_ALL, OnUpdateFileSaveAll)
	ON_WM_TIMER()
	ON_UPDATE_COMMAND_UI(ID_USESTDLIB, OnUpdateUsestdlib)
	//}}AFX_MSG_MAP
	// Global help commands
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_EXT,
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_nTraceWidth = 0;
}

CMainFrame::~CMainFrame()
{
}

void CMainFrame::HideTYCPPCommand()
{
	m_wndToolBar.GetToolBarCtrl().HideButton(ID_TYCPP);
	CMenu* menu = GetMenu();
	ASSERT(menu != 0);
	menu->RemoveMenu(ID_TYCPP, MF_BYCOMMAND);
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	LoadWindowPlacement();
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

#ifndef _DEBUG
	m_wndToolBar.GetToolBarCtrl().HideButton(ID_WRITETUTORIAL);
#endif

#ifndef TYCPP
  #ifdef _DEBUG
	m_wndToolBar.GetToolBarCtrl().HideButton(ID_WRITETUTORIAL);
  #endif
	HideTYCPPCommand();
#endif

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}




	m_wndStatusBar.SetPaneInfo(1, ID_INDICATOR_EXT, 0, 120);
	m_wndStatusBar.SetPaneText(1, "");
  
	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	LoadBarState("Settings");

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CMDIFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

void CMainFrame::LoadWindowPlacement()
{
	CString strBuffer = theApp.GetProfileString("Settings", "WindowPos");
	if (!strBuffer.IsEmpty())	{
		WINDOWPLACEMENT wp;
		int cRead = _stscanf(strBuffer, "%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
				&wp.flags, &wp.showCmd,
				&wp.ptMinPosition.x, &wp.ptMinPosition.y,
				&wp.ptMaxPosition.x, &wp.ptMaxPosition.y,
				&wp.rcNormalPosition.left, &wp.rcNormalPosition.top,
				&wp.rcNormalPosition.right, &wp.rcNormalPosition.bottom);

		if (cRead == 10)	{
			wp.showCmd = theApp.m_nCmdShow;
			if (wp.flags & WPF_RESTORETOMAXIMIZED)
				theApp.m_nCmdShow = SW_SHOWMAXIMIZED;
			SetWindowPlacement(&wp);
		}
	}
}

VOID CMainFrame::SaveWindowPlacement()
{
	WINDOWPLACEMENT wp;
	if (GetWindowPlacement(&wp))	{
		if (IsZoomed())
			wp.flags = WPF_RESTORETOMAXIMIZED;
		CString strBuffer;
		strBuffer.Format("%i:%i:%i:%i:%i:%i:%i:%i:%i:%i",
			wp.flags, wp.showCmd,
			wp.ptMinPosition.x, wp.ptMinPosition.y,
			wp.ptMaxPosition.x, wp.ptMaxPosition.y,
			wp.rcNormalPosition.left, wp.rcNormalPosition.top,
			wp.rcNormalPosition.right, wp.rcNormalPosition.bottom);
		theApp.WriteProfileString("Settings", "WindowPos", strBuffer);
	}
}


void CMainFrame::SetRowColumn(int row, int column)
{
	CString str;
	if (row != 0)
		str.Format("Ln %d, Col %d", row, column);
	m_wndStatusBar.SetPaneText(1, str.GetBuffer(0));
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnClose()
{
	if (theApp.StopDebugging())	{
		theApp.SaveOpenDocuments();
		SaveBarState("Settings");
		SaveWindowPlacement();
		::WinHelp(m_hWnd, theApp.m_pszHelpFilePath, HELP_QUIT, 0);
		CMDIFrameWnd::OnClose ();
	}
}

void CMainFrame::ShowStatus(CString& strNewStatus, CString* pstrStatus)
{
	if (pstrStatus != 0)
		m_wndStatusBar.GetPaneText(1, *pstrStatus);
	m_wndStatusBar.SetPaneText(1, strNewStatus);
	m_wndStatusBar.SendMessage(WM_PAINT,0,0);
}


void CMainFrame::OnWindowCloseAll() 
{
	theApp.SaveAllModified();
	theApp.CloseAllDocuments(FALSE);
}

void CMainFrame::OnFileSaveAll() 
{
	theApp.SaveAllDocuments();
}

void CMainFrame::OnUpdateFileSaveAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

LRESULT CMainFrame::DefWindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if (message == ID_ADDTRACE)	{
		ASSERT(theApp.m_pdlgTrace != 0);
		CListBox* pList = static_cast<CListBox*>(theApp.m_pdlgTrace->GetDlgItem(IDC_TRACELIST));
		ASSERT(pList != 0);
		char trace[201];
		ASSERT(theApp.GetDebugger() != 0);
		ReadProcessMemory(theApp.GetDebugger()->ProcessHandle(), (void*) lParam, trace, 200, 0);
		trace[200] = '\0';
		pList->AddString(trace);

		CDC* pDC = pList->GetDC();
		ASSERT(pDC != 0);
		CSize size = pDC->GetTextExtent(trace);
		m_nTraceWidth = max(m_nTraceWidth, size.cx);
		pList->SendMessage(LB_SETHORIZONTALEXTENT, m_nTraceWidth, 0);
		return 0;
	}
	return CMDIFrameWnd::DefWindowProc(message, wParam, lParam);
}

void CMainFrame::OnUpdateUsestdlib(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(theApp.IsStdLibInstalled());
}

BOOL CMainFrame::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	// TODO: Add your specialized code here and/or call the base class

	return CMDIFrameWnd::OnCommand(wParam, lParam);
}


