// Quincy.h : main header file for the QUINCY application
//

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#ifndef QUINCY_H
#define QUINCY_H

#include <set>
#include "resource.h"       // main symbols
#include "breakpoint.h"

/////////////////////////////////////////////////////////////////////////////
// CQuincyApp:
// See Quincy.cpp for the implementation of this class
//

class CQuincyDoc;
class CQuincyView;
class CErrorLogDialog;
class CTextDocument;
class CTextView;
class CWatchDialog;
class CTraceDialog;
class Parser;
class Debugger;
class CTutDlg;
class GrepResult;
class GrepInput;

// ---- maximum number of errors reported 
const int maxerrors = 50;
const int maxgreps  = 100;

struct el {						// error/warning log
	UINT m_nLineNumber;
	CString m_strErrorMsg;
	CString m_strFileSpec;
};

class CQuincyApp : public CWinApp
{
//	friend class Compiler;
	BOOL m_bGrepInputCreated;	// true if grep input dialog has been created
	BOOL m_bGrepResultCreated;	// true if grep result dialog has been created
	BOOL m_bWatchCreated;		// true if watch window has been created
	BOOL m_bTraceCreated;		// true if trace window has been created
	BOOL m_bCpp;				// true if compiling .cpp
	BOOL m_bDoLink;				// true if linking
	BOOL m_bBypassChangeTest;	// true when user says run in spite of changes
	BOOL m_bClosing;			// true when shutting down and debugger is still running
	BOOL m_bShowErrors;			// true to show compile errors in Notepad

#ifdef TYCPP
	BOOL m_bReadTutorial;		// true when a tutorial is to be loaded
	BOOL m_bTutCreated;			// true if tutorial dialog has been created
	BOOL m_bIsTutorial;			// true if tutorial table of contents is present
	BOOL m_bTutorialDisplayed;  // true when tutorial is displayed
#endif

	BOOL m_bGrepIsInstalled;	// true when grep tool is installed
	BOOL m_bWEditResIsInstalled;// true when resource editor is installed
	BOOL m_bCompilerIsInstalled;// true when the gnu compiler is installed
	BOOL m_bStdLibIsInstalled;	// true when experimental Standard C++ library is installed
	BOOL m_bUseStdLib;			// true to use experimental Standard C++ library
	bool m_bUseBCC;				// true to use BCC 5.5, false to use mingw32
	bool m_bTDisInstalled;		// true if Turbo Debugger is installed

public:
	HANDLE m_hStdin;
	HANDLE m_hStdout;
	HANDLE m_hStderr;
private:

	class Compiler *m_pCompiler;

	CStringArray m_Defines;		// #define expressions
	CStringArray m_Includes;	// #include paths
	CStringArray m_Libs;		// library paths

	CString m_strVersion;		// gnu-mingw32 compiler version
	CString m_strQuincyBinPath;	// quincy.exe path
	CString m_strLCCPath;		// resource compiler and editor path
	CString m_strRIncludePath;	// resource compiler include path
	CString m_strIncludePath;	// compiler include path
public:
	CTraceDialog* m_pdlgTrace;
	CString m_strOldStatus;		// status line text before compiling
private:
	CString m_strLibPath[4];	// paths to standard C/C++ libraries
	CString m_strToolsPath;		// path to assembler, librarian, linker
	CString m_strStartupCode;	// path to compiler's startup code (crt0.o)

	CString m_strStdLibPath;	// path to experimental Standard C++ library
	CString m_strStdIncludePath;// path to experimental Standard C++ header files
	CString m_strCmdLineOptions;// additional user-specified gcc command-line options

#ifdef TYCPP
	CString m_strTutorialPath;	// path to tutorials
	CString m_strTutorial;		// name of current tutorial file
#endif
	CString m_strTempPath;		// path pointed to by TEMP environment variable
	CString m_strWorkingPath;	// working path for programs and the compiler

	CView* m_pView;				// -> view of document being compiled
	DWORD m_nExitCode;			// exit code from compiler programs

	// ----- options
	BOOL m_bDebugging;				// option for adding debug information
	BOOL m_bExceptions;				// option for enabling C++ exception handling 
	BOOL m_bRTTI;					// option for enabling C++ runtime type information
	BOOL m_bStrict;					// option for enabling C/C++ strict ANSI/ISO compliance
	bool m_bPrintLineNos;			// true for printing line numbers
	int m_CommandLinePromptOption;	// option for command line prompt
	int m_nOptimize;				// compile optimize option
	int m_tabstops;					// editor tabstops
	BOOL m_bAutoindent;				// TRUE for autoindent feature in editor
	BOOL m_bSyntaxColors;			// TRUE for syntax colors
	COLORREF m_backgroundcolor;		// syntax color values
	COLORREF m_normalcolor;
	COLORREF m_keywordcolor;
	COLORREF m_commentcolor;
	COLORREF m_stringcolor;
	CString m_strCommandLine;		// default command line
	CString m_strRuntimeDirectory;	// runtime directory
	CString m_strCompiler;			// where the compiler is installed

	UINT m_nGrepCount;				// # entries in grep log
	CWatchDialog* m_pdlgWatch;
	GrepInput* m_pdlgGrepInput;		// grep input dialog
	GrepResult* m_pdlgGrepResult;	// log of grep results
	// ------ examine and watch expression parser
	Parser* m_pParser;
	// ------ source level debugger
	Debugger* m_pDebugger;

	// --- programmed breakpoints 
	std::set<FileReference> breakpoints;

	CRect m_ConsoleRect;			// screen position of console window

#ifdef TYCPP
	CTutDlg* m_pCTutDlg;
#endif
	struct el GrepLog[maxgreps];

	OSVERSIONINFO osversion;

	// ------------------ methods

	void SetCompilerPaths();

	void LoadArray(CStringArray& array, const CString& rstr);
	void GetArray(CString& str, const CStringArray& array) const;

	BOOL RunGrep(const CString& strCmd);
	void RetabAllTextDocuments(int newtabstops);

public:
	CQuincyApp();
	~CQuincyApp();

	const OSVERSIONINFO& OSVersion() const
		{ return osversion; }

	// ----- IDE methods
	const int Tabstops() const
		{ return m_tabstops; }
	const BOOL AutoIndentOption() const
		{ return m_bAutoindent; }
	const BOOL SyntaxColorsOption() const
		{ return m_bSyntaxColors; }
	const GetMargin() const
		{ return 2; }		// set an option later? for "off" maybe a la devstudio
	CQuincyDoc* GetProjectDocument(CString* pstrPath = 0);
	CQuincyView* GetProjectView();
	CTextDocument* GetTextDocument(CString pstrPath);
	CTextView* GetTextView(CString pstrPath);
	HANDLE BuildStdout(const CString& rstrPath);
	HANDLE BuildStdin(const CString& rstrPath);
	HANDLE BuildStderr(const CString& rstrPath);
	HANDLE BuildStderr();
	CString GetFileName(const CString& rstrPath);
	CString GetFilePath(const CString& rstrPath);
	void SaveOpenDocuments();
	void SaveAllDocuments();
	void ShowStatusText(CString& strText, CString* pstrOldText = 0);
	void SaveDialogWindowPosition(const CString& strWindow, CWnd* pWnd);
	void RestoreDialogWindowPosition(const CString& strWindow, CWnd* pWnd);
	CWnd* MaximizeDocument(const CString& strDocument);
	void SetBypassChangeTest();
	void BringToTop();
	void SerializeOptions(CArchive& ar, CStringArray& array);
	void SerializeProjectOptions(CArchive& ar, int nSig);
	void InvalidateAllViews();
	void RunResourceEditor(const CString& strItem = CString());
	void CloseConsoleHandle(HANDLE& hStd, DWORD stdhandle);
	void CloseConsoleHandles();
	bool IsOnCDROM(const CString& strFileSpec);
	bool PrintLineNumbers() const
		{ return m_bPrintLineNos; }
	void SetPrintLineNumbers(bool bset)
		{ m_bPrintLineNos = bset; }
	const CStringArray& GetIncludeArray() const
		{ return m_Includes; }
	const CStringArray& GetDefinesArray() const
		{ return m_Defines; }
	const CStringArray& GetLibraryArray() const
		{ return m_Libs; }

//	bool ToggleMenuCommand(int id);
	void SetMenuCommand(int id);
	void ResetMenuCommand(int id);

	COLORREF BackgroundColor() const
		{ return m_bSyntaxColors ? m_backgroundcolor : RGB(255,255,255); }
	COLORREF NormalColor() const
		{ return m_bSyntaxColors ? m_normalcolor : RGB(0,0,0); }
	COLORREF KeywordColor() const
		{ return m_keywordcolor; }
	COLORREF CommentColor() const
		{ return m_commentcolor; }
	COLORREF StringColor() const
		{ return m_stringcolor; }
	BOOL StrictANSI() const
		{ return m_bStrict; }

#ifdef TYCPP
	// ----- tutorial methods
	void ReadTutorial();
	void WriteTutorial(const CString& strTutorial);
	BOOL IsTutorial(const CString& strFileName);
	void DispatchTutorialCommand(const CString& strCommand);
	void LoadTutorials();
	void CreateTutorialDialog();
	void OpenTutorial(const CString& strTut);
	void SetTutorialDisplay(BOOL bOnOff)
		{  m_bTutorialDisplayed = bOnOff; }

#endif

	// ----- compiler methods
	Compiler& GetCompiler()
	{ ASSERT(m_pCompiler != 0); return *m_pCompiler; }
	int CompareFileTimes(CString& strFile1, CString& strFile2);
	bool SelectFileAndLine(const CString& strFile, int nline);
	void SelectErrorLine(int nsel);
	void SelectGrepLine(int nsel);
	CString WorkingFile(const CString& strFile);
	BOOL CanCompile() const
		{ return m_bCompilerIsInstalled; }
	BOOL IsStdLibInstalled() const
		{ return m_bStdLibIsInstalled; }
	BOOL IsLibraryTarget();
	BOOL IsGUITarget();
	BOOL IsDLLTarget();
	BOOL IsConsoleTarget();
	BOOL IsProjectFileLoaded();
	void SetBuildingCPP(BOOL bSet);
	void ShowErrors()
		{ m_bShowErrors = TRUE; }
	bool UseBCC() const
		{ return m_bUseBCC; }
	bool CompileRunning() const;
	const CString& RIncludePath() const
		{ return m_strRIncludePath; }
	const CString& ToolsPath() const
		{ return m_strToolsPath; }
	const CString& LCCPath() const
		{ return m_strLCCPath; }
	BOOL IsAddingDebugInfo() const
		{ return m_bDebugging; }
	int OptimizeLevel() const
		{ return m_nOptimize; }
	BOOL IncludeRTTI() const
		{ return m_bRTTI; }
	BOOL IncludeExceptions() const
		{ return m_bExceptions; }
	const CString& IncludePath() const
		{ return m_strIncludePath; }
	const CString& CmdLineOptions() const
		{ return m_strCmdLineOptions; }
	BOOL UseStdLib() const
		{ return m_bUseStdLib; }
	const CString& StdIncludePath() const
		{ return m_strStdIncludePath; }
	const CString* LibPath() const
		{ return m_strLibPath; }
	const CString& StartupCode() const
		{ return m_strStartupCode; }
	const CString& QuincyBinPath() const
		{ return m_strQuincyBinPath; }
	const CString& StdLibPath() const
		{ return m_strStdLibPath; }

	// ----- executor methods
	bool TDisInstalled() const
		{ return m_bTDisInstalled; }
	void RunApplicationProgram(CString& strCmd, BOOL bStepping = FALSE, BOOL useTD = FALSE);
	void GetCommandLine(CString& strCmdLine);
	CString GetProjectRuntimeDirectory();
	const CString& GetRuntimeDirectory() const
		{ return m_strRuntimeDirectory; }
	const CString& GetTemporaryDirectory() const
		{ return m_strTempPath; }
	void ParseFileSpec(CString& strFileSpec, CString strCmdLine, int nIndex);
	// ------ editor methods
	void ShowLineColumn(int nLine, int nColumn);

	// ------- debugger methods
	BOOL TestProgramChanged();
	BOOL StepProgram();
	BOOL StepOut();
	void StopStepping();
	BOOL StopDebugging();
	BOOL ExecuteProgram();
	Debugger* GetDebugger(BOOL bVerify = TRUE) const;
	const CString& WorkingPath() const;
	void StopDebugger();
	bool ToggleBreakpoint(const CString& strFile, int nLineNo);
	void AddBreakpoint(const CString& strFile, int nLineNo);
	int BreakpointCount() const
		{ return breakpoints.size(); }
	void AdjustTextLine(const CString& strFile, int nLineno, int nAdjust);
	void DeletedTextLine(const CString& strFile, int nLineno)
		{ AdjustTextLine(strFile, nLineno, -1); }
	void InsertedTextLine(const CString& strFile, int nLineno)
		{ AdjustTextLine(strFile, nLineno, 1); }
	bool IsBreakpoint(const CString& strFile, int nLineno);
	std::set<FileReference>& GetProgrammedBreakpoints()
		{ return breakpoints; }
	void ClearBreakpoints()
		{ breakpoints.clear(); }

	// ------ watch and examine methods
	void CreateWatchWindow();
	void RefreshAllWatches();
	void GetVariableValue(const CString& strVarName, CString& strVarValue);
	void SetVariableValue(const CString& strVarName, const CString& strVarValue);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CQuincyApp)
	public:
	virtual BOOL InitInstance();
	virtual CDocument* OpenDocumentFile(LPCTSTR lpszFileName);
	virtual BOOL OnIdle(LONG lCount);
	virtual int ExitInstance();
	//}}AFX_VIRTUAL
#ifdef TYCPP
#ifdef CRAMCOURSE
	virtual BOOL OnDDECommand(LPTSTR lpszCommand);
#endif
#endif

// Implementation

	//{{AFX_MSG(CQuincyApp)
	afx_msg void OnAppAbout();
	afx_msg void OnStop();
	afx_msg void OnUpdateStop(CCmdUI* pCmdUI);
	afx_msg void OnOptions();
	afx_msg void OnShowErrorlog();
	afx_msg void OnUpdateShowErrorlog(CCmdUI* pCmdUI);
	afx_msg void OnUpdateDebugClearbreakpoints(CCmdUI* pCmdUI);
	afx_msg void OnDebugClearbreakpoints();
	afx_msg void OnDebugStepover();
	afx_msg void OnUpdateDebugExamine(CCmdUI* pCmdUI);
	afx_msg void OnDebugExamine();
	afx_msg void OnDebugWatch();
	afx_msg void OnUpdateDebugWatch(CCmdUI* pCmdUI);
	afx_msg void OnUpdateDebugStepover(CCmdUI* pCmdUI);
	afx_msg void OnHelp();
	afx_msg void OnResourceeditor();
	afx_msg void OnGrep();
	afx_msg void OnUpdateGrep(CCmdUI* pCmdUI);
	afx_msg void OnUpdateResourceeditor(CCmdUI* pCmdUI);
	afx_msg void OnFileOpen();
	afx_msg void OnDebugTrace();
	afx_msg void OnUpdateUsestdlib(CCmdUI* pCmdUI);
	afx_msg void OnFilePrintSetup();
	//}}AFX_MSG
#ifdef TYCPP
	afx_msg void OnTycpp();
	afx_msg void OnUpdateTycpp(CCmdUI* pCmdUI);
	afx_msg void OnWritetutorial();
#endif
	DECLARE_MESSAGE_MAP()
};

extern CQuincyApp theApp;

inline void CQuincyApp::SetBypassChangeTest()
{
	m_bBypassChangeTest = TRUE;
}
inline Debugger* CQuincyApp::GetDebugger(BOOL bVerify) const
{
	if (bVerify)
		ASSERT(m_pDebugger);
	return m_pDebugger;
}
inline const CString& CQuincyApp::WorkingPath() const
{
	return m_strWorkingPath;
}
/////////////////////////////////////////////////////////////////////////////

#endif // QUINCY_H
