// TextView.cpp : implementation file
//

#include "stdafx.h"
#include "Quincy.h"
#include "debugger.h"
#include "TextView.h"
#include "QuincyView.h"
#include "TextDocument.h"
#include "Compiler.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTextView

static const int margin = 1;				// margin for breakpoint and program counter carets
static const char breakpointchar = '>';		// breakpoint margin displaycharacter

IMPLEMENT_DYNCREATE(CTextView, CEditorView)

CTextView::CTextView()
{
}

CTextView::~CTextView()
{
}

BEGIN_MESSAGE_MAP(CTextView, CEditorView)
	//{{AFX_MSG_MAP(CTextView)
	ON_COMMAND(ID_BUILD, OnBuild)
	ON_COMMAND(ID_BUILDALL, OnBuildall)
	ON_COMMAND(ID_COMPILE, OnCompile)
	ON_COMMAND(ID_EXECUTE, OnExecute)
	ON_COMMAND(ID_DEBUG_BREAKPOINT, OnDebugBreakpoint)
	ON_COMMAND(ID_DEBUG_STEP, OnDebugStep)
	ON_COMMAND(ID_DEBUG_STEPTOCURSOR, OnDebugSteptocursor)
	ON_UPDATE_COMMAND_UI(ID_BUILD, OnUpdateBuild)
	ON_UPDATE_COMMAND_UI(ID_BUILDALL, OnUpdateBuildAll)
	ON_UPDATE_COMMAND_UI(ID_COMPILE, OnUpdateCompile)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_ALL, OnUpdateFileSaveAll)
	ON_COMMAND(ID_DEBUG_STEPOUTOFFUNCTION, OnDebugStepoutoffunction)
	ON_COMMAND(ID_TURBODEBUGGER, OnTurbodebugger)
	ON_UPDATE_COMMAND_UI(ID_TURBODEBUGGER, OnUpdateTurbodebugger)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_CHAR()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTextView diagnostics

#ifdef _DEBUG
void CTextView::AssertValid() const
{
	CEditorView::AssertValid();
}

void CTextView::Dump(CDumpContext& dc) const
{
	CEditorView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTextView message handlers


void CTextView::OnBuildall() 
{
	// ----- if a project document is open, send build command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->SendMessage(WM_COMMAND, ID_BUILDALL, 0);
}

void CTextView::OnBuild() 
{
	// ----- if a project document is open, send build command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->SendMessage(WM_COMMAND, ID_BUILD, 0);
	else
		DoCompile(TRUE, FALSE);
}

BOOL CTextView::DoCompile(BOOL bDoLink, BOOL bDoExecute, BOOL bDoStep, BOOL bUseTD) 
{
	theApp.StopStepping();

	CDocument* pDoc = GetDocument();
	ASSERT(pDoc != 0);

	CString& rstrPath = const_cast<CString&>(pDoc->GetPathName());
	if (pDoc->IsModified())
		pDoc->OnSaveDocument(rstrPath);

	// ----- make a copy of the file spec without the extension;
	BOOL isCpp = rstrPath.Right(4).CompareNoCase(".cpp") == 0;
	CString strSpec = rstrPath.Left(rstrPath.GetLength()-(isCpp ? 4 : 2));

	CString strExeSpec = strSpec + ".exe";

	// ---- queue the source and object files for compile/link
	theApp.GetCompiler().ClearArrays();
	theApp.GetCompiler().AddSourceFile(rstrPath);
	if (bDoLink)
		theApp.GetCompiler().AddObjectFile(theApp.GetCompiler().MakeObjectFileName(strSpec));
	theApp.GetCompiler().BuildTarget(strExeSpec, 
		(bDoExecute ? (bDoStep ? Compiler::step : (bUseTD? Compiler::turbodebugger : Compiler::execute)) : Compiler::none));

	return TRUE;
}

void CTextView::OnCompile()
{
	DoCompile(FALSE, FALSE);
}

void CTextView::OnDebugStep() 
{
	// ----- if a project document is open, send step command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->PostMessage(WM_COMMAND, ID_DEBUG_STEP, 0);
	else if (!theApp.StepProgram())
		BuildAndRunProgram(TRUE);
}

void CTextView::OnDebugStepoutoffunction() 
{
	theApp.StepOut();
}

void CTextView::OnExecute() 
{
	// ----- if a project document is open, send execute command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->PostMessage(WM_COMMAND, ID_EXECUTE, 0);
	else if (!theApp.ExecuteProgram())
		BuildAndRunProgram(FALSE);
}

void CTextView::OnTurbodebugger() 
{
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->PostMessage(WM_COMMAND, ID_TURBODEBUGGER, 0);
	else 
		BuildAndRunProgram(FALSE, TRUE);
}

void CTextView::BuildAndRunProgram(BOOL bStep, BOOL useTD)
{
	CDocument* pDoc = GetDocument();
	ASSERT(pDoc != 0);
	CString& rstrPath = const_cast<CString&>(pDoc->GetPathName());
	// ----- make a copy of the file spec with the .EXE extension;
	BOOL isCpp = rstrPath.Right(4).CompareNoCase(".cpp") == 0;
	CString strExe = rstrPath.Left(rstrPath.GetLength()-(isCpp ? 4 : 2)) + ".exe";

	// ---- execute the .exe associated with the .c or .cpp source code document
	int rtn;
	if (pDoc->IsModified())	{
		CString strMsg = theApp.GetFileName(rstrPath) + " has been modified. Build?";
		if ((rtn = AfxMessageBox(strMsg, MB_YESNOCANCEL | MB_ICONQUESTION)) == IDYES)
			DoCompile(TRUE, TRUE, bStep, useTD);
		else if (rtn == IDNO && _access(strExe, 0) == 0)	{
			theApp.SetBypassChangeTest();
			theApp.RunApplicationProgram(strExe, bStep, useTD);
		}
	}
	else if (_access(strExe, 0) != 0)	{
		CString strMsg = theApp.GetFileName(strExe) + " does not exist. Build?";
		if (AfxMessageBox(strMsg, MB_YESNO | MB_ICONQUESTION) == IDYES)
			DoCompile(TRUE, TRUE, bStep, useTD);
	}
	else if (theApp.CompareFileTimes(rstrPath, strExe) > 0)	{
		CString strMsg = theApp.GetFileName(strExe) + " out of date. Rebuild?";
		if ((rtn = AfxMessageBox(strMsg, MB_YESNOCANCEL | MB_ICONQUESTION)) == IDYES)
			DoCompile(TRUE, TRUE, bStep, useTD);
		else if (rtn == IDNO)
			theApp.RunApplicationProgram(strExe, bStep, useTD);
	}
	else
		theApp.RunApplicationProgram(strExe, bStep, useTD);
}

// ----- return the document file specification
CString CTextView::SourceCodeFileName()
{
	CTextDocument* pDoc = (CTextDocument*)GetDocument();
	ASSERT(pDoc != 0);
	return pDoc->GetPathName();
}
// ----- called to set or clear a breakpoint in this source code file
void CTextView::OnDebugBreakpoint() 
{
	// ---- get the name of the source code file
	CString strFile = GetDocument()->GetPathName();
	// ----- get the current line number from the source code file
	int nLineno = CurrentLineNumber();		// current line number
	bool bSet = theApp.ToggleBreakpoint(strFile, nLineno);

	if (bSet)	{
		Debugger* pDebugger = theApp.GetDebugger(false);
		if (pDebugger != 0)
			pDebugger->DoFixBreakpoints();
	}
	DrawMargins();
}
// --- write a 16 X 15 bitmap into the editor's left margin
//     nLine is the window's vertical pixel position
void CTextView::WriteMarginBitmap(int nRow, CBitmap& bm)
{
	CDC* pCDC = GetDC();

	CDC dcMem;
	dcMem.CreateCompatibleDC(pCDC);
	CBitmap* pOld = static_cast<CBitmap*>(dcMem.SelectObject(&bm));
	dcMem.SetMapMode(pCDC->GetMapMode());
	pCDC->BitBlt(0, nRow, 16, 15, &dcMem, 0, 0, SRCCOPY);
	dcMem.SelectObject(pOld);
}

// ---- compute the window's zero-based client area pixel position number
//      from the one-based text line number
inline int CTextView::RelativeRow(int nLineno)
{
	ASSERT(nLineno > 0);
	int lno = nLineno - TopLine();	// zero-based line number
	int top = TopRow() % m_fontheight;
	int y = lno * m_fontheight - top;
	return y;
}

void CTextView::OnDraw(CDC* pDC) 
{
	CEditorView::OnDraw(pDC);
	DrawMargins();
}

void CTextView::DrawMargins()
{
	PadMargin();
	// ---- get the name of the source code file
	CString strFile = SourceCodeFileName();
	// ----- compute top and bottom lines of the client window
	int nLine = TopLine(); // one-based line number
	RECT rc;
	GetClientRect(&rc);
	int nBottomLine = nLine + rc.bottom / m_fontheight; // one-based line number

	// --- set flag if this file is the current debugged source code file
	bool bSrc = false;
	Debugger* pDebugger = theApp.GetDebugger(false);
	if (pDebugger != 0)	{
		const CString* pCurrentSrcFile = pDebugger->CurrentSrcFile();
		if (pCurrentSrcFile != 0)
			bSrc = strFile.CompareNoCase(*pCurrentSrcFile) == 0;
	}
	// ---- iterate the lines, displaying 
	//      breakpoint margin carets and the program counter cursor bar
	do	{
		bool bPC;
		if (bPC = (bSrc && nLine == theApp.GetDebugger()->CurrentLineNo()) != false)
			SetProgramCounterDisplay(nLine, TRUE);
		bool bBrk = theApp.IsBreakpoint(strFile, nLine);
		if (bBrk)	{
			if (bPC)
				SetPCBreakpointDisplay(nLine);
			else
				SetBreakpointDisplay(nLine, TRUE);
		}
	} while (nLine++ < nBottomLine);
}

// ---- set or clear a breakpoint bitmap in the margin
// ------ nLineno is the one-based file line number
void CTextView::SetBreakpointDisplay(int nLineno, BOOL bOnOff)
{
	int nRelativeRow = RelativeRow(nLineno);
	CBitmap bm;
	bm.LoadBitmap(bOnOff ? IDB_BITMAP4 : IDB_BITMAP3);
	WriteMarginBitmap(nRelativeRow, bm);
	bm.DeleteObject();
}
// ---- set a breakpoint and pc bitmap in the margin
// ------ nLineno is the one-based file line number
void CTextView::SetPCBreakpointDisplay(int nLineno)
{
	int nRelativeRow = RelativeRow(nLineno);
	CBitmap bm;
	bm.LoadBitmap(IDB_BITMAP5);
	WriteMarginBitmap(nRelativeRow, bm);
	bm.DeleteObject();
}
// ---- set or clear the program counter cursor in the margin
// ------ nLineno is the zero-based file line number
void CTextView::SetProgramCounterDisplay(int nLineno, BOOL bOnOff)
{
	int nRelativeRow = RelativeRow(nLineno);
	CBitmap bm;
	bm.LoadBitmap(bOnOff ? IDB_BITMAP2 : IDB_BITMAP3);
	WriteMarginBitmap(nRelativeRow, bm);
	bm.DeleteObject();
}

void CTextView::OnDebugSteptocursor() 
{
	if (!theApp.TestProgramChanged())	{
		int nLineNo = CurrentLineNumber();
		CString strFile = SourceCodeFileName();
		Debugger::StepTo(strFile, nLineNo);
		OnExecute();
	}
}

void CTextView::OnUpdateBuild(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanBuild() && (theApp.IsProjectFileLoaded() || IsSourceFile()));
}

void CTextView::OnUpdateBuildAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanBuild() && theApp.IsProjectFileLoaded());
}

void CTextView::OnUpdateCompile(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanBuild() && IsSourceFile());
}

BOOL CTextView::CanBuild()
{
	Debugger* pDebugger = theApp.GetDebugger(FALSE);
	return !theApp.CompileRunning() && pDebugger == 0 && theApp.CanCompile();
}
BOOL CTextView::IsSourceFile()
{
	CTextDocument* pDoc = (CTextDocument*)GetDocument();
	ASSERT(pDoc != 0);
	return pDoc->IsSourceFile();
}

void CTextView::OnUpdateFileSaveAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}

void CTextView::OnUpdateTurbodebugger(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(theApp.UseBCC() && theApp.TDisInstalled());
}


