// ---------- build.cpp
#include "stdafx.h"
#include "Quincy.h"
#include "QuincyView.h"
#include "QuincyDoc.h"
#include "TextDocument.h"

// ----- project Build command
void CQuincyView::BuildProject(BOOL bDoExecute, BOOL bDoStep)
{
	CDocument* pDoc = GetDocument();
	ASSERT(pDoc != 0);
	const CString& strPath = pDoc->GetPathName();
	if (!strPath.IsEmpty())
		pDoc->OnSaveDocument(strPath);
	theApp.BuildTarget(m_strTargetFile,
		bDoExecute ? (bDoStep ? CQuincyApp::step : CQuincyApp::execute) : CQuincyApp::none);
}

BOOL CQuincyView::TargetIsCurrent(BOOL bForceRebuild)
{
	CListCtrl& rlist = GetListCtrl();
	int count = rlist.GetItemCount();
	if (count == 0)
		return TRUE;

	BOOL bIsCurrent = !bForceRebuild;

	theApp.SetBuildingCPP(FALSE);

	theApp.ClearArrays();

	CString strOldStatus;
	theApp.ShowStatusText(CString("Checking dependencies"), &strOldStatus);
	theApp.ClearErrorLog();

	// ---- iterate through the source files in the project
	for (int nItem = 0; nItem < count; nItem++)	{
		CString strSrcFile = rlist.GetItemText(nItem, 0);

		BOOL bCpp = strSrcFile.Right(4).CompareNoCase(".cpp") == 0;

		if (bCpp)
			theApp.SetBuildingCPP(TRUE);

		BOOL bLib = strSrcFile.Right(2).CompareNoCase(".a") == 0;
		BOOL bRc  = strSrcFile.Right(3).CompareNoCase(".rc") == 0;
		BOOL bRes = strSrcFile.Right(4).CompareNoCase(".res") == 0;
		BOOL bDef = strSrcFile.Right(4).CompareNoCase(".def") == 0;

		if (!bLib && !bRes)
			// ---- if the source file is in memory and modified, save it
			UpdateSourceFile(strSrcFile);

		if (bLib)	{
			// --- object library, add to compiler's list of files to include in link
			theApp.AddLibraryFile(strSrcFile);

			if (!bForceRebuild)	{
				CString strLibPathFile;
				const CStringArray& rstrLibs = theApp.GetLibraryArray();
				int nSize = rstrLibs.GetSize();
				int i;
				BOOL isLib = strSrcFile.Left(3).CompareNoCase("lib") == 0;
				for (i = 0; i < nSize; i++)	{
					if (isLib)
						strLibPathFile = rstrLibs[i] + "\\" + strSrcFile;
					else
						strLibPathFile = strSrcFile;
					if (_access(strLibPathFile, 0) == 0)
						break;
				}
				// ---- test to see if the library file exists and is newer than the executable
				if (i < nSize && theApp.CompareFileTimes(m_strTargetFile, strLibPathFile) < 0)
					bIsCurrent = FALSE;
			}
		}
		else if (bDef)	{
			theApp.AddSourceFile(strSrcFile);
			if (theApp.CompareFileTimes(m_strTargetFile, strSrcFile) < 0)
				bIsCurrent = FALSE;
			else	{
				CString strImportLib = theApp.GetFileName(theApp.m_strTargetName.Left(theApp.m_strTargetName.GetLength() - 3) + "a");
				if (theApp.CompareFileTimes(strImportLib, strSrcFile) < 0)
					bIsCurrent = FALSE;
			}
		}
		else	{
			// --- c, cpp, res, or rc source file
			// ---- make a copy of the source file name without the path or file extension
			CString strSrcName = theApp.GetFileName(strSrcFile.Left(strSrcFile.GetLength()-((bCpp | bRes) ? 4 : (bRc ? 3 : 2))));
			// ---- make a copy of the source file's object file specification
			CString strObjFile = strSrcName + ".o";
			// ---- add the object file to the compiler's link list
			theApp.AddObjectFile(strObjFile);
			// ---- test for rebuild all or if the source file is newer than the object
			if (bForceRebuild || theApp.CompareFileTimes(strObjFile, strSrcFile) < 0 ||
					// ----- see if any #included files are newer than the source file
					(!bRes && IncludesAreNewer(strSrcFile, strObjFile)))	{
				// ---- queue the source file to be compiled
				theApp.AddSourceFile(strSrcFile);
				bIsCurrent = FALSE;
				m_IncludesChecked.RemoveAll();
			}
			// ---- test to see if the object file is newer than the executable
			else if (theApp.CompareFileTimes(m_strTargetFile, strObjFile) < 0)
				bIsCurrent = FALSE;
		}
	}
	if (bIsCurrent)
		theApp.ClearArrays();
	theApp.ShowStatusText(strOldStatus);
	return bIsCurrent;
}

// ---- if the named source file is in memory and modified, save it
void CQuincyView::UpdateSourceFile(CString& strSrcFile)
{
	CTextDocument* pDoc = theApp.GetTextDocument(strSrcFile);
	if (pDoc != 0)	{
		POSITION dpos = pDoc->GetFirstViewPosition();
		ASSERT(dpos != 0);
		CEditView* pView = static_cast<CEditView*>(pDoc->GetNextView(dpos));
		ASSERT(pView != 0);
		CEdit& rEdit = pView->GetEditCtrl();
		if (rEdit.GetModify())
			pDoc->OnSaveDocument(strSrcFile);
	}
}

BOOL CQuincyView::IncludesAreNewer(CString& strSrcFile, CString& strObjFile)
{
	TRY	{
		CStdioFile ifile(strSrcFile, CFile::modeRead);
		char buf[81];
		while (ifile.ReadString(buf, 80) != 0)	{
			// ---- parse the line for an #include statement with "" delimiters
			char *cp = buf;
			while (isspace(*cp))
				cp++;
			if (*cp++ != '#')
				continue;
			while (isspace(*cp))
				cp++;
			if (strncmp(cp, "include", 7) != 0)
				continue;
			cp += 7;
			while (isspace(*cp))
				cp++;
			if (*cp != '"')
				continue;

			cp++;

			// ---- extract the filename of the included file
			while (isspace(*cp))
				cp++;
			char ifilename[81];
			char *cp2 = ifilename;
			while (*cp != '"' && !isspace(*cp) && cp2 < ifilename + 80)
				*cp2++ = *cp++;
			*cp2 = '\0';
			CString strInclFile(ifilename);

			// ---- if the included source file is in memory and modified, save it
			UpdateSourceFile(strInclFile);

			const CStringArray& rstrIncludes = theApp.GetIncludeArray();
			int nSize = rstrIncludes.GetSize();
			int i;
			if (nSize && (_access(strInclFile, 0) != 0))	{
				for (i = 0; i < nSize; i++)	{
					strInclFile = rstrIncludes[i] + "\\" + theApp.GetFileName(strInclFile);
					if (_access(strInclFile, 0) == 0)
						break;
				}
				if (i == nSize)
					continue;
			}

			// ---- see if this file has already been checked
			nSize = m_IncludesChecked.GetSize();
			for (i = 0; i < nSize; i++)
				if (strInclFile == m_IncludesChecked[i])
					break;
			if (i < nSize)
				continue;

			// ---- compare the times of the two files
			if (theApp.CompareFileTimes(strInclFile, strObjFile) > 0)
				return TRUE;

			int nEntry = m_IncludesChecked.Add(strInclFile);
			// ---- now check nested includes
			BOOL bNewer = IncludesAreNewer(strInclFile, strObjFile);

			if (bNewer)
				return TRUE;
		}
	}
	CATCH_ALL(e)
	{
		// -- didn't find the .h file, perhaps it is not included because of #ifdef, etc.
	}
	END_CATCH_ALL
	return FALSE;
}

void CQuincyView::MakeTargetName()
{
	CQuincyDoc* pDoc = static_cast<CQuincyDoc*>(GetDocument());
	ASSERT(pDoc != 0);
	m_strTargetFile =  pDoc->TargetName();
	if (m_strTargetFile.Find('\\') == -1 || m_strTargetFile[1] != ':')	{
		// --- incomplete or no path information in the target, prefix with the path of the project
		CString strProjPath = pDoc->GetPathName();
		// strip the file name
		int nIndex = strProjPath.ReverseFind('\\');
		if (nIndex == -1)
			nIndex = strProjPath.ReverseFind(':');
		if (nIndex != -1)
			strProjPath = strProjPath.Left(nIndex + 1);
		// ----- build the target file specification
		m_strTargetFile =  strProjPath + pDoc->TargetName();
	}
//	_chdir(strProjPath);
}

BOOL CQuincyView::CanExecuteProgram(BOOL bDoStep)
{
	MakeTargetName();
	CString msg = theApp.GetFileName(m_strTargetFile);
	BOOL bExeExists = _access(m_strTargetFile, 0) == 0;
	BOOL bIsCurrent = TargetIsCurrent();
	if (bExeExists)	{
		// ---- exe file exists
		if (bIsCurrent)
			return TRUE;
		msg += " is out of date. Rebuild?";
	}
	else
		// ---- exe file does not exist
		msg += " does not exist. Build?";
	int rtn = AfxMessageBox(msg, MB_YESNOCANCEL | MB_ICONQUESTION);
	if (rtn == IDYES)
		BuildProject(TRUE, bDoStep);
	else if (rtn != IDCANCEL && bExeExists)
		return TRUE;
	theApp.StopStepping();
	return FALSE;
}

void CQuincyView::OnExecute() 
{
	if (!theApp.ExecuteProgram())
		if (CanExecuteProgram(FALSE))
			theApp.RunApplicationProgram(m_strTargetFile);
}

void CQuincyView::OnDebugStep() 
{
	if (!theApp.StepProgram())
		if (CanExecuteProgram(TRUE))
			theApp.RunApplicationProgram(m_strTargetFile, TRUE);
}

void CQuincyView::OnBuild()
{
	MakeTargetName();
	if (TargetIsCurrent())	{
		CString msg = theApp.GetFileName(m_strTargetFile);
		msg += " is up to date";
		AfxMessageBox(msg, MB_OK);
	}
	else
		BuildProject(FALSE, FALSE);
}

void CQuincyView::OnBuildAll()
{
	MakeTargetName();
	TargetIsCurrent(TRUE);
	BuildProject(FALSE, FALSE);
}

