// ----- compiler.cpp

#include "stdafx.h"
#include "Quincy.h"
#include "ErrorLogDialog.h"

#define USEGCC


void CQuincyApp::ClearArrays()
{
	m_SourceFiles.RemoveAll();
	m_ObjectFiles.RemoveAll();
	m_LibraryFiles.RemoveAll();
}

void CQuincyApp::AddLibraryFile(const CString& rstrLib)
{
	if (rstrLib.Left(3).CompareNoCase("lib") == 0)	{
		CString strLib("-l");
		strLib += rstrLib.Mid(3, rstrLib.GetLength()-5);
		m_LibraryFiles.Add(strLib);
	}
	else
		m_LibraryFiles.Add(rstrLib);
}

// --- called from CQuincyDoc class's Serialize function
void CQuincyApp::SerializeProjectOptions(CArchive& ar, int nSig)
{

	if (ar.IsStoring())	{
		ar << m_bDebugging;
		ar << m_bExceptions;
		ar << m_bRTTI;
		if (nSig != 9797)
			ar << m_bStrict;
	}
	else	{
		ar >> m_bDebugging;
		ar >> m_bExceptions;
		ar >> m_bRTTI;
		if (nSig != 9797)
			ar >> m_bStrict;
	}
	SerializeOptions(ar, m_Defines);
	SerializeOptions(ar, m_Includes);
	SerializeOptions(ar, m_Libs);
}
// --- called only from above
void CQuincyApp::SerializeOptions(CArchive& ar, CStringArray& array)
{
	if (ar.IsStoring())	{
		int len = array.GetSize();
		ar << len;
		for (int i = 0; i < len; i++)
			ar << array[i];
	}
	else	{
		array.RemoveAll();
		int len;
		ar >> len;
		while (len--)	{
			CString str;
			ar >> str;
			array.Add(str);
		}
	}
}

bool CQuincyApp::IsOnCDROM(const CString& strFileSpec)
{
	if (strFileSpec.GetLength() < 3)
		return false;
	char root[4];
	for (int i = 0; i < 3; i++)
		root[i] = strFileSpec[i];
	root[i] = '\0';
	return GetDriveType(root) == DRIVE_CDROM;
}

void CQuincyApp::BuildTarget(const CString& strTargetName, CompileAction action)
{
	if (IsOnCDROM(strTargetName))	{
		std::string msg;
		msg += const_cast<CString&>(strTargetName).GetBuffer(0);
		msg += " is directed to a read-only medium (e.g. CD-ROM).",
		AfxMessageBox(msg.c_str(), MB_ICONSTOP);
		return;
	}
	m_CompileAction = action;
	m_strTargetName = strTargetName;
	ClearErrorLog();
	ShowStatusText(CString("Building Program"), &m_strOldStatus);
	AfxBeginThread(&CQuincyApp::BldProg, 0);
}

UINT CQuincyApp::BldProg(void*)
{
	theApp.BuildTarget();
	return 0;
}

CString CQuincyApp::WorkingFile(const CString& strFile)
{
	CString strWk;
	if (!m_strWorkingPath.IsEmpty())
		strWk = m_strWorkingPath;
	else if (!m_strTempPath.IsEmpty())
		strWk = m_strTempPath;
	if (strWk[strWk.GetLength()-1] != '\\')
		strWk += "\\";
	strWk += strFile;
	return strWk;
}

// --- put quotes around a file specification that has one or more spaces
//     so the file specification can be used on a command line
CString CQuincyApp::Enquote(const CString& str) const
{
	if (str.Find(' ') == -1)
		return str;
	CString qstr("\"" + str);
	// if the spec ends with backslash, add another backslash 
	if (qstr[qstr.GetLength()-1] == '\\')
		qstr += "\\";
	qstr += "\"";
	return qstr;
}

void CQuincyApp::BuildTarget()
{
	int nSize = m_SourceFiles.GetSize();

	DWORD exitcode = 0;
	DWORD ExitCode = 0;

	CString strDefname;
	CString strSrcfile;
	BOOL bCpp = FALSE;
	BOOL bDef = FALSE;
	BOOL bFoundDef = FALSE;
	int n;
	CString strCmd;

	CString strOFile;

	BuildStdout(WorkingFile("quincy.out"));
	BuildStderr(WorkingFile("quincy.err"));
	// --- compile the programs listed in the source code list
	for (n = 0; n < nSize; n++)	{

		if (m_CompileStatus == aborting)
			break;
		CString strFile = m_SourceFiles[n];
		if (strFile.IsEmpty())
			break;

		strFile.MakeLower();	// gcc 2.95.1  bug

		BOOL bRc  = strFile.Right(3).CompareNoCase(".rc")  == 0;
		BOOL bRes = strFile.Right(4).CompareNoCase(".res") == 0;
		bCpp = strFile.Right(4).CompareNoCase(".cpp") == 0;
		bDef = strFile.Right(4).CompareNoCase(".def") == 0;
		m_bLinkCpp |= bCpp;

		// ---- make a copy of the filename without the path
		m_strFilename = GetFileName(strFile);

		// ---- make a copy of the source file name without the file extension
		strSrcfile = m_strFilename.Left(m_strFilename.GetLength()-((bCpp | bRes | bDef) ? 4 : (bRc ? 3 : 2)));

		// ---- make a copy of the source file path and name without the file extension
		CString strSrcPath = strFile.Left(strFile.GetLength()-((bCpp | bRes | bDef) ? 4 : (bRc ? 3 : 2)));

		// ---- make a copy of the assembler output file name
		strOFile = strSrcPath + ".o";

		if (bRc)	{
			// ------ compiling a resource script

			CString strResFile = strSrcPath + ".res";
			remove(strResFile);

			char path[MAX_PATH];
			_getcwd(path, MAX_PATH);

			strCmd = m_strLCCPath + "lrc.exe";
			strCmd += " /o";
			strCmd += strResFile;
			strCmd += " /i";
			strCmd += path;
			strCmd += " /i";
			strCmd += m_strRIncludePath;
			strCmd += " ";
			strCmd += m_strFilename;
			m_CompileStatus = compiling;
			if (RunCompilerProgram(strCmd, &exitcode))	{
				// ------ finished compiling rc into res
				ExitCode |= exitcode;
				if (m_CompileStatus == aborting)
					break;
				if (exitcode != 0)
					continue;
				bRes = TRUE;
				m_strFilename = strResFile;
			}
		}

		if (bRes)	{
			// --- converting resource .res file into object .o file
			strCmd = m_strToolsPath + "windres.exe";
			strCmd += " -i ";
			strCmd += m_strFilename;
			strCmd += " -o ";
			strCmd += strOFile;
			if (RunCompilerProgram(strCmd, &exitcode, TRUE))	{
				// ------ finished converting .res into .o
				ExitCode |= exitcode;
				if (m_CompileStatus == aborting)
					break;
			}
			continue;
		}

		if (bDef)	{
			if (IsDLLTarget())	{
				// ----- build an import library for a DLL
				CString strImportLib = m_strTargetName.Left(m_strTargetName.GetLength() - 3) + "a";
				strCmd = m_strToolsPath + "dlltool.exe";
				strCmd += " --dllname ";
				strCmd += m_strTargetName;		// name of the dll
				strCmd += " --def ";
				strCmd += m_strFilename;		// name of the def
				strCmd += " --output-lib lib";
				strCmd += strImportLib;			// name of the import lib

				if (RunCompilerProgram(strCmd, &exitcode, TRUE))	{
					ExitCode |= exitcode;
					if (m_CompileStatus == aborting)
						break;
				}
				strDefname = m_strFilename;
			}
			else if (!bFoundDef)	{
				AfxMessageBox("Non-DLL project has .def file", MB_OK | MB_ICONQUESTION);
				ExitCode = 1;
			}
			bFoundDef = TRUE;
			continue;
		}

		remove(strOFile);

#ifdef USEGCC

		strCmd = m_strToolsPath + "gcc.exe";
		if (m_bDebugging)
			strCmd += " -g";
		if (!m_bDebugging && m_nOptimize)	{
				strCmd += " -O";
				strCmd += '0' + m_nOptimize;
		}
		if (bCpp)	{
			if (m_bRTTI)
				strCmd += " -frtti";
			if (m_bExceptions)
				strCmd += " -fexceptions";
		}

		if (m_bStdLibIsInstalled && m_bUseStdLib)
			strCmd += " -fhonor-std";

		else if (m_bStrict)
			strCmd += " -pedantic-errors";

		int i;
		for (i = 0; i < m_Includes.GetSize(); i++)	{
			strCmd += " -I";
			strCmd += Enquote(m_Includes[i]);
		}

		if (m_bStdLibIsInstalled && m_bUseStdLib)	{
			strCmd += " -I";
			strCmd += Enquote(m_strStdIncludePath);
		}


		if (IsGUITarget())
				strCmd += " -mwindows";
		if (IsDLLTarget())
				strCmd += " -mdll";
		for (i = 0; i < m_Defines.GetSize(); i++)	{
			strCmd += " -D";
			strCmd += m_Defines[i];
		}

		strCmd += " -o ";
		strCmd += Enquote(strOFile);
		strCmd += " -c ";

		strCmd += Enquote(strFile);

		// ----- run the compiler
		m_CompileStatus = compiling;
		if (RunCompilerProgram(strCmd, &exitcode))
			ExitCode |= exitcode;

#else

		// ---- make a copy of the preprocessor output file name
		CString strIFile = WorkingFile(strSrcfile + (bCpp ? ".ii" : ".i"));
		// ---- make a copy of the compiler output file name
		CString strSFile = WorkingFile(strSrcfile + ".s");

		// ------ delete any previous compiler output files
		remove(strSFile);		// assembly language
		remove(strIFile);		// preprocessed source code

		// ------ build GNU preprocessor (CPP) command

		strCmd = m_strCompilerPath + "cpp.exe ";

		if (bCpp)
			strCmd += "-lang-c++ -D__cplusplus ";
		else
			strCmd += "-lang-c ";

		if (m_bDebugging)
			strCmd += "-g ";

		strCmd += "-iprefix ";

		strCmd += m_strIncludePath;
		strCmd += " ";

		int i;
		for (i = 0; i < m_Includes.GetSize(); i++)	{
			strCmd += "-I ";
			strCmd += m_Includes[i];
			strCmd += " ";
		}

		for (i = 0; i < m_Defines.GetSize(); i++)	{
			strCmd += "-D ";
			strCmd += m_Defines[i];
			strCmd += " ";
		}

		if (m_strVersion.Left(4).CompareNoCase("egcs") == 0)
			strCmd += "-undef ";

		if (bCpp)
			strCmd += 
				"-D__GNUG__=2 "
				"-D__EXCEPTIONS "
				"-Asystem(unix) ";
		else
			strCmd += 
				"-D_WIN32 ";
		strCmd +=
			"-D__GNUC__=2 "
			"-D__cdecl=__attribute__((__cdecl__)) "
			"-D__GNUC_MINOR__=95 "
			"-D__i386 "
			"-D__i386__ "
			"-D__MINGW32__=0.2 "
			"-D__STDC__=1 "
			"-D__stdcall=__attribute__((__stdcall__)) "
			"-D_stdcall=__attribute__((__stdcall__)) "
			"-D___stdcall__=__attribute__((__stdcall__)) "
			"-D___stdcall=__attribute__((__stdcall__)) "
			"-D__declspec(x)=__attribute__((x)) "
			"-D__WIN32 "
			"-D__WIN32__ "
			"-D__WINNT "
			"-D__WINNT__ "
			"-D_WIN32 "
			"-D_X86_=1 "
			"-Di386 "
			"-DWIN32 "
			"-DWINNT "
			"-Acpu(i386) "
			"-Asystem(winnt) "
			"-Amachine(i386) "
			"-remap ";

		strCmd += strFile + " ";

		strCmd += strIFile;
		// ----- run the preprocessor
		m_CompileStatus = preprocessing;

		if (RunCompilerProgram(strCmd, &exitcode))	{

			ExitCode |= exitcode;

			if (m_CompileStatus == aborting)	{
				remove(strIFile);
				break;
			}

			// ------ finished preprocessing, build the compiler command
			strCmd = m_strCompilerPath + "cc1";
			if (bCpp)
				strCmd += "plus";
			strCmd += ".exe";
			if (bCpp)	{
				if (m_bRTTI)
					strCmd += " -frtti";
				if (m_bExceptions)
					strCmd += " -fexceptions";
			}
			if (m_bStrict)
				strCmd += " -pedantic-errors";
			strCmd += " -quiet ";
			strCmd += strIFile;
			if (m_bDebugging)
				strCmd += " -g";
			if (IsGUITarget())
				strCmd += " -mwindows";
			strCmd += " -version";

			if (!m_bDebugging && m_nOptimize)	{
				strCmd += " -O";
				strCmd += '0' + m_nOptimize;
			}

			strCmd += " -o ";
			strCmd += strSFile;

			// ------ run the compiler
			m_CompileStatus = compiling;
			if (RunCompilerProgram(strCmd, &exitcode))	{

				// ------ finished compiling
				ExitCode |= exitcode;
				// ------ delete the preprocessor output
				remove(strIFile);

				if (m_CompileStatus == aborting)	{
					remove(strSFile);
					break;
				}

				if (ExitCode == 0)	{
					// ------ build the assembler command
					strCmd = m_strToolsPath + "as -o ";
					strCmd += strOFile;
					strCmd += " ";
					strCmd += strSFile;
					// ------ run the assembler
					m_CompileStatus = assembling;
					if (RunCompilerProgram(strCmd, &exitcode))	{
						// ------ finished assembling
						ExitCode |= exitcode;
						// ------ delete the compiler output
						remove(strSFile);
					}
				}
			}
		}
#endif // USEGCC

	}

	nSize = m_ObjectFiles.GetSize();

	if (m_CompileStatus != aborting && ExitCode == 0 && nSize != 0)	{

		CString strObjFiles;
		CString strFile;

		for (n = 0; n < nSize; n++)	{
			strFile = m_ObjectFiles[n];
			if (strFile.IsEmpty())
				break;
			strObjFiles += Enquote(strFile) + " ";
		}

		if (IsLibraryTarget())	{
			m_CompileStatus = buildinglibrary;

			// ----- build the librarian command
			strCmd = m_strToolsPath + "ar";
			strCmd += " rc ";
			strCmd += Enquote(m_strTargetName);
			strCmd += " ";

			strCmd += strObjFiles;

			RunCompilerProgram(strCmd, &exitcode);

			ExitCode |= exitcode;

		}
		else if (IsDLLTarget())	{
			if (bFoundDef)	{
				// ------ build dll
				m_CompileStatus = buildinglibrary;
				exitcode = RunLinker(1, strObjFiles);
				ExitCode |= exitcode;
				if (exitcode == 0 && m_CompileStatus != aborting)	{

					strCmd = Enquote(m_strToolsPath + "dlltool.exe");
					strCmd += " --dllname ";
					strCmd += Enquote(m_strTargetName);		// name of the dll

					strCmd += " --base-file ";
					strCmd += Enquote(WorkingFile("base.tmp"));
					strCmd += " --output-exp ";
					strCmd += Enquote(WorkingFile("temp.exp"));
					strCmd += " --def ";
					strCmd += Enquote(strDefname);		// name of the def

					if (RunCompilerProgram(strCmd, &exitcode, TRUE))	{
						ExitCode |= exitcode;
						if (exitcode == 0 && m_CompileStatus != aborting)	{
							exitcode = RunLinker(2, strObjFiles);
							ExitCode |= exitcode;
						}
					}
					remove(WorkingFile("base.tmp"));
					remove(WorkingFile("temp.exp"));
				}
			}
			else
				AfxMessageBox("DLL project has no .def file", MB_OK | MB_ICONSTOP);
		}
		else	{
			m_CompileStatus = linking;
			exitcode = RunLinker(0, strObjFiles);
			ExitCode |= exitcode;

			// --- if this is a compile and link of only one IDE translation unit
			//     and no user library files are in the project (no project, actually),
			//     no need to keep the object file around.
			if (!IsProjectFileLoaded())
				remove(strOFile);
		}
	}
	if (ExitCode != 0)
		m_CompileAction = none;

	CloseConsoleHandles();
	m_CompileStatus = m_CompileStatus == aborting ? aborted : finished;
}

// --- pass == 0 = normal link; 1 = 1st pass of dll build; 2 = 2nd pass of dll build
DWORD CQuincyApp::RunLinker(int pass, CString& strObjFiles)
{
	DWORD exitcode = 0;
	int nSize = m_LibraryFiles.GetSize();

	if (pass != 2)	{
		CString strFile;
		for (int n = 0; n < nSize; n++)	{
			strFile = m_LibraryFiles[n];
			if (strFile.IsEmpty())
				break;
			strObjFiles += Enquote(strFile) + " ";
		}
	}
	// ----- build the linker command
	CString strCmd = m_strToolsPath + "ld ";

	if (IsGUITarget())
		strCmd += "--subsystem windows ";

	if (IsDLLTarget())
		strCmd += "--dll -e _DllMainCRTStartup@12 ";

	strCmd += "-o ";
	strCmd += pass == 1 ? "junk.tmp " : (Enquote(m_strTargetName) + " ");

	if (!m_bDebugging)
		strCmd += "-s ";

	strCmd += Enquote(m_strStartupCode + (pass ? "\\dllcrt1.o": "\\crt1.o"));
	strCmd += " ";

	if (IsGUITarget() && m_bDebugging)
		strCmd += Enquote(m_strBinPath + "\\trace.o") + " ";

	if (m_bStdLibIsInstalled && m_bUseStdLib)	{
		strCmd += " -L";
		strCmd += Enquote(m_strStdLibPath);
		strCmd += " ";
	}

	int i;
	for (i = 0; i < 4; i++)	{
		strCmd += " -L";
		strCmd += Enquote(m_strLibPath[i]);
	}
	for (i = 0; i < m_Libs.GetSize(); i++)	{
		strCmd += " -L";
		strCmd += Enquote(m_Libs[i]);
	}

	if (pass == 1)
		strCmd += " --base-file base.tmp";

	strCmd += " ";
	strCmd += strObjFiles;

	if (pass == 2)
		strCmd += "temp.exp";

	if (m_bLinkCpp)
		strCmd += " -lstdc++";

	strCmd += " -lmingw32 -lgcc -lmoldname -lcrtdll";

	if (IsGUITarget() || IsDLLTarget())
		strCmd += " -luser32 -lgdi32 -lcomdlg32 -lwinmm -lcomctl32 -lctl3d32";

	strCmd += " -lkernel32 -ladvapi32 -lshell32 -lmingw32 -lgcc -lmoldname -lcrtdll";

	if (!RunCompilerProgram(strCmd, &exitcode))
		exitcode = 1;
	if (pass && exitcode == 0)
		remove(pass == 1 ? "junk.tmp" : "temp.exp");
	return exitcode;
}

BOOL CQuincyApp::RunCompilerProgram(CString& strCmd, DWORD* exitcode, BOOL bIsRC)
{
	DWORD dwFlags = STARTF_USESHOWWINDOW | 	STARTF_USESTDHANDLES;

	STARTUPINFO suinfo = {
		sizeof(STARTUPINFO),	// cb
		0, 0,					// reserved
		"Quincy compiler",		// lpTitle
		0, 0,					// dwX, dwY
		0, 0,					// dwXSize, dwYSize
		0, 0,					// dwXCountChars, dwYCountChars
		0,						// dwFillAttribute
		dwFlags,				// dwFlags
#ifdef USEGCC
		SW_HIDE,				// wShowWindow
#else
		SW_SHOWMINIMIZED,		// wShowWindow
#endif
		0, 0,					// reserved
		0,						// hStdInput;
		m_hStdout,				// hStdOutput;
		m_hStderr,				// hStdError;
	};

	char* pCmd = strCmd.GetBuffer(0);
	BOOL rtn = CreateProcess(	0,
								pCmd,0,0,
								TRUE, 
#ifdef USEGCC
								bIsRC ? 0 : CREATE_NEW_CONSOLE,
#else
								bIsRC ? 0 : DETACHED_PROCESS, 
#endif
								0,
								0,
								&suinfo,
								&m_CompileInformation);
	if (rtn)	{
		while (WaitForSingleObject(m_CompileInformation.hProcess,100) == WAIT_TIMEOUT)
			if (m_CompileStatus == aborting)
				TerminateProcess(m_CompileInformation.hProcess,0);
//
// This way (INFINITE) can abort only between compiler program passes
//
//		WaitForSingleObject(m_CompileInformation.hProcess,INFINITE);
//
		GetExitCodeProcess(m_CompileInformation.hProcess, exitcode);
		CloseHandle(m_CompileInformation.hThread);
		m_CompileInformation.hThread = 0;
		CloseHandle(m_CompileInformation.hProcess);
		m_CompileInformation.hProcess = 0;
	}
	else	{
		AfxMessageBox(strCmd + "\nCannot execute compiler program", MB_ICONSTOP);
		m_CompileStatus = aborting;
	}
	return rtn;
}

void CQuincyApp::ClearErrorLog()
{
	m_nErrorCount = 0;
	if (m_bErrorCreated == TRUE)
		m_pdlgErrors->ShowWindow(SW_HIDE);
}

// ---- parse error messages into the compiler error message log
void CQuincyApp::ProcessErrors()
{
	char buf[201];
	CString strErr(WorkingFile("quincy.err"));
	if (_access(strErr, 0) != 0)
		return;
	CStdioFile ifile(strErr, CFile::modeRead | CFile::shareDenyNone);
	CString strPrevLine;
	while (ifile.ReadString(buf, 200) != 0)	{
		if (strncmp(buf, "GNU", 3) == 0)
			continue;
		char *cp1 = buf;
		char *cp2 = buf;

		if (*cp1 == '/' && *(cp1+1) == '/' && *(cp1+3) == '/')	{
			// --- convert //C/ to C:/
			*cp1++ = *(cp1+2);	// drive letter
			*cp1++ = ':';
			cp2 = cp1++;
		}

		// ---- strip double slashes and backslashes from the error message
		while (*cp1)	{
			if (*cp1 == '\\' && *(cp1+1) == '\\')
				cp1++;
			else if (*cp1 == '/' && *(cp1+1) == '/')
				cp1++;
			*cp2++ = *cp1++;
		}
		*cp2 = '\0';

		CString strLn(buf);
		int nLinkerror = 0;
		int offset = strLn.Find(".c:");
		if (offset == -1)	{
			offset = strLn.Find(".cpp:");
			if (offset == -1)	{
				offset = strLn.Find(".h:");
				if (offset == -1)	{
					offset = strLn.Find(".i:");
					if (offset == -1)	{
						offset = strLn.Find(".ii:");
						if (offset == -1)	{
							offset = strLn.Find(".cc:");
							if (offset == -1)	{
								offset = strLn.Find(".rc:");
								if (offset != -1)	{
									nLinkerror = 4;	// .rc compiler error
									offset++;
								}
							}
							else	{
								nLinkerror = 3;	// .cc compiler error
								offset++;
							}
						}
						else
							nLinkerror = 2;	// .cpp link error
					}
					else
						nLinkerror = 1;		// .c link error
				}
			}
			else
				offset += 2;	// .cpp compiler error
		}

		CString strer;

		ErrorLog[m_nErrorCount].m_strFileSpec = "";
		ErrorLog[m_nErrorCount].m_nLineNumber = 0;

		if (offset == -1)
			strer = strLn;	// unparsed error message, display as is
		else if (nLinkerror != 0 && nLinkerror != 3 && nLinkerror != 4)	{
			// ---- linker error message, scan to beginning of .i or .ii file name
			char ch;
			int foffset = offset > 0 ? offset-1 : 0;
			while (foffset >= 0 && (ch = strLn[foffset]) == '_' || isalpha(ch) || isdigit(ch))
				--foffset;
			strer = strLn.Mid(foffset+1, offset-foffset);
			strer += "c";
			if (nLinkerror == 2)
				strer += "pp";
			ErrorLog[m_nErrorCount].m_strFileSpec = strer;
			strer += strLn.Mid(offset+1+nLinkerror);
		}
		else	{
			// ---- compiler warning/error message found
			offset += 3;
			if (strLn[offset] == ' ' && nLinkerror != 3 && nLinkerror != 4)
				// ---- no line number, might be a library link error
				strer = strLn;	// display as is
			else	{
				if (nLinkerror != 3)	{
					// ---- retrieve the source code line number for this error
					UINT nLineNo = atoi(strLn.GetBuffer(0)+offset);
					ErrorLog[m_nErrorCount].m_nLineNumber = nLineNo;
					while (isdigit(strLn[offset]))
						offset++;
					if (strLn.Left(offset) == strPrevLine)
						continue;
					strPrevLine = strLn.Left(offset);
				}
				// ---- parse out file name and error message
				char ch;
				int foffset = 0;
				--offset;
				while ((ch = strLn[offset]) != '\\' && ch != '/' && ch != ' ')	{
					if (ch == ':')	{
						if (foffset == 0)
							foffset = offset;
						else if (nLinkerror == 3)
							break;
					}
					if (offset-- == 0)
						break;
				}
				strer = (strLn.Right(strLn.GetLength() - (offset+1)));

				if (nLinkerror != 3)	{
					// ------ parse out file spec
					CString strFileSpec = strLn.Left(foffset);
					--foffset;
					while (strFileSpec[foffset] != ' ')
						if (foffset-- == 0)
							break;
					ErrorLog[m_nErrorCount].m_strFileSpec = 
						strFileSpec.Right(strFileSpec.GetLength()-(foffset+1));
				}
			}
		}
		// ------ put error entry into error log array
		ErrorLog[m_nErrorCount].m_strErrorMsg = strer.Left(strer.GetLength()-1);

		if (++m_nErrorCount == maxerrors)
			break;
	}
	ifile.Close();
	if (m_nErrorCount > 0)	{
		m_strErrorFile = strErr;
		OnShowErrorlog();
	}
	else	{
		remove(strErr);
		m_strErrorFile.Empty();
	}
	remove(WorkingFile("quincy.out"));
}

