// Quincy.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <process.h>
#include <ddeml.h>
#include <io.h>
#include "Quincy.h"
#include "AboutDlg.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "QuincyDoc.h"
#include "QuincyView.h"
#include "TextDocument.h"
#include "TextView.h"
#include "OptionsDialog.h"
#include "dlgCompiler.h"
#include "CommandLineDialog.h"
#include "ErrorLogDialog.h"
#include "ExamineDialog.h"
#include "WatchDialog.h"
#include "TraceDialog.h"
#include "stabs.h"
#include "parser.h"
#include "debugger.h"
#include "TutDlg.h"
#include "GrepResult.h"
#include "GrepInput.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CQuincyApp

BEGIN_MESSAGE_MAP(CQuincyApp, CWinApp)
	//{{AFX_MSG_MAP(CQuincyApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_STOPBUILD, OnStop)
	ON_UPDATE_COMMAND_UI(ID_STOPBUILD, OnUpdateStop)
	ON_COMMAND(ID_OPTIONS, OnOptions)
	ON_COMMAND(ID_SHOW_ERRORLOG, OnShowErrorlog)
	ON_UPDATE_COMMAND_UI(ID_SHOW_ERRORLOG, OnUpdateShowErrorlog)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_CLEARBREAKPOINTS, OnUpdateDebugClearbreakpoints)
	ON_COMMAND(ID_DEBUG_CLEARBREAKPOINTS, OnDebugClearbreakpoints)
	ON_COMMAND(ID_DEBUG_STEPOVER, OnDebugStepover)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_EXAMINE, OnUpdateDebugExamine)
	ON_COMMAND(ID_DEBUG_EXAMINE, OnDebugExamine)
	ON_COMMAND(ID_DEBUG_WATCH, OnDebugWatch)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_ADDWATCH, OnUpdateDebugAddwatch)
	ON_COMMAND(ID_DEBUG_ADDWATCH, OnDebugAddwatch)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_STEPOVER, OnUpdateDebugStepover)
	ON_COMMAND(ID_HELP, OnHelp)
	ON_COMMAND(ID_RESOURCEEDITOR, OnResourceeditor)
	ON_COMMAND(ID_GREP, OnGrep)
	ON_UPDATE_COMMAND_UI(ID_GREP, OnUpdateGrep)
	ON_UPDATE_COMMAND_UI(ID_RESOURCEEDITOR, OnUpdateResourceeditor)
	ON_COMMAND(ID_COMPILER, OnCompiler)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_DEBUG_TRACE, OnDebugTrace)
	ON_COMMAND(ID_USESTDLIB, OnUseStdLib)
	ON_UPDATE_COMMAND_UI(ID_USESTDLIB, OnUpdateUsestdlib)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
//	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
#ifdef TYCPP
	// Tutorial commands
	ON_COMMAND(ID_WRITETUTORIAL, OnWritetutorial)
	ON_COMMAND(ID_TYCPP, OnTycpp)
	ON_UPDATE_COMMAND_UI(ID_TYCPP, OnUpdateTycpp)
#endif
END_MESSAGE_MAP()

const int maxsaveddocuments = 10;	// maximum # of documents remembered when reloading

/////////////////////////////////////////////////////////////////////////////
// CQuincyApp construction

CQuincyApp::CQuincyApp()
{
	m_bCpp = FALSE;
	m_bDoLink = FALSE;
	m_pView = 0;
	m_bErrorCreated = FALSE;
	m_bGrepInputCreated = FALSE;
	m_bGrepResultCreated = FALSE;
	m_bDebugging = FALSE;
	m_bExceptions = FALSE;
	m_bRTTI = FALSE;
	m_bStrict = FALSE;
	m_bClosing = FALSE;
	m_nOptimize = 0;
	m_nExitCode = 0;
	m_CompileStatus = idle;
	m_OldCompileStatus = idle;
	m_nErrorCount = 0;
	m_pdlgErrors = new CErrorLogDialog;
	m_pdlgGrepResult = new GrepResult;
	m_pdlgGrepInput = new GrepInput;
	m_pdlgWatch = new CWatchDialog;
	m_pdlgTrace = new CTraceDialog;
	m_pParser = new Parser;
	m_bBypassChangeTest = FALSE;
	m_CommandLinePromptOption = 1;
	m_CompileAction = none;
	m_pDebugger = 0;
	m_ConsoleRect.SetRectEmpty();
	m_bLinkCpp = FALSE;
	m_bShowErrors = FALSE;
#ifdef TYCPP
	m_bReadTutorial = FALSE;
	m_bTutorialDisplayed = FALSE;
	m_bIsTutorial = FALSE;
	m_pCTutDlg = 0;
	m_bTutCreated = FALSE;
#endif
	m_bGrepIsInstalled = FALSE;
	m_bWEditResIsInstalled = FALSE;
	m_bCompilerIsInstalled = FALSE;
	m_hStdin = INVALID_HANDLE_VALUE;
	m_hStdout = INVALID_HANDLE_VALUE;
	m_hStderr = INVALID_HANDLE_VALUE;
	m_bWatchCreated = FALSE;
	m_bTraceCreated = FALSE;

	m_bStdLibIsInstalled = FALSE;
	m_bUseStdLib = FALSE;
}

CQuincyApp::~CQuincyApp()
{
	delete m_pParser;
	delete m_pdlgWatch;
	delete m_pdlgTrace;
	delete m_pdlgErrors;
	delete m_pdlgGrepResult;
	delete m_pdlgGrepInput;
	delete m_pDebugger;
#ifdef TYCPP
	delete m_pCTutDlg;
#endif
}

#ifdef TYCPP

TCHAR       szApp[] = TEXT("Quincy");    // DDE service name
DWORD       idInst = 0;                 // our DDEML instance object
HSZ         hszAppName = 0;             // the generic hsz for everything
UINT        OurFormat;                  // our custom registered format

HDDEDATA CALLBACK DdeCallback(WORD wType, WORD wFmt, HCONV hConv, HSZ hszTopic,
        HSZ hszItem, HDDEDATA hData, DWORD lData1, DWORD lData2);

void RegisterDDE()
{
    DdeInitialize(&idInst,
            (PFNCALLBACK)MakeProcInstance((FARPROC)DdeCallback, hInstance),
            APPCMD_FILTERINITS |
            CBF_SKIP_CONNECT_CONFIRMS |
            CBF_FAIL_SELFCONNECTIONS |
            CBF_FAIL_POKES,
            0);
    hszAppName = DdeCreateStringHandle(idInst, szApp, 0);
    OurFormat = RegisterClipboardFormat(szApp);
    DdeNameService(idInst, hszAppName, 0, DNS_REGISTER);
}
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only CQuincyApp object

CQuincyApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CQuincyApp initialization

BOOL CQuincyApp::InitInstance()
{
	SetRegistryKey("Al Stevens Programs");

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// ---- load Quincy's INI options
	m_strWorkingPath = GetProfileString("Directories", "WorkingPath");
	m_strCompiler    = GetProfileString("Directories", "Compiler");
	m_strRuntimeDirectory = GetProfileString("Directories", "Runtime");

#ifdef TYCPP
	m_bTutorialDisplayed = GetProfileInt("Tutorial",    "Displayed", 1);
#endif
	m_bUseStdLib = GetProfileInt("Options", "UseStdLib", 0);
	m_bDebugging = GetProfileInt("Options", "Debugging", 1);
	m_bExceptions = GetProfileInt("Options", "Exceptions", 0);
	m_bRTTI = GetProfileInt("Options", "RTTI", 0);
	m_bStrict = GetProfileInt("Options", "Strict", 0);
	m_strCommandLine = GetProfileString("Options", "Command Line");
	m_CommandLinePromptOption = GetProfileInt("Options", "Command Line Prompt", 1);
	m_nOptimize = GetProfileInt("Options", "Optimize", 0);
	CString strBuffer = GetProfileString("Settings", "Console Position");
	if (!strBuffer.IsEmpty())
		_stscanf (strBuffer, "%i:%i:%i:%i",
			&m_ConsoleRect.left,
			&m_ConsoleRect.top,
			&m_ConsoleRect.right,
			&m_ConsoleRect.bottom);

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CMultiDocTemplate* pDocTemplate;

	pDocTemplate = new CMultiDocTemplate(
		IDR_CPPTEMPLATE,
		RUNTIME_CLASS(CTextDocument),	// document class
		RUNTIME_CLASS(CChildFrame),		// frame class
		RUNTIME_CLASS(CTextView));		// view class
	AddDocTemplate(pDocTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_CTEMPLATE,
		RUNTIME_CLASS(CTextDocument),	// document class
		RUNTIME_CLASS(CChildFrame),		// frame class
		RUNTIME_CLASS(CTextView));		// view class
	AddDocTemplate(pDocTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_HTEMPLATE,
		RUNTIME_CLASS(CTextDocument),	// document class
		RUNTIME_CLASS(CChildFrame),		// frame class
		RUNTIME_CLASS(CTextView));		// view class
	AddDocTemplate(pDocTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_PROJTYPE,
		RUNTIME_CLASS(CQuincyDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CQuincyView));
	AddDocTemplate(pDocTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_RCTEMPLATE,
		RUNTIME_CLASS(CTextDocument),	// document class
		RUNTIME_CLASS(CChildFrame),		// frame class
		RUNTIME_CLASS(CTextView));		// view class
	AddDocTemplate(pDocTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_DEFTEMPLATE,
		RUNTIME_CLASS(CTextDocument),	// document class
		RUNTIME_CLASS(CChildFrame),		// frame class
		RUNTIME_CLASS(CTextView));		// view class
	AddDocTemplate(pDocTemplate);

	pDocTemplate = new CMultiDocTemplate(
		IDR_EDITORTYPE,
		RUNTIME_CLASS(CTextDocument),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CTextView));
	AddDocTemplate(pDocTemplate);

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	CWnd* pFocusWnd = 0;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Enable DDE Execute open
	EnableShellOpen();

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// ------- find path to TEMP disk space if any
	char* tempenv;
	if ((tempenv = getenv("TEMP")) != 0)	{
		m_strTempPath = tempenv;
		if (m_strTempPath.GetLength() && m_strTempPath[m_strTempPath.GetLength()-1] != '\\')
			m_strTempPath += "\\";
	}

	SetCompilerPaths();

#ifdef TYCPP
	LoadTutorials();
	int nDocno = 0;

	BOOL bCmdFile = FALSE;

	// --- suppress initial FileNew command on startup
	if (cmdInfo.m_nShellCommand != CCommandLineInfo::FileNew)	{
		if (cmdInfo.m_strFileName != "tutorial")
		{
			if (IsTutorial(cmdInfo.m_strFileName))
			{
				// --- reading a .tut file
				m_bReadTutorial = TRUE;
				int nIndex = cmdInfo.m_strFileName.ReverseFind('\\');
				if (nIndex == -1)	{
					char path[MAX_PATH];
					getcwd(path, MAX_PATH);
					m_strTutorial = path;
					m_strTutorial += "\\";
				}
				m_strTutorial += cmdInfo.m_strFileName;
				m_strWorkingPath.Empty();
			}
			else
				if (!ProcessShellCommand(cmdInfo))
					return FALSE;
			else
			{
				// ---- loading file from command line
				bCmdFile = TRUE;
				CString strPath = GetFilePath(cmdInfo.m_strFileName);
				if (!strPath.IsEmpty())
					_chdir(strPath);
			}
		}
	}
	else if (!m_bIsTutorial || !m_bTutorialDisplayed)
	{
		// ----- reload documents from the previous session
		CString strProfile;
		for (; nDocno < maxsaveddocuments; nDocno++)	{
			strProfile.Format("Document%d", nDocno+1);
			CString strDocument = GetProfileString("Documents", strProfile, 0);
			if (strDocument.IsEmpty())
				break;

			if (OpenDocumentFile(strDocument))	{
				// --- if the document was maximized on exit, maximize it now
				strProfile.Format("Document%d Maximized", nDocno+1);
				BOOL bIsMax = GetProfileInt("Documents", strProfile, 0);
				if (bIsMax)
					// --- by coincidence, GetTextView works for project document, too
					pFocusWnd = MaximizeDocument(strDocument);
			}
		}
	}

	if (!bCmdFile)	{
		// ---- reload tutorial info from the previous session
		m_strTutorial = GetProfileString("Tutorial", "Current", 0);
		// ---- reload working directory from the previous session
		CString strPath = GetProfileString("Directories", "Current", 0);
		// ------ change to current working directory
		if (!strPath.IsEmpty())
			_chdir(strPath);
	}
/*
	CString strWorkingPath = m_strBinPath + "\\..\\Working";

	if (_access(strWorkingPath, 0) == 0)
	{
		UINT nDriveType = GetDriveType(strWorkingPath.Left(3));
		if (nDriveType != DRIVE_FIXED && nDriveType != DRIVE_REMOVABLE)
			m_strWorkingPath = strWorkingPath;
	}
*/
#else
	int nDocno = 0;

	BOOL bCmdFile = FALSE;

	// --- suppress initial FileNew command on startup
	if (cmdInfo.m_nShellCommand != CCommandLineInfo::FileNew)	{
		if (!ProcessShellCommand(cmdInfo))
			return FALSE;
		// ---- loading file from command line
		bCmdFile = TRUE;
		CString strPath = GetFilePath(cmdInfo.m_strFileName);
		if (!strPath.IsEmpty())
			_chdir(strPath);
	}
	else	{
		// ----- reload documents from the previous session
		CString strProfile;
		for (; nDocno < maxsaveddocuments; nDocno++)	{
			strProfile.Format("Document%d", nDocno+1);
			CString strDocument = GetProfileString("Documents", strProfile, 0);
			if (strDocument.IsEmpty())
				break;

			if (OpenDocumentFile(strDocument))	{
				// --- if the document was maximized on exit, maximize it now
				strProfile.Format("Document%d Maximized", nDocno+1);
				BOOL bIsMax = GetProfileInt("Documents", strProfile, 0);
				if (bIsMax)
					// --- by coincidence, GetTextView works for project document, too
					pFocusWnd = MaximizeDocument(strDocument);
			}
		}
	}

	if (!bCmdFile)	{
		// ---- reload working directory from the previous session
		CString strPath = GetProfileString("Directories", "Current", 0);
		// ------ change to current working directory
		if (!strPath.IsEmpty())
			_chdir(strPath);
	}

#endif

	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	if (pFocusWnd != 0)
		pFocusWnd->SetFocus();

#ifdef TYCPP
	RegisterDDE();
	if (!bCmdFile && m_bIsTutorial && nDocno == 0 && m_bTutorialDisplayed)	{
		OnTycpp();
		ASSERT(m_pCTutDlg != 0);
		m_pCTutDlg->LoadSelectedTutorial();
	}
	// --- disable tycpp command if tutorial not present
	if (!m_bIsTutorial)
		((CMainFrame*)pMainFrame)->HideTYCPPCommand();
#endif

	return TRUE;
}

void CQuincyApp::SetCompilerPaths()
{
	// ---- set path to find the compiler executables
	m_strBinPath = m_pszHelpFilePath;
	int nIndex = m_strBinPath.ReverseFind('\\');
	ASSERT(nIndex > 0);
	m_strBinPath = m_strBinPath.Left(nIndex);
	nIndex = m_strBinPath.ReverseFind('\\');
	ASSERT(nIndex > 0);
	CString strQuincyPath = m_strBinPath.Left(nIndex);

	////////////////////////////////////////////////////////////////////////
	//
	//	This code depends on gcc-Mingw32 compiler's subdirectory structure
	//  If they change it, this code will need to be changed.
	//
	////////////////////////////////////////////////////////////////////////

	if (m_strCompiler.IsEmpty())
		m_strCompiler = strQuincyPath + "\\mingw32\\";

	#define COMPILER_BIN  m_strCompiler + "bin\\"
	#define COMPILER_PATH m_strCompiler + "lib\\gcc-lib\\i386-mingw32\\"
	#define INCLUDE_PATH  m_strCompiler + "lib\\gcc-lib\\i386-mingw32\\"
	#define RINCLUDE_PATH m_strCompiler + "i386-mingw32\\include\\"
	#define COMPILER_LIB0 m_strCompiler + "lib\\gcc-lib\\i386-mingw32\\"
	#define COMPILER_LIB1 m_strCompiler + "lib\\gcc-lib"
	#define COMPILER_LIB2 m_strCompiler + "lib"
	#define COMPILER_LIB3 m_strCompiler + "i386-mingw32\\lib"
	#define COMPILER_CRT0 m_strCompiler + "i386-mingw32\\lib"
	#define RESOURCE_BIN  strQuincyPath + "\\lcc\\bin\\"

	#define STD_INCLUDE_PATH  strQuincyPath + "\\libstdc++\\include\\g++-v3\\"
	#define STD_LIBRARY_PATH  strQuincyPath + "\\libstdc++\\lib\\"

	for (;;)	{

		CString strSpec = COMPILER_PATH + "*.*";
		struct _finddata_t file;
		long hFile;
		
		// --- gcc-mingw32 uses its version number as the only subdirectory
		//     under the "lib\\gcc-lib\\i386-mingw32\\" subdirectory
		//     By scanning for it, don't need a Quincy 99 recompile 
		//     whenever they release a new version (I hope)

		if ((hFile = _findfirst(strSpec.GetBuffer(0), &file)) != -1)	{
			while (*file.name == '.' || (file.attrib & _A_SUBDIR) == 0)
				if (_findnext(hFile, &file) != 0)
					break;
			if (*file.name != '.' && (file.attrib & _A_SUBDIR) != 0)
				m_strVersion = file.name;

			_findclose(hFile);
		}

		////////////////////////////////////////////////////////////////////////
		//
		//	End of compiler-dependent code
		//
		////////////////////////////////////////////////////////////////////////

		if (!m_strVersion.IsEmpty())	{
			char* penv = 0;
			penv = getenv("PATH");
			CString path("Path=");
			path += penv;
			if (path.Right(1) != ";")
				path += ';';
			path += COMPILER_BIN;
			_putenv(path.GetBuffer(0));

			m_strToolsPath = COMPILER_BIN;
			m_strCompilerPath = COMPILER_PATH + m_strVersion + "\\";

			m_strRIncludePath = RINCLUDE_PATH;
			m_strIncludePath  = INCLUDE_PATH + m_strVersion + "\\";
			m_strStartupCode  = COMPILER_CRT0;

			m_strLibPath[0] = COMPILER_LIB0 + m_strVersion;
			m_strLibPath[1] = COMPILER_LIB1;
			m_strLibPath[2] = COMPILER_LIB2;
			m_strLibPath[3] = COMPILER_LIB3;

			m_strStdLibPath = STD_LIBRARY_PATH;
			m_strStdIncludePath = STD_INCLUDE_PATH;

		}
		m_strLCCPath = RESOURCE_BIN;

		// --- disable compiler and tools menu items if the executables are not installed
		CMenu* pMenu = ((CMainFrame*)m_pMainWnd)->GetMenu();
		ASSERT(pMenu != 0);

		CString strGrep = m_strBinPath + "\\grep.exe";
		m_bGrepIsInstalled = _access(strGrep, 0) == 0;

		CString strWEditRes(m_strLCCPath + "weditres.exe ");
		m_bWEditResIsInstalled = _access(strWEditRes, 0) == 0;

		if (m_strVersion.IsEmpty())	{
			m_bCompilerIsInstalled = false;
			m_bStdLibIsInstalled = false;
		}
		else	{
			CString strGcc = m_strToolsPath + "gcc.exe";
			m_bCompilerIsInstalled = _access(strGcc, 0) == 0;
		}

		if (m_bCompilerIsInstalled)	{
			CString strStd = m_strStdLibPath + "libstdc++.a";
			m_bStdLibIsInstalled = _access(strStd, 0) == 0;
		}

		if (!m_bStdLibIsInstalled)
			ResetMenuCommand(ID_USESTDLIB);

		else if (m_bUseStdLib)
			SetMenuCommand(ID_USESTDLIB);

		if (!m_bCompilerIsInstalled)	{
			if (IsOnCDROM(m_strToolsPath))	{
				CString msg("Compiler files not found. Insert CD-ROM in ");
				char dr[3] = "x:";
				*dr = m_strToolsPath[0];
				msg += dr;
				if (AfxMessageBox(msg, MB_RETRYCANCEL) == IDRETRY)
					continue;
			}
			else	{
				AfxMessageBox(	"Install the gcc-Mingw32 compiler "
					"and set its path in the Tools/Compiler Paths dialog.", MB_ICONSTOP);
			}
		}
		break;
	}
}

CWnd* CQuincyApp::MaximizeDocument(const CString& strDocument)
{
	CTextView* pView = GetTextView(strDocument);
	ASSERT(pView != 0);
	CWnd* pWnd = pView->GetParent();
	ASSERT(pWnd != NULL);
	WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
	if (pWnd->GetWindowPlacement(&wndpl))	{
		wndpl.showCmd = SW_SHOWMAXIMIZED;
		pWnd->SetWindowPlacement(&wndpl);
	}
	return pView;
}

CDocument* CQuincyApp::OpenDocumentFile(LPCTSTR lpszFileName) 
{
	CString strDocument(lpszFileName);
	if (strDocument.Right(4).CompareNoCase(".prj") == 0)	{
		// ----- change to the subdirectory where project file is located
		int offset = strDocument.ReverseFind('\\');
		if (offset != -1)	{
			CString strPath = strDocument.Left(offset);
			_chdir(strPath);
		}
	}
	return CWinApp::OpenDocumentFile(lpszFileName);
}

void CQuincyApp::RunApplicationProgram(CString& strCmd, BOOL bStepping)
{
	ASSERT(m_pDebugger == 0);

	m_pDebugger = new Debugger;
	if (bStepping)
		m_pDebugger->SetStepping();
	m_pDebugger->Execute(strCmd);
}

void CQuincyApp::StopDebugger()
{
	delete m_pDebugger;
	m_pDebugger = 0;
	RefreshAllWatches();
	if (m_bClosing)	{
		((CMainFrame*)m_pMainWnd)->DoClose();
		m_bClosing = FALSE;
	}
}

void CQuincyApp::ParseFileSpec(CString& strFileSpec, CString strCmdLine, int nIndex)
{
	int nx = 0;
	char* pPath = new char[501];
	while (isspace(strCmdLine[nIndex]))
		nIndex++;
	while (nIndex < strCmdLine.GetLength() && !isspace(strCmdLine[nIndex]) && nx < 500)
		*(pPath + nx++) = strCmdLine[nIndex++];
	*(pPath + nx) = '\0';
	strFileSpec = pPath;
	delete pPath;
}

void CQuincyApp::GetCommandLine(CString& strCmdLine)
{
	strCmdLine = m_strCommandLine;
	if (m_CommandLinePromptOption == 0)	{
		CCommandLineDialog dlgCmdLine;
		if (dlgCmdLine.DoModal() == IDOK)
			strCmdLine = dlgCmdLine.m_strCommandLine;
	}
}

// ----  extract the path from a file specification
CString CQuincyApp::GetFilePath(const CString& rstrPath)
{
	int offset = rstrPath.ReverseFind('\\');
	if (offset == -1)
		offset = rstrPath.ReverseFind('/');
	return offset == -1 ? CString() : rstrPath.Left(offset);
}
// ----  extract the filename from a file specification
CString CQuincyApp::GetFileName(const CString& rstrPath)
{
	int offset = rstrPath.ReverseFind('\\');
	if (offset == -1)
		offset = rstrPath.ReverseFind('/');
	return offset == -1 ? rstrPath : rstrPath.Right(rstrPath.GetLength() - offset - 1);
}

// ----- compare the date/time stamps on two files
int CQuincyApp::CompareFileTimes(CString& strFile1, CString& strFile2)
{
	struct _stat buf1;
	char* path1 = strFile1.GetBuffer(0);
	int rtn1 = _stat(path1, &buf1);

	struct _stat buf2;
	char* path2 = strFile2.GetBuffer(0);
	int rtn2 = _stat(path2, &buf2);

	// --- a non-existing file is always older than one that exists
	if (rtn1 == -1 && rtn2 == -1)
		return 0;
	if (rtn1 == 0 && rtn2 == -1)
		return 1;
	if (rtn1 == -1 && rtn2 == 0)
		return -1;

	// ---- a zero-length file is always older than one with data
	if (buf1.st_size == 0 && buf2.st_size != 0)
		return -1;
	if (buf1.st_size != 0 && buf2.st_size == 0)
		return 1;

	return buf1.st_mtime - buf2.st_mtime;
}

CString CQuincyApp::GetProjectRuntimeDirectory()
{
	CString strWk;
	CQuincyDoc* pDoc = CQuincyDoc::CurrentProject();
	if (pDoc != 0)
		strWk = pDoc->WorkingDirectory();
	return strWk;
}

CQuincyDoc* CQuincyApp::GetProjectDocument(CString* pstrPath)
{
	CQuincyDoc* pDoc = CQuincyDoc::CurrentProject();
	if (pDoc != 0)	{
		CString strPath = pDoc->GetPathName();
		if (pstrPath != 0)
			*pstrPath = strPath;
	}
	return pDoc;
}

/*
CString CQuincyApp::GetActiveSourceFile(std::string path)
{
	CString strProj;
	GetProjectDocument(&strProj);
	if (strProj.IsEmpty() || strProj.GetLength() < 2 || strProj[1] != ':')	{
		// iterate the documents loaded in the editor looking for one that matches the argument
		CTextDocument* pDoc = GetTextDocument(CString(path.c_str()));
		if (pDoc != 0)
			strProj = pDoc->GetPathName();
	}
	return strProj;
}
*/

BOOL CQuincyApp::IsLibraryTarget()
{
	CQuincyDoc* pDoc = GetProjectDocument();
	if (pDoc != 0)
		return pDoc->IsLibraryTarget();
	return FALSE;
}

BOOL CQuincyApp::IsGUITarget()
{
	CQuincyDoc* pDoc = GetProjectDocument();
	if (pDoc != 0)
		return pDoc->IsGUITarget();
	return FALSE;
}

BOOL CQuincyApp::IsDLLTarget()
{
	CQuincyDoc* pDoc = GetProjectDocument();
	if (pDoc != 0)
		return pDoc->IsDLLTarget();
	return FALSE;
}

BOOL CQuincyApp::IsConsoleTarget()
{
	CQuincyDoc* pDoc = GetProjectDocument();
	if (pDoc != 0)
		return pDoc->IsConsoleTarget();
	return TRUE;
}

CQuincyView* CQuincyApp::GetProjectView()
{
	CQuincyDoc* pDoc = GetProjectDocument();
	if (pDoc != 0)	{
		POSITION vpos = pDoc->GetFirstViewPosition();
		ASSERT(vpos != 0);
		CQuincyView* pView = static_cast<CQuincyView*>(pDoc->GetNextView(vpos));
		ASSERT(pView != 0);
		return pView;
	}
	return 0;
}

CTextDocument* CQuincyApp::GetTextDocument(CString pstrPath)
{
	POSITION tpos = GetFirstDocTemplatePosition();
	CString strArgFile = GetFileName(pstrPath);
	while (tpos != 0)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CTextDocument* pDoc = static_cast<CTextDocument*>(pTem->GetNextDoc(dpos));
			ASSERT(pDoc != 0);
			CString strDocFile = GetFileName(pDoc->GetPathName());
			if (strArgFile.CompareNoCase(strDocFile) == 0)
				return pDoc;
		}
	}
	return 0;
}

CTextView* CQuincyApp::GetTextView(CString pstrPath)
{
	CTextDocument* pDoc = GetTextDocument(pstrPath);
	if (pDoc != 0)	{
		POSITION vpos = pDoc->GetFirstViewPosition();
		ASSERT(vpos != 0);
		CTextView* pView = static_cast<CTextView*>(pDoc->GetNextView(vpos));
		ASSERT(pView != 0);
		return pView;
	}
	return 0;
}

void CQuincyApp::RunResourceEditor(const CString& strItem)
{
	CString strCmd(m_strLCCPath + "weditres.exe " + strItem);
	STARTUPINFO suinfo = { sizeof(STARTUPINFO) };
	PROCESS_INFORMATION pi;
	CreateProcess(0, strCmd.GetBuffer(0), 0,0,0,0,0,0, &suinfo, &pi);
}

HANDLE CQuincyApp::BuildStdin(const CString& rstrPath)
{
	if (rstrPath.IsEmpty())
		m_hStdin = GetStdHandle(STD_INPUT_HANDLE);
	else	{
		SECURITY_ATTRIBUTES sa = { sizeof(SECURITY_ATTRIBUTES), 0, TRUE };
		m_hStdin = CreateFile(rstrPath, GENERIC_READ, FILE_SHARE_READ, &sa, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
	}
	return m_hStdin;
}
HANDLE CQuincyApp::BuildStdout(const CString& rstrPath)
{
	if (rstrPath.IsEmpty())
		m_hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
	else	{
		SECURITY_ATTRIBUTES sa = { sizeof(SECURITY_ATTRIBUTES), 0, TRUE };
		m_hStdout = CreateFile(rstrPath, GENERIC_WRITE, FILE_SHARE_WRITE, &sa, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
	}
	return m_hStdout;
}

HANDLE CQuincyApp::BuildStderr(const CString& rstrPath)
{
	if (rstrPath.IsEmpty())
		m_hStderr = GetStdHandle(STD_ERROR_HANDLE);
	else	{
		SECURITY_ATTRIBUTES sa = { sizeof(SECURITY_ATTRIBUTES), 0, TRUE };
		m_hStderr = CreateFile(rstrPath, GENERIC_WRITE, FILE_SHARE_WRITE, &sa, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
	}
	return m_hStderr;
}

void CQuincyApp::CloseConsoleHandle(HANDLE& hStd, DWORD stdhandle)
{
	if (hStd != INVALID_HANDLE_VALUE)	{
		if (hStd != GetStdHandle(stdhandle))
			CloseHandle(hStd);
		hStd = INVALID_HANDLE_VALUE;
	}
}

void CQuincyApp::CloseConsoleHandles()
{
	CloseConsoleHandle(m_hStdin, STD_INPUT_HANDLE);
	CloseConsoleHandle(m_hStdout,STD_OUTPUT_HANDLE);
	CloseConsoleHandle(m_hStderr,STD_ERROR_HANDLE);
}

void CQuincyApp::ShowStatusText(CString& strText, CString* pstrOldText)
{
	CMainFrame* pMainFrame = static_cast<CMainFrame*>(m_pMainWnd);
	pMainFrame->ShowStatus(strText, pstrOldText);	
}
void CQuincyApp::ShowLineColumn(int nLine, int nColumn)
{
	CString strExt;
	if (nLine != 0)
		strExt.Format("Ln %d, Col %d", nLine, nColumn);
	ShowStatusText(strExt);
}

void CQuincyApp::CompileStep()
{
	switch (m_CompileStatus)	{
		case preprocessing:
		case compiling:
			ShowStatusText(CString("Compiling " + m_strFilename));
			break;
		case linking:
			ShowStatusText(CString("Linking " + m_strTargetName));
			break;
		case buildinglibrary:
			ShowStatusText(CString("Building " + m_strTargetName));
			break;
		case finished:
		case aborted:
			// --- kludge to make toolbar OnUpdates take effect
			POINT pt;
			GetCursorPos(&pt);
			SetCursorPos(pt.x+1, pt.y);
			// --- end of kludge
			ShowStatusText(m_strOldStatus);
			if (m_CompileStatus == aborted)
				AfxMessageBox("User terminated compile", MB_ICONEXCLAMATION);
			else
				ProcessErrors();
			m_CompileStatus = idle;
			if (m_CompileAction != none)
				RunApplicationProgram(m_strTargetName, m_CompileAction == step);
			m_CompileAction = none;
			break;
		default:
			break;
	}
}

BOOL CQuincyApp::OnIdle(LONG lCount)
{
	CWinApp::OnIdle(lCount);
	if (m_CompileStatus != m_OldCompileStatus)	{
		CompileStep();
		m_OldCompileStatus = m_CompileStatus;
	}

	if (m_CompileStatus != idle)	{
		if (m_pMainWnd != 0)	{
			m_pMainWnd->KillTimer(24);
			m_pMainWnd->SetTimer(24, 250, 0);
		}
		return FALSE;
	}

	if (m_pDebugger != 0)	{
		m_pDebugger->ContinueDebugging();
		if (m_pMainWnd != 0)	{
			m_pMainWnd->KillTimer(24);
			m_pMainWnd->SetTimer(24, 250, 0);
		}
		return FALSE;
	}

#ifdef TYCPP
	if (m_bReadTutorial)
		ReadTutorial();
#endif

	if (m_bShowErrors && !m_strErrorFile.IsEmpty())	{
		WinExec(CString("notepad " + m_strErrorFile), SW_SHOW);
		m_bShowErrors = FALSE;
	}

	return FALSE;
}

void CQuincyApp::InvalidateAllViews()
{
	POSITION tpos = GetFirstDocTemplatePosition();
	while (tpos != 0)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CTextDocument* pDoc = static_cast<CTextDocument*>(pTem->GetNextDoc(dpos));
			ASSERT(pDoc != 0);
			POSITION vpos = pDoc->GetFirstViewPosition();
			ASSERT(vpos != 0);
			CTextView* pView = static_cast<CTextView*>(pDoc->GetNextView(vpos));
			ASSERT(pView != 0);
			pView->Invalidate();
		}
	}
}

BOOL CQuincyApp::TestProgramChanged()
{
	if (m_bBypassChangeTest)
		return FALSE;

	POSITION tpos = GetFirstDocTemplatePosition();
	while (tpos != 0)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CTextDocument* pDoc = static_cast<CTextDocument*>(pTem->GetNextDoc(dpos));
			ASSERT(pDoc != 0);
			CString strDocFile = GetFileName(pDoc->GetPathName());
			if (strDocFile.Right(4).CompareNoCase(".prj") != 0)	{
				POSITION vpos = pDoc->GetFirstViewPosition();
				ASSERT(vpos != 0);
				CTextView* pView = static_cast<CTextView*>(pDoc->GetNextView(vpos));
				ASSERT(pView != 0);
				CEdit& rEdit = pView->GetEditCtrl();
				if (rEdit.GetModify())	{
					if (AfxMessageBox("Program has changed. Continue?", MB_ICONQUESTION | MB_YESNO) == IDNO)	{
						OnStop();
						return TRUE;
					}
					m_bBypassChangeTest = TRUE;
					break;
				}
			}
		}
	}
	return FALSE;
}

BOOL CQuincyApp::IsProjectFileLoaded()
{
	POSITION tpos = GetFirstDocTemplatePosition();
	while (tpos != 0)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CTextDocument* pDoc = static_cast<CTextDocument*>(pTem->GetNextDoc(dpos));
			ASSERT(pDoc != 0);
			CString strDocFile = GetFileName(pDoc->GetPathName());
			if (strDocFile.Right(4).CompareNoCase(".prj") == 0)
				return TRUE;
		}
	}
	return FALSE;
}
void CQuincyApp::OnStop()
{
	if (CompileRunning())
		m_CompileStatus = aborting;
	else if (m_pDebugger != 0)
		m_pDebugger->Stop();
}

void CQuincyApp::OnOptions() 
{
	COptionsDialog dlgOptions;
	dlgOptions.m_CommandLinePromptOption = m_CommandLinePromptOption;
	dlgOptions.m_strCommandLine = m_strCommandLine;
	dlgOptions.m_strRuntimeDirectory = m_strRuntimeDirectory;
	GetArray(dlgOptions.m_strDefine, m_Defines);
	GetArray(dlgOptions.m_strInclude, m_Includes);
	GetArray(dlgOptions.m_strLib, m_Libs);
	dlgOptions.m_bDebugging = m_bDebugging;
	dlgOptions.m_bExceptions = m_bExceptions;
	dlgOptions.m_bRTTI = m_bRTTI;
	dlgOptions.m_bStrict = m_bStrict;
	dlgOptions.m_nOptimize = m_nOptimize;
	if (dlgOptions.DoModal() == IDOK)	{
		m_CommandLinePromptOption = dlgOptions.m_CommandLinePromptOption;
		m_strCommandLine = dlgOptions.m_strCommandLine;
		m_strRuntimeDirectory = dlgOptions.m_strRuntimeDirectory;
		if (!m_strRuntimeDirectory.IsEmpty() && m_strRuntimeDirectory[m_strRuntimeDirectory.GetLength()-1] != '\\')
			m_strRuntimeDirectory += '\\';
		LoadArray(m_Defines,  dlgOptions.m_strDefine);
		LoadArray(m_Includes, dlgOptions.m_strInclude);
		LoadArray(m_Libs,     dlgOptions.m_strLib);
		m_bDebugging = dlgOptions.m_bDebugging;
		m_bExceptions = dlgOptions.m_bExceptions;
		m_bRTTI = dlgOptions.m_bRTTI;
		m_bStrict = dlgOptions.m_bStrict;
		m_nOptimize = dlgOptions.m_nOptimize;
		CQuincyDoc* pDoc = GetProjectDocument();
		if (pDoc)
			pDoc->SetModifiedFlag();
	}
}

void CQuincyApp::OnCompiler() 
{
	dlgCompiler dlg;
	dlg.m_strCompiler = m_strCompiler;
	dlg.m_strWorking = m_strWorkingPath;
	if (dlg.DoModal() == IDOK)	{
		m_strCompiler = dlg.m_strCompiler;
		m_strWorkingPath = dlg.m_strWorking;
		int len = m_strCompiler.GetLength();
		if (len && m_strCompiler[len-1] != '\\')
			m_strCompiler += '\\';
		len = m_strWorkingPath.GetLength();
		if (len && m_strWorkingPath[len-1] != '\\')
			m_strWorkingPath += '\\';
		SetCompilerPaths();
	}
}

// load an array of option strings from an option string ("option1;option2;...) 
void CQuincyApp::LoadArray(CStringArray& array, const CString& rstr)
{
	int len = rstr.GetLength();
	char ch[] = " ";
	CString str;
	array.RemoveAll();	// start with an empty array
	for (int i = 0; i < len; i++)	{	// iterate the input string
		*ch = rstr[i];					// one char at a time
		if (*ch != ' ')	{				// ignore spaces
			// --- option terminates with ';' or at end of string
			if (*ch == ';' || i == len-1)	{
				if (*ch != ';')
					str += ch;
				if (!str.IsEmpty())	{
					array.Add(str);
					str.Empty();
				}
			}
			else
				str += ch;	// build option string
		}
	}
}

// make an options string ("option1;option2;...) from an array of option strings
void CQuincyApp::GetArray(CString& atr, const CStringArray& array) const
{
	int len = array.GetSize();
	atr = "";
	for (int i = 0; i < len; i++)	{
		atr += array[i];
		if (i < len-1)
			atr += ";";
	}
}

int CQuincyApp::ExitInstance() 
{
	m_bWatchCreated = FALSE;
	m_bTraceCreated = FALSE;
	WriteProfileInt("Options", "Debugging", m_bDebugging);
	WriteProfileInt("Options", "Exceptions", m_bExceptions);
	WriteProfileInt("Options", "RTTI", m_bRTTI);
	WriteProfileInt("Options", "Strict", m_bStrict);
	WriteProfileInt("Options", "Command Line Prompt", m_CommandLinePromptOption);
	WriteProfileInt("Options", "Optimize", m_nOptimize);
	WriteProfileString("Options", "Command Line", m_strCommandLine);
	WriteProfileInt("Options", "UseStdLib", m_bUseStdLib);

#ifdef TYCPP
	WriteProfileInt("Tutorial",    "Displayed", m_bTutorialDisplayed );
	WriteProfileString("Tutorial", "Current", m_strTutorial);
#endif
	WriteProfileString("Directories", "WorkingPath",  m_strWorkingPath);
	WriteProfileString("Directories", "Runtime", m_strRuntimeDirectory);
	WriteProfileString("Directories", "Compiler",     m_strCompiler);

	CString strBuffer;
	strBuffer.Format ("%i:%i:%i:%i",
			m_ConsoleRect.left,
			m_ConsoleRect.top,
			m_ConsoleRect.right,
			m_ConsoleRect.bottom);

	AfxGetApp ()->WriteProfileString ("Settings", "Console Position", strBuffer);


	char path[MAX_PATH];
	_getcwd(path, MAX_PATH);
	WriteProfileString("Directories", "Current", path);

#ifdef TYCPP
	DdeFreeStringHandle(idInst, hszAppName);
#endif
	return CWinApp::ExitInstance();
}

// ------ display the compiler error message log
void CQuincyApp::OnShowErrorlog() 
{
	ASSERT(m_pdlgErrors != 0);
	if (m_bErrorCreated != TRUE)	{
		m_pdlgErrors->Create(IDD_ERRORLOG);
		m_bErrorCreated = TRUE;
	}
	CListBox* pList = static_cast<CListBox*>(m_pdlgErrors->GetDlgItem(IDC_ERRORLIST));
	ASSERT(pList != 0);
	pList->ResetContent();

	CDC* pDC = pList->GetDC();
	int wd = 0;

	for (UINT i = 0; i < m_nErrorCount; i++)	{
		CString& strErr = ErrorLog[i].m_strErrorMsg; 
		pList->AddString(strErr);
		CSize size = pDC->GetTextExtent(strErr);
		wd = max(wd, size.cx);
	}
	pList->SendMessage(LB_SETHORIZONTALEXTENT, wd, 0);
	m_pdlgErrors->ShowWindow(SW_SHOW);
}

bool CQuincyApp::SelectFileAndLine(const CString& strFile, int nline)
{
	if (!strFile.IsEmpty())	{
		CDocument* pDoc = OpenDocumentFile(strFile);
		if (pDoc != 0)	{	// (user could delete the source code file before reading it)
			// ---- change focus to the selected document file
			POSITION dpos = pDoc->GetFirstViewPosition();
			ASSERT(dpos != 0);
			CEditView* pView = static_cast<CEditView*>(pDoc->GetNextView(dpos));
			ASSERT(pView != 0);
			pView->SetFocus();
			if (nline != 0)	{
				// ---- position text so that the specified line is current
				CEdit& rEdit = pView->GetEditCtrl();
				int nPos = rEdit.LineIndex(nline-1);
				rEdit.SetSel(nPos, nPos);
				ShowLineColumn(nline, 1);
			}
			pView->UpdateWindow();
			return true;
		}
	}
	return false;
}

void CQuincyApp::SelectErrorLine(int nsel)
{
	if (nsel != -1 && m_nErrorCount > 0)
		SelectFileAndLine(ErrorLog[nsel].m_strFileSpec, ErrorLog[nsel].m_nLineNumber);
}

void CQuincyApp::SelectGrepLine(int nsel)
{
	if (nsel != -1 && m_nGrepCount > 0)
		SelectFileAndLine(GrepLog[nsel].m_strFileSpec, GrepLog[nsel].m_nLineNumber);
}

void CQuincyApp::OnUpdateShowErrorlog(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_nErrorCount > 0);
}

void CQuincyApp::OnUpdateStop(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable(CompileRunning() || 
		(m_pDebugger != 0 && m_pDebugger->CanStop()));
}

void CQuincyApp::SaveAllDocuments()
{
	POSITION tpos = GetFirstDocTemplatePosition();
	while (tpos != 0)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CDocument* pDoc = pTem->GetNextDoc(dpos);
			ASSERT(pDoc != 0);
			if (pDoc->IsModified())	{
				const CString& strPath = pDoc->GetPathName();
				if (!strPath.IsEmpty())	{
					pDoc->OnSaveDocument(strPath);
					pDoc->SetModifiedFlag(FALSE);
				}
			}
		}
	}
}

void CQuincyApp::SaveOpenDocuments()
{
	POSITION tpos = GetFirstDocTemplatePosition();
	int nDocno = 0;
	CString strProfile;
	while (tpos != 0 && nDocno < maxsaveddocuments)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CQuincyDoc* pDoc = static_cast<CQuincyDoc*>(pTem->GetNextDoc(dpos));
			ASSERT(pDoc != 0);
			CString strPath = pDoc->GetPathName();
			strProfile.Format("Document%d", ++nDocno);
			WriteProfileString("Documents", strProfile, strPath);

			CTextView* pView = GetTextView(strPath);
			ASSERT(pView != 0);
			CWnd* pWnd = pView->GetParent();
			ASSERT(pWnd != NULL);
			WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
			if (pWnd->GetWindowPlacement(&wndpl))	{
				strProfile.Format("Document%d Maximized", nDocno);
				WriteProfileInt("Documents", strProfile, 
					wndpl.showCmd == SW_SHOWMAXIMIZED);

			}
		}
	}
	while (nDocno < maxsaveddocuments)	{
		strProfile.Format("Document%d", ++nDocno);
		WriteProfileString("Documents", strProfile, 0);
		strProfile.Format("Document%d Maximized", nDocno);
		WriteProfileString("Documents", strProfile, 0);
	}
}

void CQuincyApp::SaveDialogWindowPosition(const CString& strWindow, CWnd* pWnd)
{
	WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
	if (pWnd->m_hWnd != 0 && pWnd->GetWindowPlacement(&wndpl))	{
		RECT rc = wndpl.rcNormalPosition;
		WriteProfileInt(strWindow, "Left", rc.left);
		WriteProfileInt(strWindow, "Top",  rc.top);
	}
}

void CQuincyApp::RestoreDialogWindowPosition(const CString& strWindow, CWnd* pWnd)
{
	WINDOWPLACEMENT wndpl = { sizeof(WINDOWPLACEMENT) };
	if (pWnd->m_hWnd != 0)	{
		pWnd->GetWindowPlacement(&wndpl);
		int wd = wndpl.rcNormalPosition.right - wndpl.rcNormalPosition.left;
		int ht = wndpl.rcNormalPosition.bottom - wndpl.rcNormalPosition.top;
		wndpl.rcNormalPosition.left = 
			GetProfileInt(strWindow, "Left", wndpl.rcNormalPosition.left);
		wndpl.rcNormalPosition.top = 
			GetProfileInt(strWindow, "Top", wndpl.rcNormalPosition.top);
		wndpl.rcNormalPosition.right = wndpl.rcNormalPosition.left + wd;
		wndpl.rcNormalPosition.bottom = wndpl.rcNormalPosition.top + ht;
		pWnd->SetWindowPlacement(&wndpl);
	}
}

// ------- permit clearing all the breakpoints
void CQuincyApp::OnUpdateDebugClearbreakpoints(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable(breakpoints.size() > 0);
	InvalidateAllViews();
}

// ------- clear all the breakpoints
void CQuincyApp::OnDebugClearbreakpoints() 
{
	breakpoints.clear();
}

void CQuincyApp::BringToTop()
{
	ASSERT(m_pMainWnd != 0);
	m_pMainWnd->BringWindowToTop();
	m_pMainWnd->SetForegroundWindow();
	m_pMainWnd->Invalidate();
	m_pMainWnd->UpdateWindow();
}
// ----- the run command
BOOL CQuincyApp::ExecuteProgram()
{
	if (m_pDebugger != 0)
		return m_pDebugger->ExecuteProgram();
	return FALSE;
}
// ------- the step command
BOOL CQuincyApp::StepProgram()
{
	if (m_pDebugger != 0)
		return m_pDebugger->StepProgram();
	return FALSE;
}
// ------- the step out command
BOOL CQuincyApp::StepOut()
{
	if (m_pDebugger != 0)
		return m_pDebugger->StepOut();
	return FALSE;
}
// ----- the step over command
void CQuincyApp::OnDebugStepover() 
{
	if (m_pDebugger == 0 || m_pDebugger->StepOver() == FALSE)
		m_pMainWnd->PostMessage(WM_COMMAND, ID_DEBUG_STEP);
}
// ------ disable stepping over if no document is open
void CQuincyApp::OnUpdateDebugStepover(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}
// ------ permit examining
void CQuincyApp::OnUpdateDebugExamine(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pDebugger && m_pDebugger->OKToExamine());
}
// ------ permit watching
void CQuincyApp::OnUpdateDebugAddwatch(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}
// ------ examine command
void CQuincyApp::OnDebugExamine() 
{
	CExamineDialog dlgExamine;
	dlgExamine.DoModal();
}
// ------- get the value of a variable: used by watch and examine processes
void CQuincyApp::GetVariableValue(const CString& strVarName, CString& strVarValue)
{
	if (m_pDebugger == 0 || m_pDebugger->GetStabs() == 0)
		strVarValue = "??";
	else	{
		Item item = m_pParser->EvaluateExpression(strVarName);
		item.BuildValueDisplay(strVarValue);
	}
}
void CQuincyApp::SetVariableValue(const CString& strVarName, const CString& strVarValue)
{
	if (strVarValue.IsEmpty())
		return;
	Item item = m_pParser->EvaluateExpression(strVarName);
	if (m_pDebugger)
		item.ChangeMemoryVariable(m_pDebugger->ProcessHandle(), strVarValue);
}

void CQuincyApp::CreateWatchWindow()
{
	ASSERT(m_pdlgWatch != 0);
	if (m_bWatchCreated != TRUE)	{
		m_pdlgWatch->Create(IDD_WATCH);
		m_bWatchCreated = TRUE;
	}
}

void CQuincyApp::RefreshAllWatches()
{
	if (m_bWatchCreated) 
		m_pdlgWatch->RefreshAllWatches();
}

///////////////// breakpoint methods //////////////////////////

// ------ adjust breakpoints when user deletes line
void CQuincyApp::DeletedTextLines(const CString& strFile, int nLineno, int nLineCount)
{
	////////////////////////////////////////////
}
// ------ adjust breakpoints when user inserts line
void CQuincyApp::InsertedTextLines(const CString& strFile, int nLineno, int nLineCount)
{
}

// --- set or clear a breakpoint. return true = set, false = clear
bool CQuincyApp::ToggleBreakpoint(const CString& strFile, int nLineNo)
{
	FileReference fr(strFile, nLineNo);
	std::set<FileReference>::iterator iter = breakpoints.find(fr);
	if (iter == breakpoints.end())	{
		breakpoints.insert(fr);
		return true;
	}
	breakpoints.erase(iter);
	return false;
}

void CQuincyApp::AddBreakpoint(const CString& strFile, int nLineNo)
{
	FileReference fr(strFile, nLineNo);
	std::set<FileReference>::iterator iter = breakpoints.find(fr);
	if (iter == breakpoints.end())
		breakpoints.insert(fr);
}

// ------ test for a breakpoint set in a file at a line number
bool CQuincyApp::IsBreakpoint(const CString& strFile, int nLineno)
{
	FileReference fr(strFile, nLineno);
	std::set<FileReference>::iterator iter = breakpoints.find(fr);
	return iter != breakpoints.end();
}

void CQuincyApp::OnDebugWatch() 
{
	CreateWatchWindow();
	m_pdlgWatch->ShowWindow(SW_SHOW);
	m_pdlgWatch->SetFocus();
	if (m_pdlgWatch->GetWatchCount() == 0)
		m_pdlgWatch->AddWatch();
}

void CQuincyApp::OnDebugAddwatch() 
{
	CreateWatchWindow();
	m_pdlgWatch->AddWatch();
	m_pdlgWatch->ShowWindow(SW_SHOW);
	m_pdlgWatch->SetFocus();
}

void CQuincyApp::StopStepping()
{
	if (m_pDebugger != 0)
		m_pDebugger->StopStepping();
}

BOOL CQuincyApp::StopDebugging()
{
	if (m_pDebugger != 0)	{
		if (AfxMessageBox("This stops the debugger. Continue?", MB_ICONQUESTION | MB_YESNO) == IDYES)	{
			m_pDebugger->Stop();
			m_bClosing = TRUE;
		}
		return FALSE;
	}
	return TRUE;
}

void CQuincyApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.m_strEgcsVersion = "Mingw32 version: " + m_strVersion;
	aboutDlg.DoModal();
}

#ifdef TYCPP
void CQuincyApp::OnUpdateTycpp(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bIsTutorial);
}
#endif

void CQuincyApp::OnHelp() 
{
	::WinHelp(m_pMainWnd->m_hWnd, m_pszHelpFilePath, HELP_FINDER, 0);
}

void CQuincyApp::OnResourceeditor() 
{
	RunResourceEditor();
}
void CQuincyApp::OnUpdateResourceeditor(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bWEditResIsInstalled);
}

void CQuincyApp::OnUpdateUsestdlib(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bStdLibIsInstalled);
}

void CQuincyApp::OnFileOpen() 
{
	static char BASED_CODE szFilter[] = 
			"Quincy files (*.cpp,*.c,*.h,*.prj)"
			"|*.cpp; *.c; *.h; *.prj|"
			"All files (*.*)"
			"|*.*|"
			"|";

	CFileDialog dlg(TRUE,
					0,
					0,
					OFN_HIDEREADONLY	| 
					OFN_FILEMUSTEXIST	| 
//					OFN_NOCHANGEDIR		|
					OFN_PATHMUSTEXIST, 
					szFilter,
					m_pMainWnd
					);

	if (dlg.DoModal() == IDOK)
		OpenDocumentFile(dlg.GetPathName());
}

void CQuincyApp::OnDebugTrace() 
{
	ASSERT(m_pdlgTrace != 0);
	if (m_bTraceCreated != TRUE)	{
		m_pdlgTrace->Create(IDD_TRACEDIALOG);
		m_bTraceCreated = TRUE;
	}
	m_pdlgTrace->ShowWindow(SW_SHOW);
}

void CQuincyApp::OnUseStdLib() 
{
	m_bUseStdLib = ToggleMenuCommand(ID_USESTDLIB);
}

bool CQuincyApp::ToggleMenuCommand(int id)
{
	CMenu* pCMenu = ((CMainFrame*)(m_pMainWnd))->GetMenu();
	UINT mstate = pCMenu->GetMenuState(id, MF_BYCOMMAND);
	if ((mstate & MF_CHECKED) != 0)
		pCMenu->CheckMenuItem(id, MF_UNCHECKED);
	else
		pCMenu->CheckMenuItem(id, MF_CHECKED);
	return (mstate & MF_CHECKED) == 0;
}

void CQuincyApp::SetMenuCommand(int id)
{
	CMenu* pCMenu = ((CMainFrame*)(m_pMainWnd))->GetMenu();
	pCMenu->CheckMenuItem(id, MF_CHECKED);
}
void CQuincyApp::ResetMenuCommand(int id)
{
	CMenu* pCMenu = ((CMainFrame*)(m_pMainWnd))->GetMenu();
	pCMenu->CheckMenuItem(id, MF_UNCHECKED);
}

