// -------- tutorial.cpp

#include "stdafx.h"
#include <ddeml.h>
#include "Quincy.h"
#include "QuincyDoc.h"
#include "TextDocument.h"
#include "WatchDialog.h"
#include "debugger.h"
#include "TutDlg.h"

#ifdef TYCPP
// ----- setup a tutorial based on the contents of a tutorial profile (.tut) file in .INI format
void CQuincyApp::ReadTutorial()
{
	OnStop();
	m_bReadTutorial = FALSE;

	POSITION pos = GetFirstDocTemplatePosition();
	while (pos != 0)	{
		CDocTemplate* pTemplate = GetNextDocTemplate(pos);
		POSITION dpos = pTemplate->GetFirstDocPosition();
		while (dpos != 0)	{
			CDocument* pDoc = pTemplate->GetNextDoc(dpos);
			if (pDoc->IsModified())	{
				CString strTitle("Save changes to ");
				strTitle += pDoc->GetTitle();
				strTitle += "?";
				int rtn;
				if ((rtn = AfxMessageBox(strTitle, MB_YESNOCANCEL)) == IDYES)	{
					CString strPath = pDoc->GetPathName();
					pDoc->OnSaveDocument(strPath);
				}
				else if (rtn == IDCANCEL)
					return;
			}
		}
	}

	CloseAllDocuments(FALSE);
	
	CString strTutorial = m_strTutorial;
	CString strProfile;
	CString strSourcePath;
	// ------ change to the subdirectory of the .tut file
	//        (that's where the source code files should be)
	int offset = strTutorial.ReverseFind('\\');
	if (offset > 0)	{
		strSourcePath = strTutorial.Left(offset+1);
		_chdir(strSourcePath);
	}

	// ------ load the document files
	int i;
	int nDocct = GetPrivateProfileInt("Documents", "Docct", 0, strTutorial);
	for (i = 0; i < nDocct; i++)	{
		strProfile.Format("Doc%02d", i + 1);
		char path[MAX_PATH];
		GetPrivateProfileString("Documents", strProfile, "", path, MAX_PATH, strTutorial);
		if (*path)	{
			CString strOpenPath = strSourcePath + path;
			OpenDocumentFile(strOpenPath);
			if (nDocct == 1)
				MaximizeDocument(path);
		}
	}

	// ------ load the watches
	if (m_bWatchCreated)
		m_pdlgWatch->ClearWatches();
	int nWatchct = GetPrivateProfileInt("Watches", "Watchct", 0, strTutorial);
	if (nWatchct > 0)	{
		CreateWatchWindow();
		m_pdlgWatch->ShowWindow(SW_SHOW);
	}
	else if (m_bWatchCreated)
		m_pdlgWatch->ShowWindow(SW_HIDE);
	for (i = 0; i < nWatchct; i++)	{
		strProfile.Format("Watch%02d", i + 1);
		char watch[120];
		GetPrivateProfileString("Watches", strProfile, "", watch, 120, strTutorial);
		m_pdlgWatch->AddWatch(watch);
	}

	ClearBreakpoints();
	// ----- load the breakpoints
	int nBreakpointct = GetPrivateProfileInt("Breakpoints", "Breakpointct", 0, strTutorial);
	for (i = 0; i < nBreakpointct; i++)	{
		strProfile.Format("Breakpoint%02d", i + 1);
		char breakpoint[120];
		GetPrivateProfileString("Breakpoints", strProfile, "", breakpoint, 120, strTutorial);
		char* npLineno = strchr(breakpoint, '/');
		if (npLineno != 0)	{
			*npLineno = '\0';
			int nLineno = atoi(npLineno+1);
			if (nLineno > 0)
				theApp.AddBreakpoint(breakpoint, nLineno);

		}
	}

	// ---- load the runtime options
	m_bDebugging = GetPrivateProfileInt("Options", "Debugging", 1, strTutorial);
	m_bExceptions = GetPrivateProfileInt("Options", "Exceptions", 0, strTutorial);
	m_bRTTI = GetPrivateProfileInt("Options", "RTTI", 0, strTutorial);
	m_bStrict = GetPrivateProfileInt("Options", "Strict", 0, strTutorial);
	char line[257];
	line[256] = '\0';
	GetPrivateProfileString("Options", "Command Line", "", line, 256, strTutorial);
	m_strCommandLine = line; 
	m_CommandLinePromptOption = GetPrivateProfileInt("Options", "Command Line Prompt", 1, strTutorial);
	GetPrivateProfileString("Options", "Define", "", line, 256, strTutorial);
	LoadArray(m_Defines, line); 

	m_bUseStdLib = GetPrivateProfileInt("Options", "UseStdLib", 0, strTutorial);

	if (m_bUseStdLib && m_bStdLibIsInstalled)
		SetMenuCommand(ID_USESTDLIB);
	else
		ResetMenuCommand(ID_USESTDLIB);

	m_pMainWnd->SetForegroundWindow();
}

// ----- the Win32 API forgot to include this function
static void WritePrivateProfileInt(LPCTSTR lpapp, LPCTSTR lpkey, int nVal, LPCTSTR lpFileName)
{
	CString strVal;
	strVal.Format("%d", nVal);
	WritePrivateProfileString(lpapp, lpkey, strVal, lpFileName);
}

void CQuincyApp::WriteTutorial(const CString& strTutorial)
{
	m_strTutorial = strTutorial;

	CString strProfile;
	CString strDocument;

	// ------- write the documents
	int nDocct = 0;
	POSITION tpos = GetFirstDocTemplatePosition();
	while (tpos != 0)	{
		CDocTemplate* pTem = GetNextDocTemplate(tpos);
		ASSERT(pTem != 0);
		POSITION dpos = pTem->GetFirstDocPosition();
		while (dpos != 0)	{
			CTextDocument* pDoc = static_cast<CTextDocument*>(pTem->GetNextDoc(dpos));
			ASSERT(pDoc != 0);
			strDocument = pDoc->GetPathName();
			strProfile.Format("Doc%02d", ++nDocct);
			WritePrivateProfileString("Documents", strProfile, GetFileName(strDocument), strTutorial);
		}
	}
	WritePrivateProfileInt("Documents", "Docct", nDocct, strTutorial);

	int i;
	// ------- write the watches
	int nWatchct = m_bWatchCreated ? m_pdlgWatch->GetWatchCount() : 0;
	WritePrivateProfileInt("Watches", "Watchct", nWatchct, strTutorial);
	for (i = 0; i < nWatchct; i++)	{
		strProfile.Format("Watch%02d", i + 1);
		CString strWatch;
		m_pdlgWatch->GetWatchValue(i, strWatch);
		WritePrivateProfileString("Watches", strProfile, strWatch, strTutorial);
	}

	// ------- write the breakpoints
	WritePrivateProfileInt("Breakpoints", "Breakpointct", theApp.BreakpointCount(), strTutorial);

	const std::set<FileReference>& bp = theApp.GetProgrammedBreakpoints();
	std::set<FileReference>::const_iterator iter;
	i = 0;
	for (iter = bp.begin(); iter != bp.end(); iter++, i++)	{
		strProfile.Format("Breakpoint%02d", i + 1);
		CString strBreakpoint;
		strBreakpoint.Format("%s/%d", (*iter).m_strFile, (*iter).m_nLineNo);
		WritePrivateProfileString("Breakpoints", strProfile, strBreakpoint, strTutorial);
	}
	// ----- write the runtime options
	WritePrivateProfileInt("Options", "Debugging", m_bDebugging, strTutorial);
	WritePrivateProfileInt("Options", "Exceptions", m_bExceptions, strTutorial);
	WritePrivateProfileInt("Options", "RTTI", m_bRTTI, strTutorial);
	WritePrivateProfileInt("Options", "Strict", m_bStrict, strTutorial);
	WritePrivateProfileInt("Options", "Command Line Prompt", m_CommandLinePromptOption, strTutorial);
	WritePrivateProfileString("Options", "Command Line", m_strCommandLine, strTutorial);
	CString strArray;
	GetArray(strArray, m_Defines);
	WritePrivateProfileString("Options", "Define",  strArray, strTutorial);

	WritePrivateProfileInt("Options", "UseStdLib", m_bUseStdLib, strTutorial);
}

void CQuincyApp::OnWritetutorial()
{
	// ------ try using original tutorial name (if one was loaded)
	CString strTutorial(m_strTutorial);
	if (strTutorial.IsEmpty())	{
		// ------ try to develop a file name from project file
		GetProjectDocument(&strTutorial);
		if (strTutorial.IsEmpty())	{
			// ------ try to develop a file name from first available source code file
			POSITION tpos = GetFirstDocTemplatePosition();
			while (tpos != 0)	{
				CDocTemplate* pTem = GetNextDocTemplate(tpos);
				ASSERT(pTem != 0);
				POSITION dpos = pTem->GetFirstDocPosition();
				if (dpos != 0)	{
					CTextDocument* pDoc = static_cast<CTextDocument*>(pTem->GetNextDoc(dpos));
					ASSERT(pDoc != 0);
					strTutorial = pDoc->GetPathName();
					break;
				}
			}
		}
	}
	if (!strTutorial.IsEmpty())	{
		int nIndex = strTutorial.ReverseFind('.');
		if (nIndex != -1)
			strTutorial = strTutorial.Left(nIndex) + ".tut";
	}
	CFileDialog dlgTut(FALSE, "tut", strTutorial, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, 
		"Tutorial Files (*.tut)|*.tut|All Files (*.*)|*.*||");
	if (dlgTut.DoModal() == IDOK)
		WriteTutorial(dlgTut.GetPathName());
}

BOOL CQuincyApp::IsTutorial(const CString& strFileName)
{
	return strFileName.Right(4).CompareNoCase(".tut") == 0;
}

static struct DispatchTable {
	char* m_pCommand;
	int m_CommandID;
} dispatchtable[] = {
	{ "step",			ID_DEBUG_STEP },
	{ "stepover",		ID_DEBUG_STEPOVER },
	{ "steptocursor",	ID_DEBUG_STEPTOCURSOR },
	{ "stop",			ID_STOPBUILD },
	{ "run",			ID_EXECUTE },
};

void CQuincyApp::DispatchTutorialCommand(const CString& strCommand)
{
	for (int i = 0; i < sizeof(dispatchtable) / sizeof(DispatchTable); i++)	{
		if (strCommand.CompareNoCase(dispatchtable[i].m_pCommand) == 0)	{
			m_pMainWnd->PostMessage(WM_COMMAND, dispatchtable[i].m_CommandID);
			break;
		}
	}
}

BOOL CQuincyApp::OnDDECommand(LPTSTR lpszCommand) 
{
	int nLen = strlen(lpszCommand);
	CString strTutorial(lpszCommand);
	if (strTutorial.Left(6) == "[open(")	{
		strTutorial = strTutorial.Mid(7, nLen - 10);
		if (IsTutorial(strTutorial))	{
			m_bReadTutorial = TRUE;
			m_strTutorial = strTutorial;
			m_strWorkingPath.Empty();
			return TRUE;
		}
	}
	else if (strTutorial.Left(10) == "[exepath(\"")	{
		m_strWorkingPath = strTutorial.Right(strTutorial.GetLength()-10);
		m_strWorkingPath = m_strWorkingPath.Left(m_strWorkingPath.GetLength()-3);
	}
	else if (strTutorial.Left(10) == "[command(\"")	{
		int nIndex = strTutorial.ReverseFind('.');
		if (nIndex > 10)	{
			CString strCommand = strTutorial.Mid(10, nIndex-10);
			nIndex = strCommand.ReverseFind('\\');
			if (nIndex != -1)
				strCommand = strCommand.Right(strCommand.GetLength() - nIndex - 1);
			DispatchTutorialCommand(strCommand);
		}
		return TRUE;
	}
	return CWinApp::OnDDECommand(lpszCommand);
}

extern DWORD idInst;                 // our DDEML instance object
extern HSZ   hszAppName;             // the generic hsz for everything
extern UINT  OurFormat;                  // our custom registered format
static int count = 0;                  // our data
static int InCount = 0;                // static buffer to hold incomming data

HDDEDATA CALLBACK DdeCallback(
WORD wType,
WORD wFmt,
HCONV hConv,
HSZ hszTopic,
HSZ hszItem,
HDDEDATA hData,
DWORD lData1,
DWORD lData2)
{
    LPTSTR pszExec;

    switch (wType) {
    case XTYP_CONNECT:
        return((HDDEDATA)TRUE);

    case XTYP_ADVREQ:
        /*
         * These two transactions are the only ones that require us to
         * render our data.  By using a custom format, we don't have to
         * convert our count to text form to support CF_TEXT.
         */
        return(DdeCreateDataHandle(idInst, (PBYTE)&count, sizeof(count), 0,
                hszAppName, OurFormat, 0));

    case XTYP_ADVSTART:
        /*
         * Only allow links to our Item in our format.
         */
        return(HDDEDATA) ((UINT)wFmt == OurFormat && hszItem == hszAppName);

    case XTYP_EXECUTE:
        /*
         * Another instance wants us to do something.  DdeAccessData()
         * makes parsing of execute strings easy.  Also note, that DDEML
         * will automatically give us the string in the right form
         * (UNICODE vs ASCII) depending on which form of DdeInitialize()
         * we called.
         */
        pszExec = (LPTSTR)DdeAccessData(hData, NULL);
        if (pszExec)
			theApp.OnDDECommand(pszExec);
		DdeUnaccessData(hData);
		return (HDDEDATA)DDE_FACK;

    case XTYP_REGISTER:
        /*
         * Since a new server just arrived, lets make sure our links are
         * up to date.  Note that only one link on a
         * conversation/topic/item/format set will work anyway so we don't
         * worry about duplicate links.
         *
         * Note also that we are using hszItem - which is the InstanceSpecific
         * name of the server that is registering.  This greatly reduces the
         * number of messages that go flying around.
         */
        return((HDDEDATA)TRUE);
    }
    return(0);
}

void CQuincyApp::OnTycpp() 
{
	CreateTutorialDialog();
	m_pCTutDlg->ShowWindow(SW_SHOW);
	m_pCTutDlg->SetFocus();
	if (!m_strTutorial.IsEmpty())
		m_pCTutDlg->OpenExercise(m_strTutorial);
}

void CQuincyApp::CreateTutorialDialog()
{
	ASSERT(m_pCTutDlg != 0);
	if (m_bTutCreated != TRUE)	{
		m_pCTutDlg->Create(IDD_TUTORIAL);
		m_bTutCreated = TRUE;
	}
}

void CQuincyApp::LoadTutorials()
{
	m_strTutorialPath = GetProfileString("Directories", "Tutorial");
	if (m_strTutorialPath.IsEmpty())
		m_strTutorialPath = m_strBinPath.Left(m_strBinPath.GetLength() - 3) + "Programs";
	CString strTocPath = m_strTutorialPath + "\\TYCPP.TOC";
	m_bIsTutorial = _access(strTocPath, 0) == 0;
	if (m_bIsTutorial)
		m_pCTutDlg = new CTutDlg(strTocPath);
}

void CQuincyApp::OpenTutorial(const CString& strTut)
{
	if (strTut.Left(7) == "Program")	{
		// --- extract the chapter number from the exercise title
		char cch[3];
		cch[0] = strTut[8];
		cch[1] = strTut[9];
		cch[2] = '\0';
		int chapno = atoi(cch);

		// --- extract the exercise number from the title
		int ndx = 9;
		if (strTut[ndx] != '.')
			ndx++;
		cch[0] = strTut[++ndx];
		cch[1] = strTut[++ndx];
		int exno = atoi(cch);

		CString strTutorial;
		strTutorial.Format("%s%02d\\PR%02d%03d.tut", m_strTutorialPath + "\\Chap",
			chapno, chapno, exno);
		m_bReadTutorial = TRUE;
		m_strTutorial = strTutorial;
	}
}
#endif
