// ------- breakpoint.h

#ifndef BREAKPOINT_H
#define BREAKPOINT_H

#include <map>

typedef long MemoryAddress;

struct FileReference	{
	CString m_strFile;
	int m_nLineNo;
	FileReference(CString fname = CString(), int lno = 0) : m_strFile(fname), m_nLineNo(lno)	{ }
	bool operator<(const FileReference& fr) const
		{ return (m_strFile == fr.m_strFile) ? (m_nLineNo < fr.m_nLineNo) : (m_strFile < fr.m_strFile); }
	bool operator==(const FileReference& fr) const
		{ return (m_strFile == fr.m_strFile) ? (m_nLineNo == fr.m_nLineNo) : (m_strFile < fr.m_strFile); }
	FileReference& operator=(const FileReference& fr)
	{
		if (this != &fr)	{
			m_strFile = fr.m_strFile;
			m_nLineNo = fr.m_nLineNo;
		}
		return *this;
	}
};

// ----- breakpoint structure
struct Breakpoint	{
	FileReference m_FileRef;			// source code file, line number
	unsigned char m_cSaveInst;			// op code of instruction to be broken
	Breakpoint(FileReference fref = FileReference()) : m_FileRef(fref), m_cSaveInst(0)
		{ }
	bool operator<(const Breakpoint& bp) const { return m_FileRef < bp.m_FileRef; }
	bool operator==(const Breakpoint& bp) const { return m_FileRef == bp.m_FileRef; }
	Breakpoint& operator=(const Breakpoint& bp);
};

class RuntimeBreakpoints	{
	typedef std::map<MemoryAddress, Breakpoint> bp_map;
public:
	typedef bp_map::iterator bp_iterator;
private:
	bp_map cntr;
	void SetBreakpoint(MemoryAddress addr, Breakpoint& bp, HANDLE hProcess, bool bSet);
public:
	void LoadBreakpoint(const FileReference& fref, MemoryAddress addr);
	void SetBreakpoint(MemoryAddress addr, HANDLE hProcess, bool bSet = true);
	void SetAllBreakpoints(HANDLE hProcess, bool bSet = true);
	Breakpoint* FindBreakpoint(MemoryAddress addr);
	void ClearAllBreakpoints(HANDLE hProcess);
#ifdef _DEBUG
	void DumpBreakpoints();
#endif
};

#endif

