// TextDocument.cpp : implementation file
//

#include "stdafx.h"
#include "Quincy.h"
#include "TextDocument.h"
#include "debugger.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTextDocument

IMPLEMENT_DYNCREATE(CTextDocument, CDocument)

CTextDocument::CTextDocument()
{
}

CTextDocument::~CTextDocument()
{
}

BEGIN_MESSAGE_MAP(CTextDocument, CDocument)
	//{{AFX_MSG_MAP(CTextDocument)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_UPDATE_COMMAND_UI(ID_DELETE_FILE, OnUpdateDeleteFile)
	ON_UPDATE_COMMAND_UI(ID_INSERT_FILE, OnUpdateInsertFile)
	ON_UPDATE_COMMAND_UI(ID_EXECUTE, OnUpdateExecute)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_BREAKPOINT, OnUpdateDebugBreakpoint)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_ADDWATCH, OnUpdateDebugAddwatch)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_STEP, OnUpdateDebugStep)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_STEPOVER, OnUpdateDebugStepover)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_STEPTOCURSOR, OnUpdateDebugSteptocursor)
	ON_UPDATE_COMMAND_UI(ID_DEBUG_STEPOUTOFFUNCTION, OnUpdateDebugStepoutoffunction)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTextDocument diagnostics

#ifdef _DEBUG
void CTextDocument::AssertValid() const
{
	CDocument::AssertValid();
}

void CTextDocument::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTextDocument serialization

void CTextDocument::Serialize(CArchive& ar)
{
	POSITION pos = GetFirstViewPosition();
	ASSERT(pos != NULL);
	CEditView* pView = static_cast<CEditView*>(GetNextView(pos));
	ASSERT(pView != NULL);
	pView->SerializeRaw(ar);
	if (ar.IsStoring())
		ar.Flush();
	else	{
		if (theApp.GetProjectDocument() == 0)
			theApp.ClearBreakpoints();
		CEdit& rEdit = pView->GetEditCtrl();
		CString strin, strnew;
		rEdit.GetWindowText(strin);
		const char* cp = strin.GetBuffer(0);
		const char* cp1 = cp;
		const char* cpOrig = cp;
		while (cp != 0)	{
			cp = strchr(cp, '\n');
			if (cp != 0)	{
				if (cp == cpOrig || *(cp-1) != '\r')	{
					int first = cp1-cpOrig;
					int count = cp - cp1;
					CString sstr = strin.Mid(first, count);
					sstr += "\r\n";
					strnew += sstr;
					cp1 = cp+1;
				}
				cp++;
			}
		}
		if (!strnew.IsEmpty())
			rEdit.SetWindowText(strnew);
	}
}

// --- files you can compile
BOOL CTextDocument::IsSourceFile()
{
	CString& rstrPath = const_cast<CString&>(GetPathName());
	return rstrPath.Right(2).CompareNoCase(".c")   == 0 ||
		   rstrPath.Right(3).CompareNoCase(".rc")  == 0 ||
		   rstrPath.Right(4).CompareNoCase(".cpp") == 0;
}

// --- files you can set breakpoints in
BOOL CTextDocument::IsExecutableSourceFile()
{
	CString& rstrPath = const_cast<CString&>(GetPathName());
	return rstrPath.Right(2).CompareNoCase(".c")   == 0 ||
		   rstrPath.Right(2).CompareNoCase(".h")   == 0 ||
		   rstrPath.Right(4).CompareNoCase(".cpp") == 0;
}

/////////////////////////////////////////////////////////////////////////////
// CTextDocument commands

void CTextDocument::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable(TRUE);	
}

void CTextDocument::OnUpdateDeleteFile(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable(FALSE);	
}

void CTextDocument::OnUpdateInsertFile(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable(FALSE);	
}

void CTextDocument::OnUpdateExecute(CCmdUI* pCmdUI) 
{
	if (theApp.CompileRunning() || !theApp.CanCompile() || theApp.IsLibraryTarget() || theApp.IsDLLTarget())	{
		pCmdUI->Enable(FALSE);
		return;
	}
	Debugger* pDebugger = theApp.GetDebugger(FALSE);
	if (pDebugger != 0)	{
		pCmdUI->Enable((pDebugger->isBroken() || pDebugger->isStepping()) && !pDebugger->IsInProgram());
		return;
	}
	pCmdUI->Enable(theApp.GetProjectView() || IsSourceFile());
}

void CTextDocument::OnUpdateDebugBreakpoint(CCmdUI* pCmdUI) 
{
	if (theApp.CompileRunning() || !IsExecutableSourceFile() || !theApp.CanCompile() || theApp.IsLibraryTarget() || theApp.IsDLLTarget())	{
		pCmdUI->Enable(FALSE);
		return;
	}
	Debugger* pDebugger = theApp.GetDebugger(FALSE);
	pCmdUI->Enable(pDebugger == 0 || ((pDebugger->isStepping() || pDebugger->isBroken()) && !pDebugger->IsInProgram()));
}

void CTextDocument::OnUpdateDebugStep(CCmdUI* pCmdUI) 
{
	OnUpdateDebugAddwatch(pCmdUI);
}

void CTextDocument::OnUpdateDebugStepover(CCmdUI* pCmdUI) 
{
	OnUpdateDebugAddwatch(pCmdUI);
}

void CTextDocument::OnUpdateDebugAddwatch(CCmdUI* pCmdUI) 
{
	if (theApp.CompileRunning() || !theApp.CanCompile() || theApp.IsLibraryTarget() || theApp.IsDLLTarget())	{
		pCmdUI->Enable(FALSE);
		return;
	}
	Debugger* pDebugger = theApp.GetDebugger(FALSE);
	pCmdUI->Enable(pDebugger == 0 || !pDebugger->IsInProgram());
}

void CTextDocument::OnUpdateDebugSteptocursor(CCmdUI* pCmdUI) 
{
	if (theApp.CompileRunning() || !IsExecutableSourceFile() || !theApp.CanCompile() || theApp.IsLibraryTarget() || theApp.IsDLLTarget())	{
		pCmdUI->Enable(FALSE);
		return;
	}
	Debugger* pDebugger = theApp.GetDebugger(FALSE);
	pCmdUI->Enable(pDebugger == 0 || !pDebugger->IsInProgram());
}

void CTextDocument::OnUpdateDebugStepoutoffunction(CCmdUI* pCmdUI) 
{
	if (theApp.GetDebugger(FALSE) == 0)
		pCmdUI->Enable(FALSE);
	else
		OnUpdateDebugAddwatch(pCmdUI);
}
