// TextView.cpp : implementation file
//

#include "stdafx.h"
#include "Quincy.h"
#include "TextView.h"
#include "QuincyView.h"
#include "TextDocument.h"
#include "debugger.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTextView

static const int margin = 1;				// margin for breakpoint and program counter carets
static const char breakpointchar = '>';		// breakpoint margin displaycharacter

IMPLEMENT_DYNCREATE(CTextView, CEditView)

CTextView::CTextView()
{
	m_nFontHeight = 13;
	m_nFontWidth = 7;
}

CTextView::~CTextView()
{
}

BEGIN_MESSAGE_MAP(CTextView, CEditView)
	//{{AFX_MSG_MAP(CTextView)
	ON_COMMAND(ID_BUILD, OnBuild)
	ON_COMMAND(ID_BUILDALL, OnBuildall)
	ON_COMMAND(ID_COMPILE, OnCompile)
	ON_COMMAND(ID_EXECUTE, OnExecute)
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_CHAR()
	ON_COMMAND(ID_DEBUG_BREAKPOINT, OnDebugBreakpoint)
	ON_COMMAND(ID_DEBUG_STEP, OnDebugStep)
	ON_COMMAND(ID_DEBUG_STEPTOCURSOR, OnDebugSteptocursor)
	ON_UPDATE_COMMAND_UI(ID_BUILD, OnUpdateBuild)
	ON_UPDATE_COMMAND_UI(ID_BUILDALL, OnUpdateBuildAll)
	ON_UPDATE_COMMAND_UI(ID_COMPILE, OnUpdateCompile)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_ALL, OnUpdateFileSaveAll)
	ON_WM_TIMER()
	ON_COMMAND(ID_DEBUG_STEPOUTOFFUNCTION, OnDebugStepoutoffunction)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTextView diagnostics

#ifdef _DEBUG
void CTextView::AssertValid() const
{
	CEditView::AssertValid();
}

void CTextView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTextView message handlers

BOOL CTextView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	BOOL rtn = CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
	SetTabStops(16);

	newfont.CreateFont(m_nFontHeight,m_nFontWidth,0,0,400,0,0,0,0,1,2,1,49,"Courier");

	SetFont(&newfont, TRUE);

	CDC* pCDC = GetDC();

	pCDC->SetMapMode(MM_ISOTROPIC);

	pCDC->SetWindowExt(78, 78);
	pCDC->SetViewportExt(96, 96);

	CSize size = pCDC->GetTextExtent("ABCD", 4);

	m_nFontHeight = size.cy;
	m_nFontWidth = size.cx / 4;

	CEdit& rEdit = GetEditCtrl();
	rEdit.SetMargins(m_nFontWidth * margin + m_nFontWidth / 2, 0);
	return rtn;
}

void CTextView::OnBuildall() 
{
	// ----- if a project document is open, send build command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->SendMessage(WM_COMMAND, ID_BUILDALL, 0);
}

void CTextView::OnBuild() 
{
	// ----- if a project document is open, send build command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->SendMessage(WM_COMMAND, ID_BUILD, 0);
	else
		DoCompile(TRUE, FALSE);
}

BOOL CTextView::DoCompile(BOOL bDoLink, BOOL bDoExecute, BOOL bDoStep) 
{
	theApp.StopStepping();

	CDocument* pDoc = GetDocument();
	ASSERT(pDoc != 0);

	CString& rstrPath = const_cast<CString&>(pDoc->GetPathName());
	if (pDoc->IsModified())
		pDoc->OnSaveDocument(rstrPath);

/*
	// ----- change to the subdirectory where source file is located
	int offset = rstrPath.ReverseFind('\\');
	if (offset != -1)	{
		CString strPath = rstrPath.Left(offset);
		_chdir(strPath);
	}
*/
	// ----- make a copy of the file spec without the extension;
	BOOL isCpp = rstrPath.Right(4).CompareNoCase(".cpp") == 0;
	CString strSpec = rstrPath.Left(rstrPath.GetLength()-(isCpp ? 4 : 2));

	// ---- drop the path
//	CString strSpc = theApp.GetFileName(strSpec);
//	CString strExeSpec = theApp.GetFileName(strSpec) + ".exe";
	CString strExeSpec = strSpec + ".exe";

	// ---- queue the source and object files for compile/link
	theApp.ClearArrays();
	theApp.AddSourceFile(rstrPath);
	if (bDoLink)
//		theApp.AddObjectFile(strSpc + ".o");
		theApp.AddObjectFile(strSpec + ".o");
//	theApp.BuildTarget(strSpec + ".exe", 
	theApp.BuildTarget(strExeSpec, 
		bDoExecute ? (bDoStep ? CQuincyApp::step : CQuincyApp::execute) : CQuincyApp::none);

	return TRUE;
}

void CTextView::OnCompile()
{
	DoCompile(FALSE, FALSE);
}

void CTextView::OnDebugStep() 
{
	// ----- if a project document is open, send step command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->PostMessage(WM_COMMAND, ID_DEBUG_STEP, 0);
	else if (!theApp.StepProgram())
		BuildAndRunProgram(TRUE);
}

void CTextView::OnDebugStepoutoffunction() 
{
	theApp.StepOut();
}

void CTextView::OnExecute() 
{
	// ----- if a project document is open, send execute command to its view
	CQuincyView* pView = theApp.GetProjectView();
	if (pView != 0)
		pView->PostMessage(WM_COMMAND, ID_EXECUTE, 0);
	else if (!theApp.ExecuteProgram())
		BuildAndRunProgram(FALSE);
}

void CTextView::BuildAndRunProgram(BOOL bStep)
{
	CDocument* pDoc = GetDocument();
	ASSERT(pDoc != 0);
	CString& rstrPath = const_cast<CString&>(pDoc->GetPathName());
	// ----- make a copy of the file spec with the .EXE extension;
	BOOL isCpp = rstrPath.Right(4).CompareNoCase(".cpp") == 0;
	CString strExe = rstrPath.Left(rstrPath.GetLength()-(isCpp ? 4 : 2)) + ".exe";

	// ---- execute the .exe associated with the .c or .cpp source code document
	int rtn;
	if (pDoc->IsModified())	{
		CString strMsg = theApp.GetFileName(rstrPath) + " has been modified. Build?";
		if ((rtn = AfxMessageBox(strMsg, MB_YESNOCANCEL | MB_ICONQUESTION)) == IDYES)
			DoCompile(TRUE, TRUE, bStep);
		else if (rtn == IDNO && _access(strExe, 0) == 0)	{
			theApp.SetBypassChangeTest();
			theApp.RunApplicationProgram(strExe, bStep);
		}
	}
	else if (_access(strExe, 0) != 0)	{
		CString strMsg = theApp.GetFileName(strExe) + " does not exist. Build?";
		if (AfxMessageBox(strMsg, MB_YESNO | MB_ICONQUESTION) == IDYES)
			DoCompile(TRUE, TRUE, bStep);
	}
	else if (theApp.CompareFileTimes(rstrPath, strExe) > 0)	{
		CString strMsg = theApp.GetFileName(strExe) + " out of date. Rebuild?";
		if ((rtn = AfxMessageBox(strMsg, MB_YESNOCANCEL | MB_ICONQUESTION)) == IDYES)
			DoCompile(TRUE, TRUE, bStep);
		else if (rtn == IDNO)
			theApp.RunApplicationProgram(strExe, bStep);
	}
	else
		theApp.RunApplicationProgram(strExe, bStep);
}
// ----- current zero-based text line number
int CTextView::CurrentLineNumber()
{
	CEdit& rEdit = GetEditCtrl();
	int nLineIndex = rEdit.LineIndex(-1);		// character index of 1st char, current line
	return rEdit.LineFromChar(nLineIndex) + 1;	// current line number
}
// ----- current text column
int CTextView::CurrentColumn()
{
	CEdit& rEdit = GetEditCtrl();
	int nLineIndex = rEdit.LineIndex(-1);	// character index of 1st char, current line
	char buf[200];
	rEdit.GetLine(rEdit.LineFromChar(nLineIndex), buf, 200);

	int nStartChar, nEndChar;
	rEdit.GetSel(nStartChar, nEndChar );		// character index of current position
	int col = 0;
	int tabct = 0;
	for (int x = 0; x < nEndChar - nLineIndex; x++)	{
		if (x == 200)
			break;
		if (buf[x] == '\t')
			while (++col % 4)
				tabct++;
		else
			col++;
	}
	return (nEndChar - nLineIndex) + tabct + 1;		// current column number
}
// ------ display line and column number in the status bar
void CTextView::ShowLineColumn()
{
	theApp.ShowLineColumn(CurrentLineNumber(), CurrentColumn());
}
// ---- use this message to update line, column in the status bar and to manage
//      breakpoints when the user inserts and deletes lines of text
void CTextView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == VK_RETURN)	{
		int nColumn = CurrentColumn();
		int nLineno = CurrentLineNumber();
		CEditView::OnKeyDown(nChar, nRepCnt, nFlags);
		theApp.InsertedTextLines(SourceCodeFileName(), 
			(nColumn == 1 ? nLineno : nLineno+1), 1);
	}
	else if (nChar == VK_DELETE)	{
		CEdit& rEdit = GetEditCtrl();
		int nBefore = rEdit.GetLineCount();
		int nColumn = CurrentColumn();
		CEditView::OnKeyDown(nChar, nRepCnt, nFlags);
		int nAfter = rEdit.GetLineCount();
		if (nBefore - nAfter == 1)	{
			int nLineno = CurrentLineNumber();
			theApp.DeletedTextLines(SourceCodeFileName(), 
				(nColumn == 1 ? nLineno : nLineno + 1), 1);
		}
	}
	else
		CEditView::OnKeyDown(nChar, nRepCnt, nFlags);
	ShowLineColumn();
}
void CTextView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CEditView::OnChar(nChar, nRepCnt, nFlags);
	ShowLineColumn();
}
void CTextView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	if (bActivate)
		ShowLineColumn();
	else
		theApp.ShowLineColumn(0, 0);
	CEditView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}
void CTextView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CEditView::OnLButtonDown(nFlags, point);
	ShowLineColumn();
}
void CTextView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CEditView::OnMouseMove(nFlags, point);
	if (nFlags & MK_LBUTTON)
		ShowLineColumn();
}
// ----- return the document file name without the path
CString CTextView::SourceCodeFileName()
{
	CTextDocument* pDoc = (CTextDocument*)GetDocument();
	ASSERT(pDoc != 0);
	return pDoc->GetPathName();
}
// ----- called to set or clear a breakpoint in this source code file
void CTextView::OnDebugBreakpoint() 
{
	// ---- get the name of the source code file
	CString strFile = GetDocument()->GetPathName();
	// ----- get the current line number from the source code file
	int nLineno = CurrentLineNumber();		// current line number
	bool bSet = theApp.ToggleBreakpoint(strFile, nLineno);
	SetBreakpointDisplay(nLineno, bSet);
}
// ----- writes a character in the editor's left margin 
void CTextView::WriteMarginCharacter(int nLine, int ch, COLORREF clr)
{
	char chr[] = "   ";
	chr[0] = ch;
	HideCaret();
	CDC* pCDC = GetDC();
	CFont font;
	font.CreateFont(m_nFontHeight,m_nFontWidth,0,0,FW_HEAVY,0,0,0,0,1,2,1,49,"Courier");
	CFont* oldfont = pCDC->SelectObject(&font);
	pCDC->SetTextColor(clr);
	pCDC->TextOut(0, nLine, chr, 1);
	pCDC->SelectObject(oldfont);
	ReleaseDC(pCDC);
	ShowCaret();
}
// ---- compute the window's client area line number from the text line number
int CTextView::RelativeLine(int nLineno)
{
	CEdit& rEdit = GetEditCtrl();
	int nTopLine = rEdit.GetFirstVisibleLine();
	return (nLineno - 1) - nTopLine;
}
// ----- retrieve the line of text specified by the zero-based line number
//       the retrieved line is the length of the client area
//       the retrieved line is tab-expanded
//       if the line number is greater than the number of lines in the edit control,
//           return a blank line
//       width of line is written into caller's second argument space
//       return the address of the text on the heap
//       caller must delete the heap buffer
LPSTR CTextView::GetTextLine(int nLineno, int& nWidth)
{
	RECT rc;
	GetClientRect(&rc);

	// ---- approximate number of character positions in client area
	nWidth = (rc.right * 100 / 785) - margin - 1;

	int nCharIndex = 0;
	int nLineLength = 0;
	int nDisplayLength = nWidth;

	CEdit& rEdit = GetEditCtrl();

	int nLineCount = rEdit.GetLineCount();

	LPTSTR lpText = 0;
	LPTSTR lpExpText = 0;

	if (nLineno < nLineCount)	{
		nCharIndex = rEdit.LineIndex(nLineno);
		nLineLength = rEdit.LineLength(nCharIndex);
		nDisplayLength = max(nLineLength, nWidth);

		lpText = new char[nDisplayLength];
		memset(lpText, ' ', nDisplayLength);

		rEdit.GetLine(nLineno, lpText, nLineLength);

		// ------- count tab expansion characters in line
		int nTabExpansion = 0;
		for (int i = 0; i < nLineLength; i++)
			if (lpText[i] == '\t')
				nTabExpansion += (i % 4);

		nDisplayLength = max(nLineLength + nTabExpansion, nWidth);
	}

	lpExpText = new char[nDisplayLength+1];
	memset(lpExpText, ' ', nDisplayLength);
	lpExpText[nDisplayLength] = '\0';

	if (nLineno < nLineCount)	{
		// ----- expand tabs in text line for display
		int j = 0;
		int i = 0;
		while (i < nLineLength && j < nDisplayLength)	{
			if (lpText[i] == '\t')	{
				do	{
					lpExpText[j] = ' ';
					if (++j == nDisplayLength)
						break;
				} while (j % 4);
				i++;
			}
			else
				lpExpText[j++] = lpText[i++];
		}
	}

	delete [] lpText;
	return lpExpText;
}
// ---- set or clear a breakpoint caret in the margin
void CTextView::SetBreakpointDisplay(int nLineno, BOOL bOnOff)
{
	int nRelativeLine = RelativeLine(nLineno);
	if (nRelativeLine >= 0)
		WriteMarginCharacter(nRelativeLine * m_nFontHeight, bOnOff ? breakpointchar : ' ', 0xff);
}
// ---- set or clear the program counter cursor bar
void CTextView::SetProgramCounterDisplay(int nLineno, BOOL bOnOff)
{
	int nRelativeLine = RelativeLine(nLineno);
	if (nRelativeLine >= 0)	{
		HideCaret();
		CDC* pCDC = GetDC();
		int nWidth;
		LPTSTR lpText = GetTextLine(nLineno-1, nWidth);

		int nScrollPos = GetScrollPos(SB_HORZ);

		if (nScrollPos)	{
			CDC* pMemCDC = new CDC;
			pMemCDC->CreateCompatibleDC(pCDC);
			CBitmap* pBitmap = new CBitmap;
			pBitmap->CreateCompatibleBitmap(pCDC, nWidth*m_nFontWidth, m_nFontHeight);
			CBitmap* pOld = pMemCDC->SelectObject(pBitmap);

			pMemCDC->SelectObject(&newfont);
			if (bOnOff)
				pMemCDC->SetBkColor(0x00ff00);

			pMemCDC->TextOut(0, 0, lpText, nWidth);

			CPoint src(nScrollPos, 0);
			pMemCDC->DPtoLP(&src);

			CPoint dest(m_nFontWidth*margin+4, nRelativeLine*m_nFontHeight+1);
			CPoint dim(nWidth*m_nFontWidth-6, m_nFontHeight);

			pCDC->BitBlt(dest.x, dest.y, dim.x, dim.y, pMemCDC, src.x, src.y, SRCCOPY);

			pMemCDC->SelectObject(pOld);
			delete pBitmap;
			delete pMemCDC;
		}
		else	{
			pCDC->SelectObject(&newfont);
			if (bOnOff)
				pCDC->SetBkColor(0x00ff00);
			CRect rc;
			GetClientRect(&rc);
			pCDC->ExtTextOut(m_nFontWidth*margin+4, 
							nRelativeLine*m_nFontHeight+1, 
							ETO_CLIPPED,
							&rc,
							lpText,
							nWidth,
							0
							);
		}
		delete [] lpText;
		ReleaseDC(pCDC);
		ShowCaret();
	}
}
// ----- can't use OnPaint for this for some reason
LRESULT CTextView::DefWindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	LRESULT rtn = CEditView::DefWindowProc(message, wParam, lParam);
	if (message == WM_PAINT || message == WM_HSCROLL)	{
		// ---- get the name of the source code file
		CString strFile = SourceCodeFileName();

		// ----- compute top and bottom lines of the client window
		CEdit& rEdit = GetEditCtrl();
		int nLine = rEdit.GetFirstVisibleLine();
		RECT rc;
		GetClientRect(&rc);
		int nBottomLine = nLine + rc.bottom / m_nFontHeight;

		// --- set flag if this file is the current debugged source code file
		BOOL bSrc = FALSE;
		Debugger* pDebugger = theApp.GetDebugger(FALSE);
		if (pDebugger != 0)	{
			const CString* pCurrentSrcFile = pDebugger->CurrentSrcFile();
			if (pCurrentSrcFile != 0)
				bSrc = strFile.CompareNoCase(*pCurrentSrcFile) == 0;
		}

		// ---- iterate the lines, displaying (or clearing) 
		//      breakpoint margin carets and the program counter cursor bar
		do	{
			if (bSrc && nLine+1 == theApp.GetDebugger()->CurrentLineNo())
				SetProgramCounterDisplay(nLine+1, TRUE);
			bool bBrk = theApp.IsBreakpoint(strFile, nLine+1);
			SetBreakpointDisplay(nLine+1, bBrk);
		} while (++nLine < nBottomLine);
	}
	return rtn;
}

void CTextView::OnDebugSteptocursor() 
{
	if (!theApp.TestProgramChanged())	{
		int nLineNo = CurrentLineNumber();
		CString strFile = SourceCodeFileName();
		Debugger::StepTo(strFile, nLineNo);
		OnExecute();
	}
}

void CTextView::OnUpdateBuild(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanBuild() && (theApp.IsProjectFileLoaded() || IsSourceFile()));
}

void CTextView::OnUpdateBuildAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanBuild() && theApp.IsProjectFileLoaded());
}

void CTextView::OnUpdateCompile(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(CanBuild() && IsSourceFile());
}

BOOL CTextView::CanBuild()
{
	Debugger* pDebugger = theApp.GetDebugger(FALSE);
	return !theApp.CompileRunning() && pDebugger == 0 && theApp.CanCompile();
}
BOOL CTextView::IsSourceFile()
{
	CTextDocument* pDoc = (CTextDocument*)GetDocument();
	ASSERT(pDoc != 0);
	return pDoc->IsSourceFile();
}

void CTextView::OnUpdateFileSaveAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(TRUE);
}


