// ------- bass.cpp

#include "stdafx.h"
#include "bass.h"
#include "song.h"
#include "options.h"

void Bass::StartPlaying()
{
	Initialize();
	lastnote = 0;
	m_bNewChord = FALSE;
	m_nPlayingLine = 0;
	m_nLine = 0;
	m_nLineVariation = 0;
	m_bSwingingTune = FALSE;
	if ((theSong.m_Key.attr & CChord::minor) == 0) 	{
		int nMeasureCt = theSong.MeasureCount();
		Song::Style nStyle = theSong.CurrentStyle(nMeasureCt - 1);
		if (nStyle == Song::fourfour || nStyle == Song::ballad)	{
			int nChordCt = theSong.ChordCount();
			for (int i = nChordCt - 8; i < nChordCt; i++)
				if (theSong.m_Key.root != theSong.GetChord(i).root)
					break;
			if (i == nChordCt)
				m_bSwingingTune = TRUE;
		}
	}
}

// ------ bass lines for 2 chord types
const int measures = 8;
static short int lines[][measures][4][3] = {
	// --- major & minor
	{{ {1},     {3},  {5},  {8}      }, 
	 { {1},     {8},  {5},  {8,5,3}  },
	 { {1},     {5},  {6},  {7}      },
	 { {8,0,8}, {6},  {5},  {3}      },
	 { {17},    {16}, {15}, {14}     },
	 { {13,8,5},{3},  {1},  {-5}     },
	 { {1},     {13}, {8},  {5,0,-5} },
	 { {1},     {3},  {5},  {8}      }},
	// --- 7th, augmented, diminished, & minor 7th
	{{ {1},     {5},  {6},  {7}      },
	 { {8,0,8}, {6},  {5},  {3}      },
	 { {1},     {-2}, {-5}, {-8}     },
	 { {1},     {3},  {5},  {8}      },
	 { {13},    {8},  {5},  {8}      },
	 { {1},     {-2}, {-5}, {-8}     },
	 { {17},    {16}, {15}, {14}     },
	 { {13,8,5},{3},  {1},  {-5}     }}
};
static short int chrd;

// ---- these are for playbacks where the program can look ahead to the next chord
//      n changes with 3 variations of up to 4 notes 
//      these notes are relative to the first note which is relative
//      to the first chord in the sequence
static short int pline[][3][4] = {
	// ---- tonic to dominant in 4 beats
	{{1,3,4,5},{1,4,8,7},{1,-2,-3,-5,}},
	// ---- tonic to dominant 2 beats
	{{1,7,0,0},{1,-6,0,0,},{1,5,0,0,}},
	// ---- major or minor to 6 major (e.g. C to A or Cmi to A) or major to 6 minor (e.g. C to Ami) in 2 beats
	{{1,-2,0,0},{1,-8,0,0,},{1,5,0,0,}},
	// ---- major to 6 in 4 beats
	{{1,1,-1,-2},{1,1,-1,-2,},{1,1,5,-8,}},
	// ---- major to 7th (e.g. C to Bb) in 2 beats
	{{1,-5,0,0,},{1,-1,0,0},{1,7,0,0,}},
	// ---- major to 2nd (e.g. C to D) in 2 beats
	{{1,2,0,0,},{1,-5,0,0},{1,1,0,0,}},
	// ---- minor to 6 minor (e.g. Cmi to Ami) in 2 beats
	{{1,-2,0,0},{1,-8,0,0,},{1,5,0,0,}},
};

inline int Bass::RandomGraceCounter()
{
	return 4 + (rand() % 16);
}

Bass::Bass(MidiOut* pMidiOut) : midiOutDevice("Bass", 2, 32, pMidiOut)
{
	lastnote = 0;
	m_bNewChord = FALSE;
	m_nPlayingLine = 0;
	m_nLine = 0;
	m_nLineVariation = 0;
	m_nGraceCounter = RandomGraceCounter();
}

void Bass::TimingMessage(int measure, int beat, int clock)
{
	if (theSong.IsRest(measure, beat))	{
		LastNoteOff();
		return;
	}
	if (beat < theSong.m_nBeatsPerMeasure-1 && theSong.IsRest(measure, beat+1))
		if (clock != 0)
			return;

	// -- build a chord offset into the lines table
	chrd = 0;
	if ((theSong.m_Chord.attr & CChord::seventh) || 
			(theSong.m_Chord.attr & CChord::augmented) || 
				(theSong.m_Chord.attr & CChord::diminished))
		chrd = 1;

	BOOL bSwingingAlong = theSong.m_nTempo < 190 &&
						  (theSong.m_nStyle == Song::fourfour || 
		                   theSong.m_nStyle == Song::ballad) && 
			              !theSong.IsLastMeasure() && 
						  !theSong.IsNextToLastMeasure();
	short int* plines = &lines[chrd][measure&7][beat][1];
	BOOL bInTriplets =  *plines || *(plines+1);
	plines = &lines[chrd][measure&7][beat+1][1];
	BOOL bNextInTriplets = *plines || *(plines+1);

	switch (clock)	{
		case 0:				// on the beat
			if (theSong.m_Chord.root)	{
				m_bNewChord = lastchord != theSong.m_Chord;
				switch (theSong.m_nStyle)	{
					case Song::halftime:
					case Song::dixie:
					case Song::country:
						if ((m_bNewChord && theSong.IsPlayback()) || (beat & 1) == 0)
							PlayBassNote(measure, beat);
						break;
					case Song::fourfour:
					case Song::ballad:
					case Song::jazzwaltz:
					case Song::shuffle:
						PlayBassNote(measure, beat);
						break;
					case Song::rock:
					case Song::rockballad:
					case Song::bossa:
					case Song::easybossa:
						if ((beat & 1) == 0 || (beat == 3 && (measure & 1)))
							PlayBassNote(measure, beat);
						break;
					case Song::waltz:
						if (m_bNewChord || (beat & 1) == 0)
							PlayBassNote(measure, beat);
					default:
						break;
				}
			}
			break;
		case 16:

			if (bSwingingAlong && !bInTriplets && !bNextInTriplets)	{
				// --- play a random grace note
				if (m_nGraceCounter-- == 0)	{
					m_nGraceCounter = RandomGraceCounter();
					if (beat == 3 && lastnote != 0)	{
						NoteOff(lastnote, 0);
						NoteOn(lastnote, 100);
					}
					break;
				}
			}

			// fall through
		case 8:
			if (!theSong.m_Chord.root)
				break;

			if (m_nPlayingLine || (theSong.m_Chord.root & 0xf0))
				break;
			if (bSwingingAlong)	{
				// --- bass triplets
				if (bInTriplets)
					PlayBassNote(measure, beat, clock/8);
				break;
			}
			if (clock == 8)
				break;
			if (theSong.m_nStyle == Song::shuffle)	{
				if (theSong.IsLastMeasure())
					PlayBassNote(measure, beat);
				else	{
					// shuffle plays a grace note, which repeats the previous note played
					NoteOff(lastnote, 0);
					NoteOn(lastnote, 70);
				}
				break;
			}
			if (theSong.IsPlayback() && m_bSwingingTune)	{
				if (beat < 2 && theSong.IsLastMeasure())
					PlayNote(beat == 0 ? -3 : 1);
				else if (beat == 0 && theSong.IsNextToLastMeasure())
					PlayNote(-8, 127);
				break;
			}
			break;
		case 12:
			if (lastnote == 0)
				break;
			if ((theSong.m_nStyle == Song::bossa || 
					theSong.m_nStyle == Song::rock || 
						theSong.m_nStyle == Song::rockballad))	{
				// rock, rockballad, & bossa play an 8th note, which repeats the previous note played
				if ((beat&1) == 1)	{
					if (beat == 1 || (measure&1) == 0)
						NoteOn(lastnote, 70);
				}
			}
			break;
		default:
			break;
	}
}

void Bass::PlayBassNote(int measure, int beat, int third)
{
	if (m_nPlayingLine == -1)
		m_nPlayingLine = 0;
	lastchord = theSong.m_Chord;
	if (theSong.IsPlayback() && third == 0)	{
		int bass = (theSong.m_Chord.root >> 4) & 0xf;
		if (bass)	{
			// --- overriding bass note ( e.g. C7/G )
			int root = theSong.m_Chord.root&0xf;
			int note = bass+32;
			if (bass > root)
				note -= 12;
			PlayNoteAbsolute(note);
			m_bNewChord = FALSE;
			return;
		}
		// --- this chord does not have an override; look at the next chord
		int nChord = measure * theSong.m_nBeatsPerMeasure + (beat < 2 ? 2 : theSong.m_nBeatsPerMeasure);
		if (theSong.GetChord(nChord).root & 0xf0)	{
			// --- must be a walking pattern, play the root for this one
			m_bNewChord = FALSE;
			PlayNote(1);
			return;
		}
	}

	short int relative = 0;
	// --- select a relative (to root) note to play

	if (theSong.IsPlayback() && m_bSwingingTune && theSong.IsNextToLastMeasure())	{
		m_nPlayingLine = 0;
		switch (beat)	{
			case 0:
				relative = 1;
				break;
			case 1:
				return;
			case 2:
				relative = -7;
				break;
			case 3:
				relative = -6;
				break;
			default:
				break;
		}
	}
	else if (theSong.IsLastMeasure())	{
		m_nPlayingLine = 0;
		if (theSong.IsWaltz())	{
			if (beat)
				return;
			relative = -12;
		}
		else if (theSong.m_nStyle == Song::shuffle)	{
			if (beat > 1)
				return;
			relative = 1;
		}
		else	{
			if (theSong.IsPlayback() && m_bSwingingTune)	{
				switch (beat)	{
					case 0:
						relative = -5;
						break;
					case 1:
						relative = -1;
						break;
					case 2:
						return;
					case 3:
						relative = -12;
						break;
					default:
						break;
				}
			}
			else	{
				switch (beat)	{
					case 0:
						relative = 1;
						break;
					case 1:
						relative = -5;
						break;
					case 2:
						relative = -12;
						break;
					case 3:
						return;
					default:			// take 5?
						break;
				}
			}
		}
	}
	else	{
		if (beat < theSong.m_nBeatsPerMeasure-1 && theSong.IsRest(measure, beat+1))
			relative = 1;
		else	{
			int ms = measure & 3;
			switch (theSong.m_nStyle)	{
				case Song::ballad:
				case Song::fourfour:
					if (m_nPlayingLine > 0)	{
						relative = pline[m_nLine][m_nLineVariation][m_nPlayingLine++];
						if (m_nPlayingLine == 4 || pline[m_nLine][m_nLineVariation][m_nPlayingLine] == 0)
							m_nPlayingLine = -1;
						break;
					}
					if (theSong.IsPlayback())	{
						// --- look ahead from this chord to the next
						if ((theSong.m_Chord.attr & (CChord::diminished | CChord::augmented)) == 0)	{
							int root = theSong.m_Chord.root & 0xf;
							int nChord = measure * theSong.m_nBeatsPerMeasure + beat;
							if (beat == 0)	{
								// -- are the next three chords the same as this one?
								for (int i = 1; i < 4; i++)	{
									const CChord& rChord = theSong.GetChord(nChord + i);
									if ((rChord.root & 0xf0) || theSong.m_Chord != rChord)
										break;
								}
								if (i == 4)	{
									// --- get the chord being resolved to
									const CChord& rChord = theSong.GetChord(nChord + 4);
									if ((rChord.root & 0xf0) == 0)	{
										int rroot = rChord.root & 0xf;
										// --- resolving to the dominant?
										if (rroot == root + 5 || rroot == root - 7)	{
											SelectLine(0);
											relative = 1;
											break;
										}
										// --- resolving to the 6th?
										if (rroot == root + 9 || rroot == root - 3)	{
											SelectLine(3);
											relative = 1;
											break;
										}
									}
								}
							}
							if (beat == 0 || beat == 2)	{
								// -- is the next chord the same as this one?
								const CChord& rThisChord = theSong.GetChord(nChord + 1);
								if (theSong.m_Chord == rThisChord && (rThisChord.root & 0xf0) == 0)	{
									// --- get the chord being resolved to
									const CChord& rNextChord = theSong.GetChord(nChord + 2);
									if ((rNextChord.root & 0xf0) == 0)	{
										int rroot = rNextChord.root & 0xf;
										// --- resolving to the dominant?
										if (rroot == root + 5 || rroot == root - 7)	{
											SelectLine(1);
											relative = 1;
											break;
										}
										// --- resolving to the 6th?
										if (rroot == root + 9 || rroot == root - 3)	{
											SelectLine(((rThisChord.attr & CChord::minor) == 0) || ((rNextChord.attr & CChord::minor) == 0) ? 2 : 6);
											relative = 1;
											break;
										}
										// --- resolving to the 7th?
										if (rroot == root + 10 || rroot == root - 2)	{
											SelectLine(4);
											relative = 1;
											break;
										}
										// --- resolving from to the 2nd going up?
										if (rroot == root + 2 || rroot == root - 10)	{
											SelectLine(5);
											relative = 1;
											break;
										}
									}
								}
							}
						}
					}
					if (m_bNewChord)	{
						// if chord root has changed since last note, play the root of the new chord
						m_nPlayingLine = -1;
						m_nLine = 0;
						relative = 1;
						break;
					}
					relative = lines[chrd][measure%8][beat][third];
					break;
				case Song::jazzwaltz:
				case Song::waltz:
					relative = m_bNewChord ? 1 : lines[chrd][measure%8][beat][third];
					break;
				case Song::country:
				case Song::rock:
				case Song::rockballad:
				case Song::halftime:
				case Song::dixie:
					if (m_bNewChord)	{
						// if chord root has changed since last note, play the root of the new chord
						relative = 1;
						break;
					}
					if (beat == 3)	{
						relative = -5;
						break;
					}
					// fall through for beats 0 and 2
				case Song::bossa:
				case Song::easybossa:
				{
					// --- playing in half time or bossa, just walk the chord
					if (m_bNewChord)
						relative = 1;
					else if (beat == 0)	{
						switch (ms)	{
							case 0:
							case 2:
								relative = 1;
								break;
							case 1:
								relative = 8;
								break;
							default:
								relative = 5;
								break;
						}
					}
					else	{
						switch (ms)	{
							case 0:
							case 1:
								relative = 5;
								break;
							case 2:
								relative = -5;
								break;
							default:
								relative = 1;
								break;
						}
					}
					break;
				case Song::shuffle:
					switch (beat)	{
						case 0:
							relative = (!m_bNewChord && (ms & 1)) ? 11 : 1;
							break;
						case 1:
							relative = (!m_bNewChord && (ms & 1)) ? 10 : 5;
							break;
						case 2:
							relative = 8;
							break;
						case 3:
							relative = (!m_bNewChord && (ms & 1)) ? 5 : 10;
							break;
					}
					break;
				}
				default:
					break;
			}
		}
	}
	m_bNewChord = FALSE;

	if (m_nPlayingLine == 0)	{
		// --- adjust 3rd for minor or diminished
		if (((theSong.m_Chord.attr & CChord::minor) || (theSong.m_Chord.attr & CChord::diminished)) && 
				(relative == 5 || relative == -8))
			--relative;
		// --- adjust 7th for diminished
		else if ((theSong.m_Chord.attr & CChord::diminished) && (relative == 11 || relative == -2))
			--relative;
		else if (relative == 8 || relative == -5)	{
			// --- adjust 5th for augmented
			if (theSong.m_Chord.attr & CChord::augmented)
				relative++;
			// --- adjust 5th for flatted fifth or diminished
			else if ((theSong.m_Chord.attr & CChord::flat5) || theSong.m_Chord.attr & CChord::diminished)
				--relative;
		}
	}
	PlayNote(relative);
}

void Bass::SelectLine(int nLine)
{
	m_nLine = nLine;
	if (++m_nLineVariation == 3)
		m_nLineVariation = 0;
	m_nPlayingLine++;
}

void Bass::PlayNote(int relative, int velocity)
{
	// --- get the root of the current chord
	int note = (theSong.m_Chord.root&0xf) + 31;
	// --- adjust relative to preserve relative zero in lines table
	if (relative < 0)
		++relative;
	note += relative;
	PlayNoteAbsolute(note, velocity);
}

void Bass::PlayNoteAbsolute(int note, int velocity)
{
	LastNoteOff();
	if (note < 28)
		note += 12;
	if (note > 56)
		note -= 12;

	if (theOptions.m_bAdjustBass)
		note += 12;

	// --- play the note
	NoteOn(note, velocity);
	lastnote = note;
}

void Bass::LastNoteOff()
{
	if (lastnote)
		NoteOff(lastnote, 0);
}

