// --- drums.cpp

#include "stdafx.h"
#include "drums.h"

enum {
	accbassdrum = 35,	// 35
	bassdrum,			// 36
	sidekick,			// 37
	snare,				// 38
	handclap,			// 39
	elec_snare,			// 40
	lowfloortom,		// 41
	closedhihat,		// 42
	highfloortom,		// 43
	pedalhihat,			// 44
	lowtom,				// 45
	openhihat,			// 46
	lowmidtom,			// 47
	highmidtom,			// 48
	crash,				// 49
	hightom,			// 50
	ride,				// 51
	chinese,			// 52
	ridebell,			// 53
	tambourine,			// 54
	splash,				// 55
	cowbell,			// 56
	crash2,				// 57
	vibraslap,			// 58
	ride2,				// 59
	highbongo,			// 60
	lowbongo,			// 61
	mutehighconga,		// 62
	openhighconga,		// 63
	lowconga,			// 64
	hightimbale,		// 65
	lowtimbale,			// 66
	highagogo,			// 67
	lowagogo,			// 68
	cabasa,				// 69
	maracas,			// 70
	shortwhistle,		// 71
	longwhistle,		// 72
	shortguiro,			// 73
	longguiro,			// 74
	claves,				// 75
	highwoodblock,		// 76
	lowwoodblock,		// 77
	mutecuica,			// 78
	opencuica,			// 79
	mutetriangle,		// 80
	opentriangle		// 81
};

// implied number of styles, 2 measures, 4 beats (at most) 8 segments per beat (32nd note), up to 4 notes per hit
Note Drums::styles[][2][4][8][4] = {
	{								// style 0 (halftime)
		{							// measure 0
			//  segment 0                        |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6           |seg 7
/* beat 0 */{{{closedhihat,70},{accbassdrum,60}}},													// beat 0
/* beat 1 */{{{closedhihat,80},{sidekick,   70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,40}}}, // beat 1
/* beat 2 */{{{closedhihat,60},{accbassdrum,80}}},													// beat 2
/* beat 3 */{{{closedhihat,80},{sidekick,   80}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,80}}}, // beat 3
		},
		{							// measure 1
			//  segment 0                        |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6           |seg 7
/* beat 0 */{{{closedhihat,70},{accbassdrum,60}}},													// beat 0
/* beat 1 */{{{closedhihat,80},{sidekick,   70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,40}}}, // beat 1
/* beat 2 */{{{closedhihat,60},{accbassdrum,80}}},													// beat 2
/* beat 3 */{{{closedhihat,80},{sidekick,   80}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,80}}}, // beat 3
		},
	},
	{								// style 1 (4/4)
		{							// measure 0
			//  segment 0                                   |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6        |seg 7
/* beat 0 */{{{closedhihat,70},{accbassdrum,60},{ride,100}}},												// beat 0
/* beat 1 */{{{closedhihat,80},{sidekick,   70},{ride,100}},{{0}},{{0}},{{0}},{{0}},{{0}},{{ride,100}}},	// beat 1
/* beat 2 */{{{closedhihat,60},{accbassdrum,80},{ride,100}}},												// beat 2
/* beat 3 */{{{closedhihat,80},{sidekick,   80},{ride,100}},{{0}},{{0}},{{0}},{{0}},{{0}},{{ride,100}}},	// beat 3
		},
		{							// measure 1
			//  segment 0                                   |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6         |seg 7
/* beat 0 */{{{closedhihat,70},{accbassdrum,60},{ride,100}}},												// beat 0
/* beat 1 */{{{closedhihat,80},{sidekick,   70},{ride,100}},{{0}},{{0}},{{0}},{{0}},{{0}},{{ride,100}}},	// beat 1
/* beat 2 */{{{closedhihat,60},{accbassdrum,80},{ride,100}}},												// beat 2
/* beat 3 */{{{closedhihat,80},{sidekick,   80},{ride,100}},{{0}},{{0}},{{0}},{{0}},{{0}},{{ride,100}}},	// beat 3
		},
	},
	{								// style 2 (3/4)
		{							// measure 0
			//  segment 0                  |seg 1|seg 2|seg 3|seg 4|seg 5
/* beat 0 */{{{accbassdrum,70},{ride,80}}},	// beat 0
/* beat 1 */{{{closedhihat,80},{ride,80}}},	// beat 1
/* beat 2 */{{{closedhihat,80},{ride,90}}},	// beat 2
		},
		{						// measure 1
			//  segment 0                  |seg 1|seg 2|seg 3|seg 4|seg 5
/* beat 0 */{{{accbassdrum,70},{ride,80}}},	// beat 0
/* beat 1 */{{{closedhihat,80},{ride,80}}},	// beat 1
/* beat 2 */{{{closedhihat,80},{ride,90}}},	// beat 2
		},
	},
	{								// style 3 (jazzwaltz)
		{							// measure 0
			//  segment 0                  |seg 1|seg 2|seg 3|seg 4|seg 5
/* beat 0 */{{{accbassdrum,70},{ride,80}}},											// beat 0
/* beat 1 */{{{closedhihat,80}},           {{0}},{{0}},{{0}},{{0}},{{ride,99}}},	// beat 1
/* beat 2 */{{{0}},                        {{0}},{{0}},{{0}},{{0}},{{ride,80}}},	// beat 2
		},
		{						// measure 1
			//  segment 0                  |seg 1|seg 2|seg 3|seg 4|seg 5
/* beat 0 */{{{accbassdrum,70},{ride,80}}, {{0}},{{0}},{{0}},{{0}},{{ride,60}}},	// beat 0
/* beat 1 */{{{closedhihat,80},{ride,80}}}, 										// beat 1
/* beat 2 */{{{closedhihat,80},{ride,90}}},											// beat 2
		},
	},
	{								// style 4 (bossa)
		{						// measure 0
//           | segment 0                                                    |seg 1|seg 2|seg 3|seg 4
/* beat 0 */{{{closedhihat,50},{accbassdrum,99},{maracas,80},{sidekick,90}},{{0}},{{0}},{{0}},{{closedhihat,70},{cabasa,50}}},									// beat 0
/* beat 1 */{{{closedhihat,70},{maracas,80}},                               {{0}},{{0}},{{0}},{{closedhihat,90},{cabasa,50},{accbassdrum,70},{sidekick,70}}},	// beat 1
/* beat 2 */{{{closedhihat,90},{accbassdrum,99},{maracas,80}},              {{0}},{{0}},{{0}},{{openhihat,72},  {cabasa,50}}},									// beat 2
/* beat 3 */{{{closedhihat,90},{sidekick,70},   {maracas,80}},              {{0}},{{0}},{{0}},{{closedhihat,70},{cabasa,50},{accbassdrum,70}}},					// beat 3
		},
		{							// measure 1
//           | segment 0                                      |seg 1|seg 2|seg 3|seg 4
/* beat 0 */{{{accbassdrum,99},{sidekick,90},   {maracas,80}},{{0}},{{0}},{{0}},{{closedhihat,70},{cabasa,50}}},					// beat 0
/* beat 1 */{{{closedhihat,90},{sidekick,70},   {maracas,80}},{{0}},{{0}},{{0}},{{closedhihat,70},{cabasa,50},{accbassdrum,70}}},	// beat 1
/* beat 2 */{{{closedhihat,90},{accbassdrum,99},{maracas,80}},{{0}},{{0}},{{0}},{{closedhihat,90},{cabasa,50},{sidekick,70}}},		// beat 2
/* beat 3 */{{{closedhihat,80},                 {maracas,70}},{{0}},{{0}},{{0}},{{closedhihat,70},{cabasa,50},{accbassdrum,70}}},	// beat 3
		},
	},
	{								// style 5 (ballad)
		{							// measure 0
			//  segment 0
/* beat 0 */{{{closedhihat,90},{accbassdrum,35}}},													// beat 0
/* beat 1 */{{{closedhihat,90},{pedalhihat, 70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,60}}},	// beat 1
/* beat 2 */{{{closedhihat,90},{accbassdrum,35}}},													// beat 2
/* beat 3 */{{{closedhihat,90},{pedalhihat, 70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,60}}},	// beat 3
		},
		{							// measure 1
			//  segment 0                        |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6           |seg 7
/* beat 0 */{{{closedhihat,90},{accbassdrum,35}}},													// beat 0
/* beat 1 */{{{closedhihat,90},{pedalhihat, 70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,60}}},	// beat 1
/* beat 2 */{{{closedhihat,90},{accbassdrum,35}}},													// beat 2
/* beat 3 */{{{closedhihat,90},{pedalhihat, 70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,60}}},	// beat 3
		},
	},
	{								// style 6 (rock)
		{							// measure 0
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90}, {accbassdrum,90}}, {{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 1 */{{{closedhihat,100},{snare,60}},       {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,80}}},
/* beat 2 */{{{closedhihat,90}, {accbassdrum,100}},{{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 3 */{{{closedhihat,100},{snare,60}},       {{0}},{{0}},{{0}},{{closedhihat,70}}},
		},
		{							// measure 1
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90}, {accbassdrum,90}}, {{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 1 */{{{closedhihat,100},{snare,60}},       {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,80}}},
/* beat 2 */{{{closedhihat,90}, {accbassdrum,100}},{{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 3 */{{{closedhihat,100},{snare,60}},       {{0}},{{0}},{{0}},{{closedhihat,70}}},
		},
	},
	{								// style 7 (rock ballad)
		{							// measure 0
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90},{accbassdrum,50}},{{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 1 */{{{closedhihat,80},{snare,20}},      {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,50}}},
/* beat 2 */{{{closedhihat,90},{accbassdrum,50}},{{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 3 */{{{closedhihat,80},{snare,20}},      {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,50}}},
		},
		{							// measure 1
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90},{accbassdrum,50}},{{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 1 */{{{closedhihat,80},{snare,20}},      {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,50}}},
/* beat 2 */{{{closedhihat,90},{accbassdrum,50}},{{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 3 */{{{closedhihat,80},{snare,20}},      {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,50}}},
		},
	},
	{								// style 8 (dixie)
		{							// measure 0
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,60},{accbassdrum,60}}},
/* beat 1 */{{{closedhihat,60}},                  {{0}},{{0}},{{0}},{{0}},{{closedhihat,80}}},
/* beat 2 */{{{closedhihat,60},{accbassdrum,70}}},
/* beat 3 */{{{closedhihat,60}},                  {{0}},{{0}},{{0}},{{0}},{{closedhihat,80}}},
		},
		{							// measure 1
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,60},{accbassdrum,60}}},
/* beat 1 */{{{closedhihat,60}},                  {{0}},{{0}},{{0}},{{0}},{{closedhihat,80}}},
/* beat 2 */{{{closedhihat,60},{accbassdrum,70}}},
/* beat 3 */{{{closedhihat,60}},                  {{0}},{{0}},{{0}},{{0}},{{closedhihat,80}}},
		},
	},
	{								// style 9 (country)
		{							// measure 0
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90},{accbassdrum,90}},{{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 1 */{{{closedhihat,90},{sidekick,60}},   {{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 2 */{{{closedhihat,90},{accbassdrum,90}},{{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 3 */{{{closedhihat,90},{sidekick,60}},   {{0}},{{0}},{{0}},{{closedhihat,40}}},
		},
		{							// measure 1
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90},{accbassdrum,90}},{{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 1 */{{{closedhihat,90},{sidekick,60}},   {{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 2 */{{{closedhihat,90},{accbassdrum,90}},{{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 3 */{{{closedhihat,90},{sidekick,60}},   {{0}},{{0}},{{0}},{{closedhihat,40}}},
		},
	},
	{								// style 10 (easy bossa)
		{						// measure 0
//           | segment 0                         |seg 1|seg 2|seg 3|seg 4
/* beat 0 */{{{closedhihat,50},{accbassdrum,40}},{{0}},{{0}},{{0}},{{closedhihat,70}}},						// beat 0
/* beat 1 */{{{closedhihat,70}},                 {{0}},{{0}},{{0}},{{closedhihat,90},{accbassdrum,40}}},	// beat 1
/* beat 2 */{{{closedhihat,90},{accbassdrum,40}},{{0}},{{0}},{{0}},{{closedhihat,70}}},						// beat 2
/* beat 3 */{{{closedhihat,90}},                 {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,70}}},	// beat 3
		},
		{							// measure 1
//           | segment 0                         |seg 1|seg 2|seg 3|seg 4
/* beat 0 */{{{sidekick,40},   {accbassdrum,40}},{{0}},{{0}},{{0}},{{closedhihat,70}}},						// beat 0
/* beat 1 */{{{closedhihat,70}},                 {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,40}}},	// beat 1
/* beat 2 */{{{closedhihat,90},{accbassdrum,40}},{{0}},{{0}},{{0}},{{closedhihat,90},{sidekick,40}}},		// beat 2
/* beat 3 */{{{closedhihat,90}},                 {{0}},{{0}},{{0}},{{closedhihat,70},{accbassdrum,40}}},	// beat 3
		},
	},
	{								// style 11 (shuffle)
		{							// measure 0
			//  segment 0
/* beat 0 */{{{closedhihat,100},{accbassdrum,70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10}}},	// beat 0
/* beat 1 */{{{closedhihat,100},{snare, 40}},     {{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10}}},	// beat 1
/* beat 2 */{{{closedhihat,100},{accbassdrum,70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10}}},	// beat 2
/* beat 3 */{{{closedhihat,100},{snare, 40}},     {{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10},{accbassdrum,60}}},	// beat 3
		},
		{							// measure 1
			//  segment 0                        |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6           |seg 7
/* beat 0 */{{{closedhihat,100},{accbassdrum,70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10}}},	// beat 0
/* beat 1 */{{{closedhihat,100},{snare, 40}},     {{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10}}},	// beat 1
/* beat 2 */{{{closedhihat,100},{accbassdrum,70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10}}},	// beat 2
/* beat 3 */{{{closedhihat,100},{snare, 40}},     {{0}},{{0}},{{0}},{{0}},{{0}},{{snare,10},{accbassdrum,60}}},	// beat 3
		},
	},
};

// implied number of styles, 4 beats (at most) 8 segments per beat (32nd note), up to 4 notes per hit
Note Drums::endings[][4][8][4] = {

	{								// style 0 (halftime)
			//  segment 0                        |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6           |seg 7
/* beat 0 */{{{closedhihat,70},{accbassdrum,60}}},													// beat 0
/* beat 1 */{{{closedhihat,80},{sidekick,   70}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,40}}}, // beat 1
/* beat 2 */{{{closedhihat,80},{sidekick,   80},{accbassdrum,60}}}, // beat 3
	},
	{								// style 1 (4/4)
			//  segment 0                                   |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6        |seg 7
/* beat 0 */{{{closedhihat,70},{accbassdrum,60},{ride,100}}},												// beat 0
/* beat 1 */{{{closedhihat,80},{sidekick,   70},{ride,100}},{{0}},{{0}},{{0}},{{0}},{{0}},{{ride,100}}},	// beat 1
/* beat 2 */{{{closedhihat,60},{accbassdrum,80},{ride,100}}},												// beat 2
/* beat 3 */{{{crash,80}}},	// beat 3
	},
	{								// style 2 (3/4)
			//  segment 0                  |seg 1|seg 2|seg 3|seg 4|seg 5
/* beat 0 */{{{accbassdrum,90},{ride,80},{closedhihat,80},{sidekick,   70}}},	// beat 0
	},
	{								// style 3 (jazz waltz)
			//  segment 0                  |seg 1|seg 2|seg 3|seg 4|seg 5
/* beat 0 */{{{accbassdrum,90},{ride,80},{closedhihat,80},{sidekick,   70}}},	// beat 0
	},
	{								// style 4 (bossa)
//  segment 0
/* beat 0 */{{{accbassdrum,99},{closedhihat,90},{maracas,80}}},
/* beat 1 */{{{accbassdrum,99},{closedhihat,90},{maracas,80},{snare,30}},                 {{0}},{{0}},{{0}},{{snare,30}}},
/* beat 2 */{{{accbassdrum,99},{closedhihat,90},{maracas,80},{snare,30}}},
	},
	{								// style 5 (ballad)
			//  segment 0                                   |seg 1|seg 2|seg 3|seg 4|seg 5|seg 6        |seg 7
/* beat 0 */{{{closedhihat,90},{accbassdrum,35}}},												// beat 0
/* beat 1 */{{{closedhihat,90},{pedalhihat, 70},{0}},{{0}},{{0}},{{0}},{{0}},{{0}},{{closedhihat,60}}},	// beat 1
/* beat 2 */{{{closedhihat,90},{accbassdrum,35}}},												// beat 2
	},
	{								// style 6 (rock)
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90}, {accbassdrum,90}}, {{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 1 */{{{closedhihat,100},{snare,40}}},
/* beat 2 */{{{closedhihat,100},{snare,40}}},
	},
	{								// style 7 (rockballad)
			//  segment 0                          |seg 1|seg 2|seg 3|seg 4            |seg 5
/* beat 0 */{{{closedhihat,90}, {accbassdrum,90}}, {{0}},{{0}},{{0}},{{closedhihat,70}}},
/* beat 1 */{{{closedhihat,100},{snare,40}}},
/* beat 2 */{{{closedhihat,100},{snare,40}}},
	},
	{								// style 8 (dixie)
/* beat 0 */{{{closedhihat,60},{accbassdrum,60}}},
/* beat 1 */{{{closedhihat,60}},                  {{0}},{{0}},{{0}},{{0}},{{closedhihat,80}}},
/* beat 2 */{{{closedhihat,60},{accbassdrum,70}}, {{0}},{{0}},{{0}},{{0}},{{0}},{{crash,100}}},
	},
	{								// style 9 (country)
/* beat 0 */{{{closedhihat,90},{accbassdrum,90}},{{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 1 */{{{closedhihat,90},{sidekick,60}},   {{0}},{{0}},{{0}},{{closedhihat,40}}},
/* beat 2 */{{{closedhihat,90},{accbassdrum,90}}},
	},
	{								// style 10 (easy bossa)
//  segment 0
/* beat 0 */{{{accbassdrum,99},{closedhihat,90}}},
/* beat 1 */{{{accbassdrum,99},{closedhihat,90},{snare,10}},{{0}},{{0}},{{0}},{{snare,10}}},
/* beat 2 */{{{accbassdrum,99},{closedhihat,90},{snare,10}}},
	},
	{								// style 2
/* beat 0 */{{{closedhihat,100},{accbassdrum,70}}},	// beat 0
/* beat 1 */{{{accbassdrum,100},{snare, 60}},{{0}},{{0}},{{0}},{{0}},{{0}},{{accbassdrum,100},{snare, 60},{crash, 100}}},	// beat 1
		0
	},
};

Drums::Drums(MidiOut* pMidiOut) : midiOutDevice("Drums", 10, 0, pMidiOut), m_nSegment(-1)
{
	SetDrumStyle();
	m_nLastNote = 0;
	m_nSegment = 0;
}

void Drums::StartPlaying()
{
	Initialize();
	m_nSegment = -1;
	for (int i = 0; i < 4; i++)	{
		lasthit[i].note = 0;
		lasthit[i].velocity = 0;
	}
}

void Drums::StopPlaying()
{
	for (int i = 0; i < 4; i++)
		if (lasthit[i].note)
			NoteOff(lasthit[i].note, 0);
}			

void Drums::SetDrumStyle()
{
	m_nDrumStyle = theSong.m_nStyle;
	switch (m_nDrumStyle)	{
		case Song::halftime:
			m_nStyleIndex = 0;
			break;
		case Song::fourfour:
			m_nStyleIndex = 1;
			break;
		case Song::waltz:
			m_nStyleIndex = 2;
			break;
		case Song::jazzwaltz:
			m_nStyleIndex = 3;
			break;
		case Song::bossa:
			m_nStyleIndex = 4;
			break;
		case Song::ballad:
			m_nStyleIndex = 5;
			break;
		case Song::rock:
			m_nStyleIndex = 6;
			break;
		case Song::rockballad:
			m_nStyleIndex = 7;
			break;
		case Song::dixie:
			m_nStyleIndex = 8;
			break;
		case Song::country:
			m_nStyleIndex = 9;
			break;
		case Song::easybossa:
			m_nStyleIndex = 10;
			break;
		case Song::shuffle:
			m_nStyleIndex = 11;
			break;
		default:
			m_nStyleIndex = 0;
			break;
	}
}

void Drums::TimingMessage(int measure, int beat, int clock)
{
	if (!theSong.IsRest(measure, beat))	{
		if (m_nDrumStyle != theSong.m_nStyle)
			SetDrumStyle();
		int segment = (clock/3);
		if (segment != m_nSegment)	{
			int i;
			m_nSegment = segment;
			Note* pnote;
			if (theSong.IsLastMeasure())
				pnote = endings[m_nStyleIndex][beat][segment];
			else	{
				if ((measure || beat) && theSong.IsRest(measure, beat-1) && segment != 0)
					return;
				if (theSong.IsRest(measure, beat+1))	{
					if (segment == 0)	{
						for (i = 0; i < 4; i++)	{
							if (lasthit[i].note)
								NoteOff(lasthit[i].note, 0);
							lasthit[i].note = 0;
						}
						i = 0;
						if ((theSong.m_nStyle == Song::shuffle || theSong.m_nStyle == Song::rock ))	{
							NoteOn(snare, 60);
							lasthit[i++].note = snare;
						}
						NoteOn(closedhihat, 100);
						lasthit[i++].note = closedhihat;
						NoteOn(accbassdrum, 70);
						lasthit[i].note = accbassdrum;
					}
					return;
				}
				pnote = styles[m_nStyleIndex][measure&1][beat][segment];
			}
			for (i = 0; i < 4; i++)	{
				if ((pnote+i)->note)	{
					for (int j = 0; j < 4; j++)	{
						if (lasthit[j].note)	{
							NoteOff(lasthit[j].note, 0);
							lasthit[j].note = 0;
						}
					}
					break;
				}
			}
			for (i = 0; i < 4; i++)	{
				int note = (pnote+i)->note;
				if (note)	{
					NoteOn(note, (pnote+i)->velocity);
					lasthit[i].note = note;
				}
			}
		}
	}
}
