// ------ guitar.cpp

#include "stdafx.h"
#include "guitar.h"
#include "song.h"

void Guitar::Initialize()
{
	midiOutDevice::Initialize();
	m_nRoot = 0;
	m_nThird = 0;
	m_nFifth = 0;
	m_nSeventh = 0;
}

void Guitar::TimingMessage(int measure, int beat, int clock)
{
	if (theSong.IsRest(measure, beat))
		return;
	if (theSong.IsLastMeasure() && beat == 3)
		return;
	switch (clock)	{
		case 0:				// on the beat
			if (theSong.m_Chord.root)
				if (theSong.m_nStyle != Song::halftime || (beat&1) == 1)
					PlayChord(beat);
			break;
		case 8:
			if (theSong.m_nStyle == Song::halftime && beat == 1)
				NotesOff();
			break;
		case 12:
			NotesOff();
			break;
		default:
			break;
	}
}

void Guitar::PlayChord(int beat)
{
	NotesOff();
	m_nRoot = 0;
	m_nThird = 0;
	m_nFifth = 0;
	m_nSeventh = 0;

	int velroot, velthird, velfifth, velseventh;

	int velocity[4][2] = {
		{54,39}, {78,51}, {48,41}, {79,57}
	};

	int vel1 = 47, vel2 = 73;

	if (theSong.m_nStyle != Song::halftime)	{
		vel1 = velocity[beat][0];
		vel2 = velocity[beat][1];
	}

	int root = theSong.m_Chord.root+44;

	if (theSong.m_Chord.attr & CChord::diminished)	{
		m_nFifth = root + 6;
		velfifth = vel1;
		m_nRoot = root;
		velroot = vel2;
		Adjust(m_nFifth, m_nRoot);
	}
	else if (theSong.m_Chord.attr & CChord::augmented)	{
		m_nFifth = root + 8;
		velfifth = vel1;
		m_nRoot = root;
		velroot = vel2;
		Adjust(m_nFifth, m_nRoot);
	}
	else if (theSong.m_Chord.attr & CChord::seventh)	{
		m_nThird = root+4;
		velthird = vel1;
		m_nSeventh = root-2;
		velseventh = vel2;
		Adjust(m_nThird, m_nSeventh);
	}
	else	{
		m_nRoot = root;
		velroot = vel1;
		m_nThird = root-8;		
		velthird = vel2;
		Adjust(m_nRoot, m_nThird);
	}
	if (theSong.m_Chord.attr & CChord::minor)
		--m_nThird;		

	if (m_nRoot)
		NoteOn(m_nRoot, velroot);
	if (m_nThird)
		NoteOn(m_nThird, velthird);
	if (m_nFifth)
		NoteOn(m_nFifth, velfifth);
	if (m_nSeventh)
		NoteOn(m_nSeventh, velseventh);
}

void Guitar::NotesOff()
{
	if (m_nRoot)
		NoteOn(m_nRoot, 0);
	if (m_nThird)
		NoteOn(m_nThird, 0);
	if (m_nFifth)
		NoteOn(m_nFifth, 0);
	if (m_nSeventh)
		NoteOn(m_nSeventh, 0);
}

