// ---- keyboard.cpp

#include "stdafx.h"
#include "keyboard.h"
#include "song.h"

void Keyboard::Initialize()
{
	midiInDevice::Initialize();
	for (int i = 0; i < 88; i++)
		m_bNotesdown[i] = FALSE;
}

void Keyboard::NoteOff(short unsigned note)
{
	if (note > 20 && note < 21 + 88)
		m_bNotesdown[note-21] = FALSE;
}

void Keyboard::NoteOn(short unsigned note)
{
	if (note > 20 && note < 21 + 88)	{
		m_bNotesdown[note-21] = TRUE;
		BuildChord();
	}
}

Keyboard::ChordVoicings Keyboard::m_Voicings[] = {
//      notes     root major  minor   7th    dim    aug   Maj7   flatted 5th
// major
	{{1, 4, 6, 9}, {9, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
// 7th
	{{1, 4, 7, 9}, {9, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 4, 6,10}, {6, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 3, 7,10}, {3, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 5, 8,11}, {1, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
// 7th, flat 5
	{{1, 5, 7,11}, {1, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE }},
	{{1,11,17,19}, {1, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, TRUE }},
// mi7
	{{1, 4, 8,11}, {1, FALSE, TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 8,11,16}, {1, FALSE, TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 8,16,23}, {1, FALSE, TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE}},
// mi7, flat 5
	{{1, 4, 7,11}, {1, FALSE, TRUE,  TRUE,  FALSE, FALSE, FALSE, TRUE }},
// maj7
	{{1, 5, 8,12}, {1, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE,  FALSE}},
// major
	{{1, 5, 8},    {1, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 8,17},    {1, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 3,17},    {1, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 4, 9},    {9, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 6,10},    {6, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
// minor
	{{1, 4, 8},    {1, FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 5,10},   {10, FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 8,16},    {1, FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 6, 9},    {6, FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE}},
// dim
	{{1, 4, 7},    {1, FALSE, FALSE, FALSE, TRUE,  FALSE, FALSE, FALSE}},
	{{1, 7,10},    {1, FALSE, FALSE, FALSE, TRUE,  FALSE, FALSE, FALSE}},
// aug
	{{1, 5, 9},    {1, FALSE, FALSE, FALSE, FALSE, TRUE,  FALSE, FALSE}},
// mi7
	{{1, 4,11},    {1, FALSE, TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1,11,16},    {1, FALSE, TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE}},
// maj7
	{{1, 5,12},    {1, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE,  FALSE}},
	{{1, 8,12},    {1, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE,  FALSE}},
// 7th
	{{1, 7, 9},    {9, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 7,11},    {9, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 5,11},    {1, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 8,11},    {1, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
	{{1, 7,12},    {3, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
// major
	{{1, 5},       {1, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
	{{1, 8},       {1, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}},
// minor
	{{1, 4},       {1, FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE}},
// maj7
	{{1,12},       {1, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE,  FALSE}},
// 7th
	{{1,11},       {1, TRUE,  FALSE, TRUE,  FALSE, FALSE, FALSE, FALSE}},
// terminal entry
	{{0, 0},       {0, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE}}
};

void Keyboard::BuildChord()
{
	ChordVoicings crd = {{0},{0}};
	// --- collect the bottom 4 notes
	short int nct = 0;
	unsigned char lowest = 0;
	for (short int i = 0; i < 88 && nct < 4; i++)	{
		if (m_bNotesdown[i])	{
			if (!lowest)
				lowest = i+1;
			unsigned char nt = (i-(lowest-1))+1;
			// --- remove octaves
			for (short int j = 0; j < nct; j++)
				if ((nt % 12) == (crd.note[j] % 12))
					break;
			if (j == nct)
				crd.note[nct++] = nt;
		}
	}
	// --- compress the notes into a tightly-voiced chord
	for (i = 0; i < nct-1; i++)
		while (crd.note[i+1] > crd.note[i]+12)
			crd.note[i+1] -= 12;
	// ----- search the table of m_Voicings
	for (short int j = 0; m_Voicings[j].chord.root != 0; j++)	{
		for (short int k = 0; k < 4; k++)
			if (m_Voicings[j].note[k] && crd.note[k] != m_Voicings[j].note[k])
				break;
		if (k == 4)	{
			// ---- found a voicing
			crd = m_Voicings[j];
			crd.chord.root = (((lowest-1 + m_Voicings[j].chord.root)-1) % 12)+1;
			if (theSong.IsPlaying())	{
				if (crd.chord.major || crd.chord.major7)	{
					// --- look for 3rd or 10th
					for (i = 1; i < 4; i++)
						if (crd.note[i] == 5 || crd.note[i] == 17)
							break;
					if (i == 4 && crd.chord.root == (theSong.m_Chord.root&0xf))	{
						// --- no 3rd or 10th & same root,
						//        use major/minor/maj7 from prev chord
						crd.chord.minor  = (theSong.m_Chord.attr & CChord::minor)  != 0;
						crd.chord.major  = (theSong.m_Chord.attr & CChord::major)  != 0;
						crd.chord.major7 = (theSong.m_Chord.attr & CChord::major7) != 0;
					}
				}
				// --- major or minor w/no 7th inherits prev 7th if same root
				if ((crd.chord.major || crd.chord.minor) && !crd.chord.seventh)
					if (crd.chord.root == (theSong.m_Chord.root&0xf))
						crd.chord.seventh = (theSong.m_Chord.attr & CChord::seventh) != 0;
			}
			break;
		}
	}
	if (crd.chord.root)
		theSong.PostChord(crd.chord);
}
