// LeadSheet.h : +header file
//
#ifndef CLEADSHEET_H
#define CLEADSHEET_H

#include "KeyDlg.h"
#include "Song.h" 
#include "DlgSelectChord.h"

#define CEDITBASE

#ifdef CEDITBASE
#define CLeadBase CEdit
#else
#define CLeadBase CWnd
#endif

/////////////////////////////////////////////////////////////////////////////
// CLeadSheet window

class CSongPage;

class CLeadSheet : public CLeadBase
{
	int m_nTopRow;			// pixel row displayed at top of window
	int m_nBottomRow;		// pixel row of last staff in song
	int m_nWindowRows;		// number of pixel rows per window
	int m_nCountOff;		// variable for visible countoff

	CDC* m_pCDC;			// memory device context for the song display
	CBitmap* m_pCBitmap;	// bitmap for the memory device context
	CBitmap* m_pOldCBitmap;

	HCURSOR m_hArrowCursor;
	HCURSOR m_hMoveCursor;
	HCURSOR m_hCopyCursor;
	HCURSOR m_hOldCursor;

	CSongPage* m_pPage;

	CRect m_SongRect;
	CRect m_PrevRect;

	int m_nAnchorMeasure;		// the first measure of a marked block
	int m_nTerminalMeasure;		// the last measure of a marked block
	int m_nSaveAnchorMeasure;	// for saving the first measure of a marked block
	int m_nSaveTerminalMeasure;	// for saving the last measure of a marked block

	BOOL m_bMarking;
	BOOL m_bDragging;
	BOOL m_bKeyboardMarking;
	BOOL m_bClicked;
	BOOL m_bCtrlDown;
	BOOL m_bShiftDown;

	BOOL m_bHasFocus;

	BOOL m_bChordOutlined;
	int m_nOutlinedChord;

	int m_nSelectedMeasure;
	int m_nSelectedChord;

	int m_nChordsPerMeasure;
	int m_nMeasures;

	CRect m_TitleRect;
	CRect m_TempoRect;
	CRect m_KeyRect;
	CRect m_ClefRect;

	CKeyDlg m_dlgKey;

	CPoint m_MousePosition;

	void ScrollStaff(int nChord, BOOL bGoingDown);
	void ScrollTo(UINT nPos);
	void ScrollDown();
	void ScrollUp();
	void PageDown();
	void PageUp();
	void NextChord();
	void PreviousChord();
	void LowerChord();
	void UpperChord();

	void SetTempo();
	void SetTitle();
	void SetKey();
	void OnClef();

	void TestKeyboardMarking();
	void AdjustBlockDisplay(int nMeasure);

	static CPoint m_Beats[];
	static CRect m_Chords[];
	static BOOL m_bRests[];
	int m_nBeatOffset;
	static CRect m_Measures[];
	CChord m_Chord;

	CDlgSelectChord m_CDlgSelectChord;

	void SaveBlock()
	{
		m_nSaveAnchorMeasure = m_nAnchorMeasure;
		m_nSaveTerminalMeasure = m_nTerminalMeasure;
	}
	void RestoreBlock();
	void SelectMeasure(CDC* pCDC, int nMeasureNo);
	void SelectChord(CDC* pCDC, int nChordNo);
	void DrawStroke(CDC* pCDC, int nOffset, CPen* pPen);
	void ReSelectChordAndMeasure();
	void InvertMeasure(int nMeasure);
	BOOL IsMeasureInBlock(int nMeasure);
	BOOL IsChordInBlock(int nChord)
		{ return IsMeasureInBlock(nChord / m_nChordsPerMeasure); }
	void ProcessChord();
	CPoint IconPoint(int nMeasureNo);

	// clipboard
	int m_nClipboardCount;
	Song::Event* m_pClipboardEvents;

	void GetBlockMarkers(int& rStart, int& rStop, int& rCount) const;
	void SetRepeats();

// Construction
public:
	CLeadSheet();
	void Initialize(CWnd* pParent);
	void BuildSong();
	void NewSong();
	void StartPlaying();
	void Countoff(BOOL bOnOff);
	void Beat(int nBeat);
	void StopPlaying();
	void ChordEvent(CChord chord);
	void ToggleEvent(int nChord, Song::EventType evType);
	void ToggleBeginRepeat();
	void ToggleEndRepeat();
	void TogglePassage();
	void ToggleEnding();
	void ToggleQuarterRest();
	void ToggleHalfRest();
	void ToggleWholeRest();
	BOOL ChangeTitle();
	void RebuildDisplay();
	void UnmarkBlock();
	void OnInsert();
	void OnCut();
	void OnCopy();
	void OnPaste();
	void OnClear();
	void OnPrint();
	void OnStyle();
	void OnUndo()
		{ OnUndoCmd(); }
	void SelectChord(int nChordNo, BOOL bGoingDown = TRUE);
	void SelectMeasure(int nMeasureNo);

	int FirstMeasure() const;
	int LastMeasure() const;

	BOOL IsBlockMarked() const
		{ return m_nAnchorMeasure != -1; }
	BOOL CanPaste() const
		{ return (m_nSelectedMeasure != -1 || IsBlockMarked()) && m_nClipboardCount > 0; }
	BOOL CanRest() const
		{ return m_nSelectedChord != -1 && m_nSelectedChord < theSong.ChordCount(); }
	BOOL CanRepeat() const
		{ return m_nSelectedMeasure != -1 && m_nSelectedChord < theSong.ChordCount(); }
	void PrintSong(CDC* pPrinterDC, int& nPageno, int& n1stPageno, int& pagey);
	void BottomOfPage(CDC* pPrinterDC, int& nPageno);

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CLeadSheet)
	public:
//	virtual BOOL CreateEx(DWORD dwExStyle, LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, int x, int y, int nWidth, int nHeight, HWND hwndParent, HMENU nIDorHMenu, LPVOID lpParam = NULL);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CLeadSheet();

	// Generated message map functions
protected:
#ifndef DEMO_VERSION
	afx_msg void OnSaveCmd();
	afx_msg void OnSaveasCmd();
#endif
	//{{AFX_MSG(CLeadSheet)
	afx_msg void OnPaint();
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnBuildmidifileCmd();
	afx_msg void OnCopyCmd();
	afx_msg void OnCutCmd();
	afx_msg void OnNewCmd();
	afx_msg void OnOpenCmd();
	afx_msg void OnPasteCmd();
	afx_msg void OnPrintCmd();
	afx_msg void OnUndoCmd();
	afx_msg void OnAbbreviated();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
#endif
