// ---- midiout.cpp

#include "stdafx.h"
#include "midiout.h"
#include "options.h"

std::vector<OpenDevice> MidiOut::devicelist;
short int MidiOut::numDevices;
bool MidiOut::m_bBuildingMidiFile = 0;
OpenDevice MidiOut::nulldevice = {0,0};

MidiOut::MidiOut()
{
    hMidiOut = 0;
	m_dwOrigMidiVolume = 0;
	m_dwNewMidiVolume = 0;
	// ---- test for MIDI output devices
	if (numDevices == 0)	{
		numDevices = midiOutGetNumDevs();
		for (int i = 0; i < numDevices; i++)
			devicelist.push_back(nulldevice);
	}
	currDevice = -1;
}
MidiOut::~MidiOut()
{
	MIDIOUTCAPS MidiOutCaps;
	midiOutGetDevCaps(currDevice, &MidiOutCaps,	sizeof(MidiOutCaps));
	if (MidiOutCaps.dwSupport & MIDICAPS_VOLUME)	{
		// --- in case the user changed the volume...
		midiOutGetVolume(hMidiOut, &m_dwNewMidiVolume);
		// --- store that value as the one to use next time in
		if (m_dwNewMidiVolume)
			theOptions.SetMasterMidiVolume(m_dwNewMidiVolume);
		// --- reset MIDI volume to its original value on startup
		if (m_dwOrigMidiVolume)
			midiOutSetVolume(hMidiOut, m_dwOrigMidiVolume);
	}
	CloseDevice();
}
void MidiOut::CloseDevice()
{
	if (hMidiOut != 0 && !m_bBuildingMidiFile)	{
		ASSERT(currDevice >= 0);
		ASSERT(devicelist[currDevice].referencecount > 0);
		if (--devicelist[currDevice].referencecount == 0)	{
			// --- close the device
			midiOutClose(hMidiOut);
			devicelist[currDevice] = nulldevice;
		}
		hMidiOut = 0;
		currDevice = -1;
	}
}

bool MidiOut::ChangeDevice(short int device)
{
	if (device >= numDevices)
		device = numDevices-1;
	if (device != currDevice && !m_bBuildingMidiFile)	{
		CloseDevice();
		if (devicelist[device].referencecount == 0)	{
				// --- open the new device
			HMIDIOUT hOut;
			if (midiOutOpen(&hOut, device, 0, 0L, 0L) != 0)	{
				currDevice = -1;
				return FALSE;
			}
			devicelist[device].hOut = hOut;
		}
		devicelist[device].referencecount++;
		hMidiOut = devicelist[device].hOut;
		currDevice = device;

		MIDIOUTCAPS MidiOutCaps;
		midiOutGetDevCaps(currDevice, &MidiOutCaps,	sizeof(MidiOutCaps));
		if (MidiOutCaps.dwSupport & MIDICAPS_VOLUME)	{
			if (m_dwOrigMidiVolume == 0)	{
				// --- save the current MIDI device volume
				midiOutGetVolume(hMidiOut, &m_dwOrigMidiVolume);
				// --- set the MIDI volume based on what was last saved
				m_dwNewMidiVolume = theOptions.GetMasterMidiVolume();
				if (m_dwNewMidiVolume)
					midiOutSetVolume(hMidiOut, m_dwNewMidiVolume);
			}
		}
	}
	return TRUE;
}

void MidiOut::DoMessage(DWORD mmsg)
{
	if (!m_bBuildingMidiFile)	{
		std::vector<OpenDevice>::iterator iter;
		for (iter = devicelist.begin(); iter != devicelist.end(); iter++)
			midiOutShortMsg(iter->hOut, mmsg);
	}
}	

void MidiOut::DeviceList(CComboBox* dlist)
{
	ASSERT(dlist != 0);
	MIDIOUTCAPS ocaps;
	for (short int i = 0; i < numDevices; i++)	{
		midiOutGetDevCaps(i, &ocaps, sizeof(ocaps));
		dlist->AddString(ocaps.szPname);
	}
}

