// ---- midoutdv.cpp

#include "stdafx.h"
#include "options.h"
#include "midifitz.h"
#include "midoutdv.h"

midiOutDevice::midiOutDevice(char* pDevName, int nChannel, int nPatch, MidiOut* pMidiOut, BOOL bRecord) : 
			midiDevice(pDevName, nChannel),
			m_nDefaultPatch(nPatch),
			m_pMidiOut(pMidiOut)
{
	m_pCMFEvents = 0;
	if (bRecord)
		m_pCMFEvents = new CMFEvents;
	Initialize();
}

void midiOutDevice::Initialize()
{
	midiDevice::Initialize();
	if (m_pCMFEvents != 0)
		m_pCMFEvents->Initialize();
	m_nPatch = theOptions.GetPatch(m_pDevName, m_nDefaultPatch);
	m_nVolume = theOptions.GetVolume(m_pDevName);
	m_nPan = theOptions.GetPan(m_pDevName);
	SetChannel(theOptions.GetChannel(m_pDevName, m_nDefaultChannel));
}
void midiOutDevice::SetChannel(int channel)
{
	midiDevice::SetChannel(channel);
	if (m_pMidiOut != 0)	{
		SetPatch(m_nPatch);
		SetPan(m_nPan);
		SetVolume(m_nVolume);
	}
}
void midiOutDevice::SetPatch(int patch)
{
	if (patch != m_nPatch)	{
		m_nPatch = patch;
		theOptions.SetPatch(m_pDevName, m_nPatch);
	}
	DWORD dwEvent  = 0xc0 | m_nChannel | (m_nPatch << 8);
	if (theApp.m_bBuildingMidiFile)	{
		if (m_pCMFEvents != 0)
			m_pCMFEvents->PostEvent(0, dwEvent);
	}
	else if (m_pMidiOut != 0)
		m_pMidiOut->SendEvent(dwEvent);
}
void midiOutDevice::SetPan(int pan)
{
	if (pan != m_nPan)	{
		m_nPan = pan;
		theOptions.SetPan(m_pDevName, m_nPan);
	}
	DWORD dwEvent  = 0xB0 | m_nChannel | (0x0a << 8) | (m_nPan << 16);
	if (theApp.m_bBuildingMidiFile)	{
		if (m_pCMFEvents != 0)
			m_pCMFEvents->PostEvent(0, dwEvent);
	}
	else if (m_pMidiOut != 0)
		m_pMidiOut->SendEvent(dwEvent);
}
void midiOutDevice::SetVolume(int vol)
{
	if (vol != m_nVolume)	{
		m_nVolume = vol;
		theOptions.SetVolume(m_pDevName, m_nVolume);
	}
	DWORD dwEvent  = 0xB0 | m_nChannel | (0x07 << 8) | (m_nVolume << 16);
	if (theApp.m_bBuildingMidiFile)	{
		if (m_pCMFEvents != 0)
			m_pCMFEvents->PostEvent(0, dwEvent);
	}
	else if (m_pMidiOut != 0)
		m_pMidiOut->SendEvent(dwEvent);
}

void midiOutDevice::NoteOn(int note, int velocity)
{
	DWORD dwEvent  = 0x90 | m_nChannel | (note << 8) | (velocity << 16);
	if (theApp.m_bBuildingMidiFile)	{
		if (m_pCMFEvents != 0)
			m_pCMFEvents->PostEvent(theApp.m_dwMidiClock, dwEvent);
	}
	else if (m_pMidiOut != 0)
		m_pMidiOut->SendEvent(dwEvent);
}
void midiOutDevice::NoteOff(int note, int velocity)
{
	DWORD dwEvent  = 0x80 | m_nChannel | (note << 8) | (velocity << 16);
	if (theApp.m_bBuildingMidiFile)	{
		if (m_pCMFEvents != 0)
			m_pCMFEvents->PostEvent(theApp.m_dwMidiClock, dwEvent);
	}
	else if (m_pMidiOut != 0)
		m_pMidiOut->SendEvent(dwEvent);
}
void midiOutDevice::ControllerChange(int controller, int velocity)
{
	if (m_pMidiOut != 0 && !theApp.m_bBuildingMidiFile)	{
		DWORD dwEvent  = 0xB0 | m_nChannel | (controller << 8) | (velocity << 16);
		m_pMidiOut->SendEvent(dwEvent);
	}
}

