// MidiFitz.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <direct.h>
#include "MidiFitz.h"
#include "Sheet.h"

#include "keyboard.h"
#include "piano.h"
#include "drums.h"
#include "bass.h"
#include "guitar.h"
#include "options.h"
#include "song.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMidiFitzApp

BEGIN_MESSAGE_MAP(CMidiFitzApp, CWinApp)
	//{{AFX_MSG_MAP(CMidiFitzApp)
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp) 
END_MESSAGE_MAP()

// The one and only CMidiFitzApp object
CMidiFitzApp theApp;
// The encapsulated application options
COptions theOptions;

CMidiFitzApp::CMidiFitzApp()
{
	m_pMidiIn = 0;
	m_pMidiPianoOut = 0;
	m_pMidiDrumsOut = 0;
	m_pMidiBassOut = 0;
	m_pMidiGuitarOut = 0;
	m_pKeyboard = 0;
	m_pPiano = 0;
	m_pDrums = 0;
	m_pBass = 0;
	m_pActivePage = 0;
	m_dwMidiClock = 0;
	m_bBuildingMidiFile = 0;
	m_pBlackPen = 0;
	m_pBoldPen  = 0;
	m_pRedPen   = 0;
	m_pClearBrush = 0;
}

// CMidiFitzApp initialization
BOOL CMidiFitzApp::InitInstance()
{
	CWinApp::InitInstance();

	{

		CWaitCursor wait;

		m_hThreeFourIcon = LoadIcon(IDI_THREEFOUR);
		m_hFourFourIcon  = LoadIcon(IDI_FOURFOUR);
		m_hStopIcon      = LoadIcon(IDI_STOP);
		m_hPianoIcon     = LoadIcon(IDI_PIANO);
		m_hDrumsIcon     = LoadIcon(IDI_DRUMS);
		m_hBassIcon      = LoadIcon(IDI_BASS);
		m_hGuitarIcon    = LoadIcon(IDI_GUITAR);
		m_hRecordIcon    = LoadIcon(IDI_RECORD);
		m_hCountoffIcon  = LoadIcon(IDI_COUNTOFF);
		m_hNoCountIcon   = LoadIcon(IDI_NOCOUNT);
		m_hRimShotIcon   = LoadIcon(IDI_RIMSHOT);

		m_h44TrebleIcon  = LoadIcon(IDI_44TREBLECLEF);
		m_h34TrebleIcon  = LoadIcon(IDI_34TREBLECLEF);
		m_h44BassIcon    = LoadIcon(IDI_44BASSCLEF);
		m_h34BassIcon    = LoadIcon(IDI_34BASSCLEF);
		m_hMeasureIcon   = LoadIcon(IDI_MEASURE);
		m_hTempoIcon     = LoadIcon(IDI_TEMPO);
		m_hFlatIcon      = LoadIcon(IDI_FLAT);
		m_hSharpIcon     = LoadIcon(IDI_SHARP);
		m_hQtrIcon       = LoadIcon(IDI_QTRREST);
		m_hRedQtrIcon    = LoadIcon(IDI_REDQTRREST);
		m_hDeltaIcon     = LoadIcon(IDI_DELTA);

		m_hUpIcon        = LoadIcon(IDI_UP);
		m_hDownIcon      = LoadIcon(IDI_DOWN);
		m_hPlayIcon      = LoadIcon(IDI_PLAY);
		m_hStopPlayIcon  = LoadIcon(IDI_STOPPLAY);
		m_hCueIcon       = LoadIcon(IDI_CUE);

		m_hNewIcon = LoadIcon(IDI_NEW);
		m_hOpenIcon = LoadIcon(IDI_OPEN);
		m_hSaveIcon = LoadIcon(IDI_SAVE);
		m_hBuildMidiFileIcon = LoadIcon(IDI_BUILDMIDIFILE);
		m_hCutIcon = LoadIcon(IDI_CUT);
		m_hCopyIcon = LoadIcon(IDI_COPY);
		m_hPasteIcon = LoadIcon(IDI_PASTE);
		m_hUndoIcon = LoadIcon(IDI_UNDO);
		m_hPrintIcon = LoadIcon(IDI_PRINT);
		m_hBeginRepeatIcon = LoadIcon(IDI_BEGINREPEAT);
		m_hEndRepeatIcon = LoadIcon(IDI_ENDREPEAT);
		m_hPassageIcon = LoadIcon(IDI_PASSAGE);
		m_hStyleIcon = LoadIcon(IDI_STYLE);
		m_hEndingIcon = LoadIcon(IDI_ENDING);
		m_hQuarterRestIcon = LoadIcon(IDI_QUARTERREST);
		m_hHalfRestIcon = LoadIcon(IDI_HALFREST);
		m_hWholeRestIcon = LoadIcon(IDI_WHOLEREST);

		m_TitleFont.CreateFont(14,6,0,0,400,0,0,0,0,1,2,1,49,"Arial");
		m_ChordFont.CreateFont(13,4,0,0,400,0,0,0,0,1,2,1,49,"Arial");
		m_StyleFont.CreateFont(
			9,								// height
			0,								// width
			0,								// escapement
			0,								// orientation
			FW_NORMAL,						// weight
			0,								// italic
			0,								// underline
			0,								// strikeout
			ANSI_CHARSET,					// charset
			OUT_DEFAULT_PRECIS,				// out precision
			CLIP_DEFAULT_PRECIS,			// clip precision
			DEFAULT_QUALITY,				// quality
			DEFAULT_PITCH,					// pitch and family
			"Small Fonts"					// face name
		);

		m_pBlackPen    = new CPen(PS_SOLID, 1, RGB(0,0,0));
		m_pRedPen      = new CPen(PS_SOLID, 1, RGB(255,0,0));
		m_pBoldPen     = new CPen(PS_SOLID, 2, RGB(0,0,0));
		m_pClearPen    = new CPen(PS_SOLID, 1, RGB(255,255,255));
		m_pClearBrush  = new CBrush(RGB(255,255,255));
		m_pBorderBrush = new CBrush(RGB(0,0,0));

		SetRegistryKey("Al Stevens Programs");
		theOptions.LoadOptions();

		if (theOptions.m_strSongPath.IsEmpty())	{
			theOptions.m_strSongPath = m_pszHelpFilePath;
			int nIndex = theOptions.m_strSongPath.ReverseFind('\\');
			if (nIndex > 0)
				theOptions.m_strSongPath = theOptions.m_strSongPath.Left(nIndex+1);
			theOptions.m_strSongPath += "Songs";
		}

		_chdir(theOptions.m_strSongPath);

		m_pMidiIn  = new MidiIn;

		m_pMidiPianoOut  = new MidiOut;
		m_pMidiDrumsOut  = new MidiOut;
		m_pMidiBassOut   = new MidiOut;
		m_pMidiGuitarOut = new MidiOut;

		m_pKeyboard = new Keyboard(m_pMidiIn);
		m_pPiano    = new Piano(m_pMidiPianoOut);
		m_pDrums    = new Drums(m_pMidiDrumsOut);
		m_pBass     = new Bass(m_pMidiBassOut);
		m_pGuitar   = new Guitar(m_pMidiGuitarOut);

#ifdef DEMO_VERSION
		m_pCSheet  = new CSheet("MidiFitz - Demo Version");
#else
		m_pCSheet  = new CSheet("MidiFitz");
#endif
		m_pMainWnd = m_pCSheet;
	}

	if (theOptions.m_strLastSong.IsEmpty() || !theSong.LoadSong(theOptions.m_strLastSong))
		theSong.BuildDummySong();


	m_pCSheet->DoModal();
	return FALSE;
}

int CMidiFitzApp::ExitInstance()
{
	delete m_pCSheet;
	delete m_pBass;
	delete m_pDrums;
	delete m_pPiano;
	delete m_pKeyboard;
	delete m_pMidiGuitarOut;
	delete m_pMidiBassOut;
	delete m_pMidiDrumsOut;
	delete m_pMidiPianoOut;
	delete m_pMidiIn;
	delete m_pBoldPen;
	delete m_pRedPen;
	delete m_pBlackPen;
	delete m_pClearPen;
	delete m_pClearBrush;
	delete m_pBorderBrush;
	return CWinApp::ExitInstance();
}

void CMidiFitzApp::SetDevices()
{
	m_pMidiIn->ChangeDevice(theOptions.m_nMidiInDevice);
	m_pMidiPianoOut->ChangeDevice(theOptions.m_nMidiPianoDevice);
	m_pMidiDrumsOut->ChangeDevice(theOptions.m_nMidiDrumsDevice);
	m_pMidiBassOut->ChangeDevice(theOptions.m_nMidiBassDevice);
	m_pMidiGuitarOut->ChangeDevice(theOptions.m_nMidiGuitarDevice);
}

void CMidiFitzApp::StartPlaying()
{
	m_pCSheet->StartPlaying();
	m_pCSheet->SaveSettings();
	m_dwMidiClock = 0;
	SetDevices();
	m_pKeyboard->Initialize();
	if (theOptions.m_nPianoPlaying)
		m_pPiano->Initialize();
	if (theOptions.m_nDrumsPlaying)
		m_pDrums->StartPlaying();
	if (theOptions.m_nBassPlaying)
		m_pBass->StartPlaying();
	if (theOptions.m_nGuitarPlaying)
		m_pGuitar->StartPlaying();
}

void CMidiFitzApp::StopPlaying()
{
	m_pKeyboard->StopPlaying();
	if (theOptions.m_nPianoPlaying)
		m_pPiano->StopPlaying();
	if (theOptions.m_nDrumsPlaying)
		m_pDrums->StopPlaying();
	if (theOptions.m_nBassPlaying)
		m_pBass->StopPlaying();
	m_pCSheet->StopPlaying();
	m_dwMidiClock = 0;
}

void CMidiFitzApp::TimingMessage(int measure, int beat, int clock)
{
	if (theOptions.m_nDrumsPlaying)
		m_pDrums->TimingMessage(measure, beat, clock);
	if (theOptions.m_nBassPlaying)
		m_pBass->TimingMessage(measure, beat, clock);
	if (theOptions.m_nGuitarPlaying)
		m_pGuitar->TimingMessage(measure, beat, clock);
	m_dwMidiClock++;
}

void CMidiFitzApp::ChordEvent(CChord chord)
{
	m_pCSheet->ChordEvent(chord);
}

void CMidiFitzApp::SetBuildingMidiFile(bool bSet)
{
	m_bBuildingMidiFile = bSet;
	MidiOut::SetBuildingMidiFile(bSet);
}

void CMidiFitzApp::BuildMidiFile()
{
	CFileDialog dlg(FALSE,
					"mid",
					theSong.m_strTitle,
					OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR,
					"Standard Midi Format Files (*.mid)|*.mid||"
					);
	if (dlg.DoModal() == IDOK)	{
		CWaitCursor wait;
		try	{
			WriteMidiFile(dlg.GetPathName());
		}
		catch (CFileException)	{
			AfxMessageBox("Cannot create MIDI file", MB_ICONSTOP);
		}
	}
}

void CMidiFitzApp::Parse32Bits(unsigned char* cBytes, DWORD dw32)
{
	for (int i = 3; i >= 0; --i)	{
		*(cBytes + i) = (unsigned char)(dw32 & 0xff);
		dw32 >>= 8;
	}
}

int CMidiFitzApp::WriteVarLength(CFile& midiFile, DWORD dw32)
{
	unsigned char cBytes[4];
	int nCt = 1;
	for (int i = 3; i >= 0; --i)	{
		*(cBytes + i) = (unsigned char)(dw32 & 0x7f);
		dw32 >>= 7;
	}
	for (i = 0; i < 3; i++)	{
		if (*(cBytes + i) != 0)	{
			while (i < 3)	{
				*(cBytes + i) |= 0x80;
				midiFile.Write(cBytes + i++, 1);
				nCt++;
			}
		}
	}
	midiFile.Write(cBytes + 3, 1);
	return nCt;
}

// --- convert tempo (quarter notes/minute) to
//     microseconds per quarter note
inline DWORD tmpo2us(UINT tmpo)
{
	DWORD us = 60000000L / tmpo;
	return us;
}

void CMidiFitzApp::WriteMidiFile(const CString& strPath)
{
	CFile midiFile(strPath, CFile::modeCreate | CFile::modeWrite | CFile::typeBinary);

	// ---- write file header (type 1, 3 tracks, 120 (0x78) delta time ticks per quarter note
	char MThd[] = "MThd\x00\x00\x00\x06\x00\x01\x00\x04\x00\x78";
	midiFile.Write(MThd, sizeof MThd - 1);

	// ---- write first track chunk with the title, time signature, MIDI clocks, tempo

	// --- track name length
	char* pTitle = theSong.m_strTitle.GetBuffer(0); 
	unsigned int nLen = strlen(pTitle);

	DWORD dwTrkLen = nLen + 28;

	// --- have to return here when we know the length
	DWORD dwPos = midiFile.GetPosition();

	// --- track chunk ID and length
	unsigned char MTrk[] = "MTrk\00\00\00\00";
	midiFile.Write(MTrk, sizeof MTrk - 1);

	// NOTE: Each meta event starts with "\x00\xff", which is a zero value delta time and 
	//       the 0xff byte that identifies a meta event
	//       the meta event code itself follows that sequence
	//       the number of data bytes follows the event code
	//       the data bytes follow the data byte count

	//
	// --- track name (song title)
	//
	midiFile.Write("\x00\xff\x03", 3);
	// --- track name length
	int nNameLenLen = WriteVarLength(midiFile, nLen);
	// --- track name (song title)
	midiFile.Write(pTitle, nLen);

	// --- time signature
	midiFile.Write("\x00\xff\x58\x04", 4);
	// --- (3 or 4) * 2^2 = 3/4 or 4/4
	midiFile.Write(theSong.IsWaltz() ? "\x03\x02" : "\x04\x02", 2);
	// --- 24 clocks per metronome tick ( = 1 tick per quarter note), 
	// ---  8 32nd notes in a quarter note
	midiFile.Write("\x18\x08", 2);

	// --- key signature
	midiFile.Write("\x00\xff\x59\x02", 4);
	CChord& rCh = theSong.m_Key;
	// --- root key (-7=7 flats ... 0=C ... 7=7 sharps)
	int mm = (rCh.attr & CChord.minor) ? 1 : 0;
	// --- # of sharps (positive) or flats (negative) in the key signature
	static int sf[] = {3,-2,5,0,-5,2,-3,4,-1,6,1,-4};
	int sfx = (rCh.root & 0xf) - 1;
	if (mm)	{
		sfx += 3;
		if (sfx > 11)
			sfx -= 12;
	}
	// --- sharps/flats
	midiFile.Write(&sf[sfx], 1);
	// --- major (0) or minor (1)
	midiFile.Write(&mm, 1);

	// --- tempo
	midiFile.Write("\x00\xff\x51\x03", 4);
	// --- compute microseconds per quarter note (aka "24ths of a microsecond per MIDI clock")
	DWORD us = tmpo2us(theSong.m_nTempo);
	unsigned char Us[4];
	Parse32Bits(Us, us);
	midiFile.Write(Us+1, 3);

	// --- end of track
	midiFile.Write("\x00\xff\x2f\x00", 4);

	midiOutDevice* pDevices[] = {
		m_pDrums,
		m_pBass,
		m_pGuitar
	};

	dwTrkLen += nNameLenLen;
	Parse32Bits(MTrk+4, dwTrkLen);

	DWORD dwRetPos = midiFile.GetPosition();
	midiFile.Seek(dwPos, CFile::begin);
	midiFile.Write(MTrk, sizeof MTrk - 1);
	midiFile.Seek(dwRetPos, CFile::begin);

	for (int i = 0; i < sizeof(pDevices) / sizeof(midiOutDevice*); i++)	{
		dwTrkLen = 0;
		unsigned char MTrk[] = "MTrk\00\00\00\00";
		// --- have to return here when we know the length
		DWORD dwPos = midiFile.GetPosition();
		// --- track chunk ID and length
		midiFile.Write(MTrk, sizeof MTrk - 1);

		midiOutDevice& rDev = *pDevices[i];
		int nEventCount = rDev.EventCount();

		// --- instrument name
		const char* pName = rDev.DeviceName();
		int nLen = strlen(pName);
		// --- track name
		midiFile.Write("\x00\xff\x03", 3);
		int nNameLenLen = WriteVarLength(midiFile, nLen);
		midiFile.Write(pName, nLen);
		dwTrkLen += nLen + nNameLenLen + 3;

		// --- write the MIDI events for this track
		for (int j = 0; j < nEventCount; j++)	{
			// --- the delta time for this event
			DWORD dwClock = rDev.Clock(j) * 5;
			dwTrkLen += WriteVarLength(midiFile, dwClock);
			// --- the event itself
			DWORD dwMidiEvent = rDev.MidiEvent(j);
			// --- the length of the event (implied, not written)
			int nLen = (dwMidiEvent & 0xf0) == 0xc0 ? 2 : 3;
			// --- write the event
			midiFile.Write(&dwMidiEvent, nLen);
			// --- accumulate the track length
			dwTrkLen += nLen;
		}

		// --- end of track
		midiFile.Write("\x00\xff\x2f\x00", 4);
		dwTrkLen += 4;

		// --- build the track length
		Parse32Bits(MTrk+4, dwTrkLen);
		// --- save the current file position for the next track
		DWORD dwRetPos = midiFile.GetPosition();
		// --- seek to the length field for this track
		midiFile.Seek(dwPos, CFile::begin);
		// --- write the length of the current track
		midiFile.Write(MTrk, sizeof MTrk - 1);
		// --- return to write the next track
		midiFile.Seek(dwRetPos, CFile::begin);
	}
}

void CMidiFitzApp::InitializeControls()
{
	m_pCSheet->InitializeControls();
}

void CMidiFitzApp::SetMouse(const CButton& rButton)
{
	CRect rc;
	rButton.GetWindowRect(&rc);
	SetCursorPos(rc.left + rc.Width() / 2, rc.top + rc.Height() / 2);
}