// MidiFitz.h : main header file for the MIDIFITZ application
//
#ifndef MIDIFITZ_H
#define MIDIFITZ_H

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols
#include "midicord.h"

/////////////////////////////////////////////////////////////////////////////
// CMidiFitzApp:
// See MidiFitz.cpp for the implementation of this class
//

class CPage;
class MidiIn;
class MidiOut;
class Song;
class Keyboard;
class Piano;
class Drums;
class Bass;
class Guitar;
class CSheet;

class CMidiFitzApp : public CWinApp
{
public:
	MidiIn*  m_pMidiIn;
	MidiOut* m_pMidiPianoOut;
	MidiOut* m_pMidiDrumsOut;
	MidiOut* m_pMidiBassOut;
	MidiOut* m_pMidiGuitarOut;

	Keyboard* m_pKeyboard;
	Piano*    m_pPiano;
	Drums*    m_pDrums;
	Bass*     m_pBass;
	Guitar*   m_pGuitar;

	BOOL m_bBuildingMidiFile;

	CSheet* m_pCSheet;
	CPage* m_pActivePage;

	// ---- Play page icons
	HICON m_hThreeFourIcon;
	HICON m_hFourFourIcon;
	HICON m_hStopIcon;
	HICON m_hPianoIcon;
	HICON m_hDrumsIcon;
	HICON m_hBassIcon;
	HICON m_hGuitarIcon;
	HICON m_hRecordIcon;
	HICON m_hCountoffIcon;
	HICON m_hNoCountIcon;
	HICON m_hRimShotIcon;

	// ---- lead sheet icons
	HICON m_h44TrebleIcon;	// handle of the treble clef 4/4 icon
	HICON m_h34TrebleIcon;	// handle of the treble clef 3/4 icon
	HICON m_h44BassIcon;	// handle of the bass clef 4/4 icon
	HICON m_h34BassIcon;	// handle of the bass clef 3/4 icon
	HICON m_hMeasureIcon;	// handle of the measure selection icon
	HICON m_hTempoIcon;		// handle of the tempo icon
	HICON m_hFlatIcon;		// handle of the flat (b) icon
	HICON m_hSharpIcon;		// handle of the sharp (#) icon
	HICON m_hQtrIcon;		// quarter rest icon
	HICON m_hRedQtrIcon;	// red (selected) quarter rest icon
	HICON m_hDeltaIcon;		// Maj7 symbol

	// ---- Sets page icons
	HICON m_hUpIcon;
	HICON m_hDownIcon;
	HICON m_hPlayIcon;
	HICON m_hStopPlayIcon;
	HICON m_hCueIcon;

	// ---- Song page icons
	HICON m_hNewIcon;
	HICON m_hOpenIcon;
	HICON m_hSaveIcon;
	HICON m_hBuildMidiFileIcon;
	HICON m_hCutIcon;
	HICON m_hCopyIcon;
	HICON m_hPasteIcon;
	HICON m_hUndoIcon;
	HICON m_hPrintIcon;
	HICON m_hBeginRepeatIcon;
	HICON m_hEndRepeatIcon;
	HICON m_hPassageIcon;
	HICON m_hStyleIcon;
	HICON m_hEndingIcon;
	HICON m_hQuarterRestIcon;
	HICON m_hHalfRestIcon;
	HICON m_hWholeRestIcon;

	// --- lead sheet tools
	CFont m_TitleFont;
	CFont m_ChordFont;
	CFont m_StyleFont;

	CPen* m_pBlackPen;
	CPen* m_pBoldPen;
	CPen* m_pRedPen;
	CPen* m_pClearPen;
	CBrush* m_pClearBrush;
	CBrush* m_pBorderBrush;

	CString m_strChord;
	CString m_strBeat;

	DWORD m_dwMidiClock;

	CMidiFitzApp();
	void SetDevices();
	void StartPlaying();
	void TimingMessage(int measure, int beat, int clock);
	void StopPlaying();
	void ChordEvent(CChord chord);
	void SetActivePage(CPage* pActivePage)
		{ m_pActivePage = pActivePage; }
	CPage* ActivePage() const
		{ return m_pActivePage; }
	void SetBuildingMidiFile(bool bSet);
	void BuildMidiFile();
	void InitializeControls();
	void SetMouse(const CButton& rButton);
private:
	void WriteMidiFile(const CString& strPath);
	void Parse32Bits(unsigned char* cBytes, DWORD dw32);
	int WriteVarLength(CFile& midiFile, DWORD dw32);


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMidiFitzApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation

	//{{AFX_MSG(CMidiFitzApp)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

extern CMidiFitzApp theApp;

#endif
/////////////////////////////////////////////////////////////////////////////
