// MidiPage.cpp : implementation file
//

#include "stdafx.h"
#include "options.h"
#include "MidiPage.h"
#include "song.h"
#include "midiin.h"
#include "midiout.h"
#include "keyboard.h"
#include "piano.h"
#include "drums.h"
#include "bass.h"
#include "guitar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMidiPage property page

IMPLEMENT_DYNCREATE(CMidiPage, CPage)

CMidiPage::CMidiPage() : CPage(CMidiPage::IDD, "MIDI Page")
{
	//{{AFX_DATA_INIT(CMidiPage)
	m_nPianoPatch = -1;
	m_nDrumsPatch = -1;
	m_nBassPatch = -1;
	m_nGuitarPatch = -1;
	//}}AFX_DATA_INIT
	m_psp.dwFlags |= PSP_HASHELP;

	m_pBlackOffPen = new CPen(PS_SOLID, 1, RGB(0,0,0));
	m_pWhiteOffPen = new CPen(PS_SOLID, 1, RGB(255,255,255));

	const COLORREF blackon = RGB(255, 0, 0);
	const COLORREF whiteon = RGB(255, 0, 0);

	m_pBlackOnPen = new CPen(PS_SOLID, 1, blackon);
	m_pWhiteOnPen = new CPen(PS_SOLID, 1, whiteon);
}

CMidiPage::~CMidiPage()
{
	delete m_pWhiteOnPen;
	delete m_pBlackOnPen;
	delete m_pWhiteOffPen;
	delete m_pBlackOffPen;
}

void CMidiPage::DoDataExchange(CDataExchange* pDX)
{
	CPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMidiPage)
	DDX_Control(pDX, IDC_CHORDDISPLAY, m_Chord);
	DDX_Control(pDX, IDC_KEYBOARD, m_Keyboard);
	DDX_Control(pDX, IDC_PIANODEVICE, m_PianoDevice);
	DDX_Control(pDX, IDC_BASSDEVICE, m_BassDevice);
	DDX_Control(pDX, IDC_GUITARDEVICE, m_GuitarDevice);
	DDX_Control(pDX, IDC_DRUMSDEVICE, m_DrumsDevice);
	DDX_Control(pDX, IDC_INDEVICE, m_InDevice);
	DDX_Control(pDX, IDC_KEYBOARDSPIN, m_KeyboardChannel);
	DDX_Control(pDX, IDC_PIANOSPIN, m_PianoChannel);
	DDX_Control(pDX, IDC_DRUMSSPIN, m_DrumsChannel);
	DDX_Control(pDX, IDC_BASSSPIN, m_BassChannel);
	DDX_Control(pDX, IDC_GUITARSPIN, m_GuitarChannel);
	DDX_CBIndex(pDX, IDC_PIANOPATCH, m_nPianoPatch);
	DDX_CBIndex(pDX, IDC_DRUMSPATCH, m_nDrumsPatch);
	DDX_CBIndex(pDX, IDC_BASSPATCH, m_nBassPatch);
	DDX_CBIndex(pDX, IDC_GUITARPATCH, m_nGuitarPatch);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMidiPage, CPage)
	//{{AFX_MSG_MAP(CMidiPage)
	ON_CBN_SELCHANGE(IDC_BASSPATCH, OnSelchangeBass)
	ON_CBN_SELCHANGE(IDC_GUITARPATCH, OnSelchangeGuitar)
	ON_CBN_SELCHANGE(IDC_DRUMSPATCH, OnSelchangeDrums)
	ON_CBN_SELCHANGE(IDC_PIANOPATCH, OnSelchangePiano)
	ON_CBN_SELCHANGE(IDC_INDEVICE, OnSelchangeIndevice)
	ON_CBN_SELCHANGE(IDC_PIANODEVICE, OnSelchangePianodevice)
	ON_CBN_SELCHANGE(IDC_BASSDEVICE, OnSelchangeBassdevice)
	ON_CBN_SELCHANGE(IDC_GUITARDEVICE, OnSelchangeGuitardevice)
	ON_CBN_SELCHANGE(IDC_DRUMSDEVICE, OnSelchangeDrumsdevice)
	ON_NOTIFY(UDN_DELTAPOS, IDC_KEYBOARDSPIN, OnDeltaposKeyboardspin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_PIANOSPIN, OnDeltaposPianospin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_DRUMSSPIN, OnDeltaposDrumsspin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_BASSSPIN, OnDeltaposBassspin)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

inline void CMidiPage::SetPianoPatch()
{
	UpdateData();
	theApp.m_pPiano->SetPatch(m_nPianoPatch > 7 ? m_nPianoPatch + 8 : m_nPianoPatch);
}
inline void CMidiPage::SetDrumsPatch()
{
	UpdateData();
	theApp.m_pDrums->SetPatch(m_nDrumsPatch);
}
inline void CMidiPage::SetBassPatch()
{
	UpdateData();
	theApp.m_pBass->SetPatch(m_nBassPatch + 32);
}

inline void CMidiPage::SetGuitarPatch()
{
	UpdateData();
	theApp.m_pGuitar->SetPatch(m_nGuitarPatch + 24);
}

BOOL CMidiPage::SaveSettings()
{
	if (CPage::SaveSettings())	{
		theApp.m_pKeyboard->SetChannel(m_KeyboardChannel.GetPos());

		theApp.m_pPiano->SetChannel(m_PianoChannel.GetPos());
		SetPianoPatch();

		theApp.m_pDrums->SetChannel(m_DrumsChannel.GetPos());
		SetDrumsPatch();

		theApp.m_pBass->SetChannel(m_BassChannel.GetPos());
		SetBassPatch();

		theApp.m_pGuitar->SetChannel(m_GuitarChannel.GetPos());
		SetGuitarPatch();

		theOptions.m_nMidiInDevice = m_InDevice.GetCurSel();
		theOptions.m_nMidiPianoDevice  = m_PianoDevice.GetCurSel();
		theOptions.m_nMidiBassDevice   = m_BassDevice.GetCurSel();
		theOptions.m_nMidiGuitarDevice = m_GuitarDevice.GetCurSel();
		theOptions.m_nMidiDrumsDevice  = m_DrumsDevice.GetCurSel();

		return TRUE;
	}
	return FALSE;
}

void CMidiPage::DisplayNote(short unsigned note, BOOL bOn)
{
	if (!m_bIsActive || note < 21 || note > 108)
		return;
	note -= 21;			// make note relative to 0;
	int octave = note / 12;
	note %= 12;

	static int notetypes[] = {3, 0, 2, 1, 0, 3, 0, 2, 1, 0, 3, 0};

	int notetype =	(octave == 0 && note == 0) ? 1 :
					(octave == 7 && note == 3) ? 4 :
					notetypes[note];

	static int offset[] = {0,2,4,8,10,12,14,16,20,22,24,26};

	struct stroke {
		int y;
		int length;
	};
	static stroke strokes[][3] = {
		// note type 0: all black notes
		{ {0,  0}, {12,12}, {0,  0} },
		// note type 1: C, F, and A in octave 0
		{ {12,21}, {12,21}, {25, 8} },
		// note type 2: B and E
		{ {25, 8}, {12,21}, {12,21} },
		// note type 3: A, D,  and G
		{ {25, 8}, {12,21}, {25, 8} },
		// note type 4: C in octave 7
		{ {12,21}, {12,21}, {12,21} }
	};

	CDC* pDC = m_Keyboard.GetDC();

	CPen* pNewPen;

	if (notetype == 0)
		// black key
		pNewPen = bOn ? m_pBlackOnPen : m_pBlackOffPen;
	else
		// white key
		pNewPen = bOn ? m_pWhiteOnPen : m_pWhiteOffPen;

	CPen* pOldPen = pDC->SelectObject(pNewPen);

	int x = offset[note] + octave * 28 + 4;	// 7 octaves, 28 pixels/octave, 4 leading pixels

	for (int x1 = 0; x1 < 3; x1++)	{
		stroke& rstroke = strokes[notetype][x1];
		if (rstroke.length)	{
			pDC->MoveTo(x + x1, rstroke.y);
			pDC->LineTo(x + x1, rstroke.y + rstroke.length);
		}
	}

	pDC->SelectObject(pOldPen);

	if (bOn && m_PrevChord != theSong.m_Chord)	{
		m_PrevChord = theSong.m_Chord;
		theSong.BuildChordDisplay(theApp.m_strChord, theSong.m_Chord);
		m_Chord.SetWindowText(theApp.m_strChord);
	}
}

void CMidiPage::InitializeControls()
{
	if (m_bIsInitialized)	{

		m_KeyboardChannel.SetPos(theApp.m_pKeyboard->GetChannel());

		m_PianoChannel.SetPos(theApp.m_pPiano->GetChannel());
		m_nPianoPatch = theApp.m_pPiano->GetPatch();
		if (m_nPianoPatch > 7)
			m_nPianoPatch -= 8;

		m_DrumsChannel.SetPos(theApp.m_pDrums->GetChannel());
		m_nDrumsPatch = theApp.m_pDrums->GetPatch();

		m_BassChannel.SetPos(theApp.m_pBass->GetChannel());
		m_nBassPatch = theApp.m_pBass->GetPatch() - 32;

		m_GuitarChannel.SetPos(theApp.m_pGuitar->GetChannel());
		m_nGuitarPatch = theApp.m_pGuitar->GetPatch() - 24;

		UpdateData(FALSE);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMidiPage message handlers

BOOL CMidiPage::OnInitDialog() 
{
	CPage::OnInitDialog();
	m_KeyboardChannel.SetRange(1,16);
	m_PianoChannel.SetRange(1,16);
	m_DrumsChannel.SetRange(1,16);
	m_BassChannel.SetRange(1,16);
	m_GuitarChannel.SetRange(1,16);
	theApp.m_pMidiIn->DeviceList(&m_InDevice);
	m_InDevice.SetCurSel(theOptions.m_nMidiInDevice);

	theApp.m_pMidiPianoOut->DeviceList(&m_PianoDevice);
	theApp.m_pMidiDrumsOut->DeviceList(&m_DrumsDevice);
	theApp.m_pMidiBassOut->DeviceList(&m_BassDevice);
	theApp.m_pMidiGuitarOut->DeviceList(&m_GuitarDevice);
	
	m_PianoDevice.SetCurSel(theOptions.m_nMidiPianoDevice);
	m_DrumsDevice.SetCurSel(theOptions.m_nMidiDrumsDevice);
	m_BassDevice.SetCurSel(theOptions.m_nMidiBassDevice);
	m_GuitarDevice.SetCurSel(theOptions.m_nMidiGuitarDevice);
	InitializeControls();
	return TRUE;
}

void CMidiPage::OnSelchangePiano() 
{
	UpdateData();
	SetPianoPatch();
}
void CMidiPage::OnSelchangeDrums() 
{
	UpdateData();
	SetDrumsPatch();
}
void CMidiPage::OnSelchangeBass() 
{
	UpdateData();
	SetBassPatch();
}
void CMidiPage::OnSelchangeGuitar() 
{
	UpdateData();
	SetGuitarPatch();
}
void CMidiPage::OnSelchangeIndevice() 
{
	int indevice = m_InDevice.GetCurSel();
	if (theApp.m_pMidiIn->ChangeDevice(indevice))
		theOptions.m_nMidiInDevice = indevice;
	else
		AfxMessageBox("Cannot open input device", MB_OK | MB_ICONSTOP);
}
void CMidiPage::OnSelchangePianodevice() 
{
	int outdevice = m_PianoDevice.GetCurSel();
	if (theApp.m_pMidiPianoOut->ChangeDevice(outdevice))	{
		UpdateData();
		theOptions.m_nMidiPianoDevice = outdevice;
		SetPianoPatch();
		theApp.m_pPiano->SetVolume();
		theApp.m_pPiano->SetPan();
	}
	else
		AfxMessageBox("Cannot open output device", MB_OK | MB_ICONSTOP);
}
void CMidiPage::OnSelchangeBassdevice() 
{
	int outdevice = m_BassDevice.GetCurSel();
	if (theApp.m_pMidiBassOut->ChangeDevice(outdevice))	{
		UpdateData();
		theOptions.m_nMidiBassDevice = outdevice;
		SetBassPatch();
		theApp.m_pBass->SetVolume();
		theApp.m_pBass->SetPan();
	}
	else
		AfxMessageBox("Cannot open output device", MB_OK | MB_ICONSTOP);
}
void CMidiPage::OnSelchangeGuitardevice() 
{
	int outdevice = m_GuitarDevice.GetCurSel();
	if (theApp.m_pMidiGuitarOut->ChangeDevice(outdevice))	{
		UpdateData();
		theOptions.m_nMidiGuitarDevice = outdevice;
		SetGuitarPatch();
		theApp.m_pGuitar->SetVolume();
		theApp.m_pGuitar->SetPan();
	}
	else
		AfxMessageBox("Cannot open output device", MB_OK | MB_ICONSTOP);
}
void CMidiPage::OnSelchangeDrumsdevice() 
{
	int outdevice = m_DrumsDevice.GetCurSel();
	if (theApp.m_pMidiDrumsOut->ChangeDevice(outdevice))	{
		UpdateData();
		theOptions.m_nMidiDrumsDevice = outdevice;
		SetDrumsPatch();
		theApp.m_pDrums->SetVolume();
		theApp.m_pDrums->SetPan();
	}
	else
		AfxMessageBox("Cannot open output device", MB_OK | MB_ICONSTOP);
}
void CMidiPage::OnDeltaposInstrumentspin(midiDevice* pmidiDevice, NMHDR* pNMHDR, LRESULT* pResult)
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	int channel = pNMUpDown->iPos + pNMUpDown->iDelta;
	channel = channel == 17 ? 1 : channel == 0 ? 16 : channel;
	pmidiDevice->SetChannel(channel);
	*pResult = 0;
}
void CMidiPage::OnDeltaposKeyboardspin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnDeltaposInstrumentspin(theApp.m_pKeyboard, pNMHDR, pResult);
}
void CMidiPage::OnDeltaposPianospin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnDeltaposInstrumentspin(theApp.m_pPiano, pNMHDR, pResult);
}
void CMidiPage::OnDeltaposDrumsspin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnDeltaposInstrumentspin(theApp.m_pDrums, pNMHDR, pResult);
}
void CMidiPage::OnDeltaposBassspin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnDeltaposInstrumentspin(theApp.m_pBass, pNMHDR, pResult);
}
