// ------ options.cpp

#include "stdafx.h"
#include "MidiFitz.h"
#include "Options.h"

COptions::COptions()
{
	// --- Midi page options
	m_nMidiInDevice    =  0;
	m_nKeyboardChannel = -1;

	m_nMidiPianoDevice   =  0;
	m_nMidiBassDevice   =  0;
	m_nMidiDrumsDevice   =  0;
	m_nPianoPatch      = -1;
	m_nDrumsPatch      = -1;
	m_nBassPatch       = -1;
	m_nGuitarPatch       = -1;
	m_nPianoChannel    = -1;
	m_nDrumsChannel    = -1;
	m_nBassChannel     = -1;
	m_nGuitarChannel     = -1;

	// --- Play page options
	m_nPianoPlaying  = 1;
	m_nDrumsPlaying  = 1;
	m_nBassPlaying   = 1;
	m_nGuitarPlaying = 1;
	m_nCountoff      = 1;
	m_nStyle         = 0;

	// --- Settings page options
	m_nPianoPan     = 64;
	m_nPianoVolume  = 64;
	m_nDrumsPan     = 64;
	m_nDrumsVolume  = 64;
	m_nBassPan      = 64;
	m_nBassVolume   = 64;
	m_nGuitarPan    = 64;
	m_nGuitarVolume = 64;
	m_bAdjustBass   = FALSE;

	// --- Master MIDI out device volume
	m_dwMidiVolume = 0;

	// --- Active PropertyPage
	m_nActivePage = 0;
}

void COptions::LoadOptions()
{
	// --- Midi page options
	m_nMidiInDevice    = theApp.GetProfileInt("MIDIIn",   "Device",   0);
	m_nKeyboardChannel = theApp.GetProfileInt("Keyboard", "Channel", -1);

	m_nMidiPianoDevice  = theApp.GetProfileInt("Piano", "Device",  -1);
	m_nMidiDrumsDevice  = theApp.GetProfileInt("Drums", "Device",  -1);
	m_nMidiBassDevice   = theApp.GetProfileInt("Bass",  "Device",  -1);
	m_nMidiGuitarDevice = theApp.GetProfileInt("Guitar","Device",  -1);
	m_nPianoPatch       = theApp.GetProfileInt("Piano", "Patch",   -1);
	m_nDrumsPatch       = theApp.GetProfileInt("Drums", "Patch",   -1);
	m_nBassPatch        = theApp.GetProfileInt("Bass",  "Patch",   -1);
	m_nGuitarPatch      = theApp.GetProfileInt("Guitar","Patch",   -1);
	m_nPianoChannel     = theApp.GetProfileInt("Piano", "Channel", -1);
	m_nDrumsChannel     = theApp.GetProfileInt("Drums", "Channel", -1);
	m_nBassChannel      = theApp.GetProfileInt("Bass",  "Channel", -1);
	m_nGuitarChannel    = theApp.GetProfileInt("Guitar","Channel", -1);

	// --- Play page options
	m_nPianoPlaying  = theApp.GetProfileInt("Play", "Piano",      1);
	m_nDrumsPlaying  = theApp.GetProfileInt("Play", "Drums",      1);
	m_nBassPlaying   = theApp.GetProfileInt("Play", "Bass",       1);
	m_nGuitarPlaying = theApp.GetProfileInt("Play", "Guitar",     1);
	m_nCountoff      = theApp.GetProfileInt("Play", "Countoff",   1);
	m_nStyle         = theApp.GetProfileInt("Play", "Style",      0);
	m_nClef          = theApp.GetProfileInt("Play", "Clef",       0);
	m_nAbbreviated   = theApp.GetProfileInt("Play", "Abbreviate", 0);

	// --- Settings page options
	m_nPianoPan     = theApp.GetProfileInt("Piano",  "Pan",    64);
	m_nPianoVolume  = theApp.GetProfileInt("Piano",  "Volume", 64);
	m_nDrumsPan     = theApp.GetProfileInt("Drums",  "Pan",    64);
	m_nDrumsVolume  = theApp.GetProfileInt("Drums",  "Volume", 64);
	m_nBassPan      = theApp.GetProfileInt("Bass",   "Pan",    64);
	m_nBassVolume   = theApp.GetProfileInt("Bass",   "Volume", 64);
	m_nGuitarPan    = theApp.GetProfileInt("Guitar", "Pan",    64);
	m_nGuitarVolume = theApp.GetProfileInt("Guitar", "Volume", 64);

	m_bAdjustBass   = theApp.GetProfileInt("Bass",   "Adjust", 0);

	// --- Sets page options
	m_nSet          = theApp.GetProfileInt("Sets",    "Set",    0);
	m_nSong         = theApp.GetProfileInt("Sets",    "Song",   0);

	// --- Master MIDI out device volume
	m_dwMidiVolume = theApp.GetProfileInt("MIDIOut", "Volume", 0x7f7f7f7f);

	// --- Last song loaded
	m_strLastSong = theApp.GetProfileString("Song", "Title");

	// --- Active Song Database Path
	m_strSongPath = theApp.GetProfileString("Song", "Path");

	// --- Active PropertyPage
	m_nActivePage = theApp.GetProfileInt("Pages", "Active", 0);
}

COptions::~COptions()
{
	// --- Midi page options
	theApp.WriteProfileInt("MIDIIn",   "Device",  m_nMidiInDevice    );
	theApp.WriteProfileInt("Keyboard", "Channel", m_nKeyboardChannel );

	theApp.WriteProfileInt("Piano", "Device",  m_nMidiPianoDevice );
	theApp.WriteProfileInt("Drums", "Device",  m_nMidiDrumsDevice );
	theApp.WriteProfileInt("Bass",  "Device",  m_nMidiBassDevice  );
	theApp.WriteProfileInt("Guitar","Device",  m_nMidiGuitarDevice);
	theApp.WriteProfileInt("Piano", "Patch",   m_nPianoPatch      );
	theApp.WriteProfileInt("Drums", "Patch",   m_nDrumsPatch      );
	theApp.WriteProfileInt("Guitar","Patch",   m_nGuitarPatch     );
	theApp.WriteProfileInt("Bass",  "Patch",   m_nBassPatch       );
	theApp.WriteProfileInt("Piano", "Channel", m_nPianoChannel    );
	theApp.WriteProfileInt("Drums", "Channel", m_nDrumsChannel    );
	theApp.WriteProfileInt("Bass",  "Channel", m_nBassChannel     );
	theApp.WriteProfileInt("Guitar","Channel", m_nGuitarChannel   );

	// --- Play page options
	theApp.WriteProfileInt("Play", "Piano",      m_nPianoPlaying );
	theApp.WriteProfileInt("Play", "Drums",      m_nDrumsPlaying );
	theApp.WriteProfileInt("Play", "Bass",       m_nBassPlaying  );
	theApp.WriteProfileInt("Play", "Guitar",     m_nGuitarPlaying);
	theApp.WriteProfileInt("Play", "Countoff",   m_nCountoff     );
	theApp.WriteProfileInt("Play", "Style",      m_nStyle        );
	theApp.WriteProfileInt("Play", "Clef",       m_nClef         );
	theApp.WriteProfileInt("Play", "Abbreviate", m_nAbbreviated  );

	// --- Settings page options
	theApp.WriteProfileInt("Piano",  "Pan",    m_nPianoPan     );
	theApp.WriteProfileInt("Piano",  "Volume", m_nPianoVolume  );
	theApp.WriteProfileInt("Drums",  "Pan",    m_nDrumsPan     );
	theApp.WriteProfileInt("Drums",  "Volume", m_nDrumsVolume  );
	theApp.WriteProfileInt("Bass",   "Pan",    m_nBassPan      );
	theApp.WriteProfileInt("Bass",   "Volume", m_nBassVolume   );
	theApp.WriteProfileInt("Guitar", "Pan",    m_nGuitarPan    );
	theApp.WriteProfileInt("Guitar", "Volume", m_nGuitarVolume );
	theApp.WriteProfileInt("Bass",   "Adjust", m_bAdjustBass);

	// --- Sets page options
	theApp.WriteProfileInt("Sets",   "Set",    m_nSet          );
	theApp.WriteProfileInt("Sets",   "Song",   m_nSong         );

	// --- Master MIDI out device volume
	theApp.WriteProfileInt("MIDIOut", "Volume", m_dwMidiVolume );

	// --- Last song loaded
	theApp.WriteProfileString("Song", "Title", m_strLastSong);

	// --- Active Song Database Path
	theApp.WriteProfileString("Song", "Path", m_strSongPath);

	// --- Active PropertyPage
	theApp.WriteProfileInt("Pages", "Active", m_nActivePage);
}

int COptions::GetChannel(const char* pDevName, int nDefaultChannel) const
{
	int channel = 0;
	switch (*pDevName)	{
		case 'K':
			channel = m_nKeyboardChannel;
			break;
		case 'P':
			channel = m_nPianoChannel;
			break;
		case 'D':
			channel = m_nDrumsChannel;
			break;
		case 'B':
			channel = m_nBassChannel;
			break;
		case 'G':
			channel = m_nGuitarChannel;
			break;
		default:
			break;
	}
	return channel == -1 ? nDefaultChannel : channel;
}
int COptions::GetPatch(const char* pDevName, int nDefaultPatch) const
{
	int patch = 0;
	switch (*pDevName)	{
		case 'P':
			patch = m_nPianoPatch;
			break;
		case 'D':
			patch = m_nDrumsPatch;
			break;
		case 'B':
			patch = m_nBassPatch;
			break;
		case 'G':
			patch = m_nGuitarPatch;
			break;
		default:
			break;
	}
	return patch == -1 ? nDefaultPatch : patch;
}
int COptions::GetPan(const char* pDevName) const
{
	int pan = 0;
	switch (*pDevName)	{
		case 'P':
			pan = m_nPianoPan;
			break;
		case 'D':
			pan = m_nDrumsPan;
			break;
		case 'B':
			pan = m_nBassPan;
			break;
		case 'G':
			pan = m_nGuitarPan;
			break;
		default:
			break;
	}
	return pan;
}
int COptions::GetVolume(const char* pDevName) const
{
	int volume = 0;
	switch (*pDevName)	{
		case 'P':
			volume = m_nPianoVolume;
			break;
		case 'D':
			volume = m_nDrumsVolume;
			break;
		case 'B':
			volume = m_nBassVolume;
			break;
		case 'G':
			volume = m_nGuitarVolume;
			break;
		default:
			break;
	}
	return volume;
}
void COptions::SetChannel(const char* pDevName, int nChannel)
{
	switch (*pDevName)	{
		case 'K':
			m_nKeyboardChannel = nChannel;
			break;
		case 'P':
			m_nPianoChannel = nChannel;
			break;
		case 'D':
			m_nDrumsChannel = nChannel;
			break;
		case 'B':
			m_nBassChannel = nChannel;
			break;
		case 'G':
			m_nGuitarChannel = nChannel;
			break;
		default:
			break;
	}
}
void COptions::SetPatch(const char* pDevName, int nPatch)
{
	switch (*pDevName)	{
		case 'P':
			m_nPianoPatch = nPatch;
			break;
		case 'D':
			m_nDrumsPatch = nPatch;
			break;
		case 'B':
			m_nBassPatch = nPatch;
			break;
		case 'G':
			m_nGuitarPatch = nPatch;
			break;
		default:
			break;
	}
}
void COptions::SetPan(const char* pDevName, int nPan)
{
	switch (*pDevName)	{
		case 'P':
			m_nPianoPan = nPan;
			break;
		case 'D':
			m_nDrumsPan = nPan;
			break;
		case 'B':
			m_nBassPan = nPan;
			break;
		case 'G':
			m_nGuitarPan = nPan;
			break;
		default:
			break;
	}
}
void COptions::SetVolume(const char* pDevName, int nVolume)
{
	switch (*pDevName)	{
		case 'P':
			m_nPianoVolume = nVolume;
			break;
		case 'D':
			m_nDrumsVolume = nVolume;
			break;
		case 'B':
			m_nBassVolume = nVolume;
			break;
		case 'G':
			m_nGuitarVolume = nVolume;
			break;
		default:
			break;
	}
}
