// Page.cpp : implementation file
//

#include "stdafx.h"
#include "MidiFitz.h"
#include "Page.h"
#include "Sheet.h"

IMPLEMENT_DYNCREATE(CPage, CPropertyPage)

CSheet* CPage::m_pCSheet;

BEGIN_MESSAGE_MAP(CPage, CPropertyPage)
	// --- these entries implement context help
	ON_COMMAND(ID_HELP, OnHelp)
	ON_WM_HELPINFO()
	// --- this entry implements tooltips
	ON_NOTIFY_EX(TTN_NEEDTEXT, 0, OnToolTipNotify)
END_MESSAGE_MAP()

CPage::CPage(int id, char* pHelpKey) : CPropertyPage(id)
{
	m_nID = id;
	m_pCSheet = 0;
	m_bIsActive = FALSE;
	m_bIsInitialized = FALSE;
	m_pHelpKey = pHelpKey;
}

BOOL CPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	m_bIsInitialized = TRUE;
	EnableToolTips(TRUE);			// enable tooltips
	if (m_pCSheet == 0)	{
		m_pCSheet = (CSheet*)GetParent();
		m_pCSheet->Initialize();
	}
	return TRUE;
}

BOOL CPage::OnSetActive() 
{
	theApp.SetActivePage(this);
	m_bIsActive = TRUE;
	return CPropertyPage::OnSetActive();
}

BOOL CPage::OnKillActive() 
{
	m_bIsActive = FALSE;
	SaveSettings();
	return CPropertyPage::OnKillActive();
}

BOOL CPage::SaveSettings()
{
	if (::IsWindow(m_hWnd))	{	// might never have tabbed to page
		UpdateData();
		return TRUE;
	}
	return FALSE;
}

void CPage::OnOK()
{
#ifndef DEMO_VERSION
	if (m_pCSheet->CanExit())
#endif	
		CPropertyPage::OnOK();
}

// ------ called when F1 or ? icon is used on a control
BOOL CPage::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if (pHelpInfo->iContextType == HELPINFO_WINDOW)	{
		int nID = ::GetDlgCtrlID((HWND)(pHelpInfo->hItemHandle));
		theApp.WinHelp(nID + 0x10000, HELP_CONTEXTPOPUP);
		return TRUE;
	}
	return CPropertyPage::OnHelpInfo(pHelpInfo);
}

BOOL CPage::OnToolTipNotify(UINT, NMHDR* pNMHDR, LRESULT*)
{
	TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pNMHDR;
    if (pTTT->uFlags & TTF_IDISHWND)    {
        UINT nID = ::GetDlgCtrlID((HWND)pNMHDR->idFrom);
        pTTT->hinst = AfxGetResourceHandle();
        switch (nID)  {

			case IDC_RESET:
				pTTT->lpszText = "Reset settings";
				return(TRUE);
			case IDC_PIANOBUTTON:
				pTTT->lpszText = "Piano";
				return(TRUE);
			case IDC_DRUMSBUTTON:
				pTTT->lpszText = "Drums";
				return(TRUE);
			case IDC_BASSBUTTON:
				pTTT->lpszText = "Bass";
				return(TRUE);
			case IDC_GUITARBUTTON:
				pTTT->lpszText = "Guitar";
				return(TRUE);
			case IDC_PIANOPAN:
				pTTT->lpszText = "Piano balance";
				return(TRUE);
			case IDC_DRUMSPAN:
				pTTT->lpszText = "Drums balance";
				return(TRUE);
			case IDC_BASSPAN:
				pTTT->lpszText = "Bass balance";
				return(TRUE);
			case IDC_GUITARPAN:
				pTTT->lpszText = "Guitar balance";
				return(TRUE);
			case IDC_PIANOVOLUME:
				pTTT->lpszText = "Piano volume";
				return(TRUE);
			case IDC_DRUMSVOLUME:
				pTTT->lpszText = "Drums volume";
				return(TRUE);
			case IDC_BASSVOLUME:
				pTTT->lpszText = "Bass volume";
				return(TRUE);
			case IDC_GUITARVOLUME:
				pTTT->lpszText = "Guitar volume";
				return(TRUE);
			case IDC_COUNT:
	            pTTT->lpszText = "Countoff";
		        return(TRUE);
			case IDC_RECORD:
	            pTTT->lpszText = "Record";
		        return(TRUE);
			case IDC_STARTSTOP:
	            pTTT->lpszText = theSong.IsPlaying() ? "Stop playing" : "Tap out tempo";
		        return(TRUE);
			case IDC_NEW:
	            pTTT->lpszText = "New song";
		        return(TRUE);
			case IDC_OPEN:
	            pTTT->lpszText = "Open";
		        return(TRUE);
			case IDC_SAVE:
	            pTTT->lpszText = "Save";
		        return(TRUE);
			case IDC_BUILDMIDIFILE:
	            pTTT->lpszText = "Build SMF File (*.mid)";
		        return(TRUE);
			case IDC_CUT:
	            pTTT->lpszText = "Cut";
		        return(TRUE);
			case IDC_COPY:
	            pTTT->lpszText = "Copy";
		        return(TRUE);
			case IDC_PASTE:
	            pTTT->lpszText = "Paste";
		        return(TRUE);
			case IDC_UNDO:
	            pTTT->lpszText = "Undo";
		        return(TRUE);
			case IDC_PRINT:
	            pTTT->lpszText = "Print";
		        return(TRUE);
			case IDC_BEGINREPEAT:
	            pTTT->lpszText = "Begin repeat";
		        return(TRUE);
			case IDC_ENDREPEAT:
	            pTTT->lpszText = "End repeat";
		        return(TRUE);
			case IDC_PASSAGE:
	            pTTT->lpszText = "Passage";
		        return(TRUE);
			case IDC_STYLE:
	            pTTT->lpszText = "Song style";
		        return(TRUE);
			case IDC_ENDING:
	            pTTT->lpszText = "Ending";
		        return(TRUE);
			case IDC_QUARTERREST:
	            pTTT->lpszText = "Quarter note rest";
		        return(TRUE);
			case IDC_HALFREST:
	            pTTT->lpszText = "Half note rest";
		        return(TRUE);
			case IDC_WHOLEREST:
	            pTTT->lpszText = "Whole note rest";
		        return(TRUE);
			case IDC_PLAYBACK:
	            pTTT->lpszText = theSong.IsPlaying() ? "Stop" : "Play";
		        return(TRUE);
			case IDC_CUE:
	            pTTT->lpszText = "Cue past repeat";
		        return(TRUE);
			case IDC_UP:
	            pTTT->lpszText = "Move song up";
		        return(TRUE);
			case IDC_DOWN:
	            pTTT->lpszText = "Move song down";
		        return(TRUE);
			default:
				pTTT->lpszText = "";
				break;
        }
    }
    return(FALSE);
}

// ----- called when the Help button is clicked on the page
void CPage::OnHelp()
{
	theApp.WinHelp((long)m_pHelpKey, HELP_KEY);
}

void CPage::OnDestroy()
{
	WinHelp(0L, HELP_QUIT);
	CPropertyPage::OnDestroy();
}
