// Play.cpp : implementation file
//

#include "stdafx.h"
#include "options.h"
#include "MidiFitz.h"
#include "Play.h"
#include "drums.h"
#include "keyboard.h"
#include "Sheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPlay property page

IMPLEMENT_DYNCREATE(CPlay, CPage)

// --- convert tempo (quarter notes/minute) to
//     milleseconds per midi clock tick (24 ticks per quarter note)
inline UINT tmpo2ms(UINT tmpo)
{
	UINT ms = 1000 / (((tmpo) * 24) / 60);
	return ms;
}
// --- convert milleseconds between quarter notes to tempo (quarter notes/minute)
inline UINT ms2tmpo(UINT ms)
{
	return 60000U / ms;	// quarter notes per minute
}

CPlay::CPlay() : CPage(CPlay::IDD, "Play Page")
{
	m_psp.dwFlags |= PSP_HASHELP;
	//{{AFX_DATA_INIT(CPlay)
	m_nTempo = 0;
	m_nStyleButton = -1;
	//}}AFX_DATA_INIT
	m_bRecording = FALSE;
	m_nTempoCtr = 0;
	m_nTimer = 0;
	m_bCounting = FALSE;
	m_nCounting = 1;
}

CPlay::~CPlay()
{
}

void CPlay::DoDataExchange(CDataExchange* pDX)
{
	CPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPlay)
	DDX_Control(pDX, IDC_RECORD, m_RecordButton);
	DDX_Control(pDX, IDC_CHORD, m_Chord);
	DDX_Control(pDX, IDC_BEAT, m_Beat);
	DDX_Control(pDX, IDC_COUNT, m_CountOff);
	DDX_Control(pDX, IDC_STARTSTOP, m_StartButton);
	DDX_Control(pDX, IDC_PIANOBUTTON, m_PianoButton);
	DDX_Control(pDX, IDC_DRUMSBUTTON, m_DrumsButton);
	DDX_Control(pDX, IDC_BASSBUTTON, m_BassButton);
	DDX_Control(pDX, IDC_GUITARBUTTON, m_GuitarButton);
	DDX_Text(pDX, IDC_TEMPO, m_nTempo);
	DDX_Radio(pDX, IDC_SWING, m_nStyleButton);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPlay, CPage)
	//{{AFX_MSG_MAP(CPlay)
	ON_BN_CLICKED(IDC_STARTSTOP, OnStartstop)
	ON_BN_CLICKED(IDC_PIANOBUTTON, OnPianobutton)
	ON_BN_CLICKED(IDC_DRUMSBUTTON, OnDrumsbutton)
	ON_BN_CLICKED(IDC_BASSBUTTON, OnBassbutton)
	ON_BN_CLICKED(IDC_GUITARBUTTON, OnGuitarbutton)
	ON_BN_CLICKED(IDC_BALLAD, OnBallad)
	ON_BN_CLICKED(IDC_BOSSA, OnBossa)
	ON_BN_CLICKED(IDC_EASYBOSSA, OnEasybossa)
	ON_BN_CLICKED(IDC_DIXIE, OnDixie)
	ON_BN_CLICKED(IDC_COUNTRY, OnCountry)
	ON_BN_CLICKED(IDC_SHUFFLE, OnShuffle)
	ON_BN_CLICKED(IDC_JAZZWALTZ, OnJazzwaltz)
	ON_BN_CLICKED(IDC_SWING, OnSwing)
	ON_BN_CLICKED(IDC_TWOBEAT, OnTwobeat)
	ON_BN_CLICKED(IDC_WALTZ, OnWaltz)
	ON_BN_CLICKED(IDC_ROCK, OnRock)
	ON_BN_CLICKED(IDC_ROCKBALLAD, OnRockballad)
	ON_BN_CLICKED(IDC_COUNT, OnCount)
	ON_BN_CLICKED(IDC_RECORD, OnRecord)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CPlay::SaveSettings()
{
	if (CPage::SaveSettings())	{
		theOptions.m_nPianoPlaying = m_PianoButton.GetCheck();
		theOptions.m_nDrumsPlaying = m_DrumsButton.GetCheck();
		theOptions.m_nBassPlaying = m_BassButton.GetCheck();
		theOptions.m_nGuitarPlaying = m_GuitarButton.GetCheck();
		theOptions.m_nCountoff = m_nCounting;
		theOptions.m_nStyle = m_nStyleButton;
		return TRUE;
	}
	return FALSE;
}

void CPlay::ComputeExternalTempo()
{
	MMTIME mmtime;
	timeGetSystemTime(&mmtime, sizeof mmtime);
	DWORD now = mmtime.u.ms;

	if (now != m_dwPrevTime && m_dwPrevTime != 0)	{
		UINT delta = now-m_dwPrevTime;
		UINT tempo = ms2tmpo(delta);
		if (tempo != m_nTempo)	{
			m_nTempo = tempo;
			UpdateData(FALSE);
		}
	}
	m_dwPrevTime = now;
}

void CPlay::BuildMidiFile()
{
	CWaitCursor wait;
	theApp.SetBuildingMidiFile(true);
	theSong.SetPlayback(-1, -1);
	StartMsg();
	while (TimingMsg())
		;
	StopMsg();
	theApp.SetBuildingMidiFile(false);
	theApp.BuildMidiFile();
}

void CPlay::StartMsg()
{
	if (!theSong.IsPlaying() && !m_bCounting)	{

		if (theSong.IsWaltz())
			DisableFour();
		else
			DisableThree();

		if (!theSong.IsPlayback())	{
			theSong.InitializeSong();
			theSong.m_nTempo = m_nTempo;
		}
		else	{
			theSong.ResetChordOffset();
			m_nTempo = theSong.m_nTempo;
			if (m_bIsInitialized)
				UpdateData(FALSE);
		}

		MidiOut::StartMessage();

		m_nCountDown = 0;
		m_nClock = 0;
		m_nTimer = 0;
		m_dwPrevTime = 0;
		m_nTempoCtr = 0;
		m_PrevChord = CChord();
		m_nBeat = 0;
		m_nMeasure = 0;

		m_bCounting = m_nCounting != 0;
		m_bRecording = m_bIsInitialized ? (m_RecordButton.GetCheck() == 1) : FALSE;

		theSong.StartPlaying(m_bRecording);

		if (m_bIsInitialized)
			m_StartButton.SetIcon(theApp.m_hStopIcon);
		m_dwPrevTime = 0;
	}
}

static CPlay* pCPlay;

static int firstbeat = 0;

void CALLBACK TimerCallback(UINT, UINT, DWORD, DWORD, DWORD)
{
	if (firstbeat < 24)
		firstbeat++;
	else
		pCPlay->TimingMsg();
}

BOOL CPlay::TimingMsg()
{
	if (theSong.IsPlaying() || m_bCounting)	{
		MidiOut::TimingMessage();
		if (m_PrevChord != theSong.m_Chord)	{
			m_PrevChord = theSong.m_Chord;
			theSong.BuildChordDisplay(theApp.m_strChord, theSong.m_Chord);
			if (m_bIsInitialized)
				m_Chord.SetWindowText(theApp.m_strChord);
			m_pCSheet->DisplayChord();
		}

		if (m_nClock == 24)	{
			m_nClock = 0;
			if (++m_nBeat == (theSong.IsWaltz() ? 3 : 4))	{
				m_nBeat = 0;
				m_nMeasure++;
			}
		}
		if (m_nClock == 0)	{
			static char *bt[] = { "1 ", "2 ", "3 ", "4 " };
			CString strBeat;
			for (short int i = 0; i <= m_nBeat; i++)
				strBeat += bt[i];
			theApp.m_strBeat = strBeat;
			m_pCSheet->DisplayBeat();
			if (m_bCounting)
				m_pCSheet->DisplayCount(TRUE);
			if (m_bIsActive)
				m_Beat.SetWindowText(strBeat);
			if (m_bIsInitialized)	{
				if (theSong.IsPlayback() && 
						!theApp.m_bBuildingMidiFile && 
							theSong.StyleFromButton(m_nStyleButton) != theSong.m_nStyle)
					PostMessage(WM_COMMAND, ID_PUSHSTYLEBUTTON);
			}
		}
		else if (m_nClock == 12)	{
			if (m_nBeat == (theSong.IsWaltz() ? 2 : 3))	{
				theApp.m_strBeat = "";
				if (m_bIsActive)
					m_Beat.SetWindowText(theApp.m_strBeat);
				m_pCSheet->DisplayBeat();
			}
			if (m_bCounting)
				m_pCSheet->DisplayCount(FALSE);
		}

		if (m_bCounting)	{
			if (m_nClock == 0)	{
				if (m_nMeasure == 2)	{
					m_nMeasure = 0;
					m_bCounting = FALSE;
					theApp.m_pDrums->NoteOff(37, 0);
				}
				else if (m_nMeasure || (m_nBeat & 1) == 0 || theSong.IsWaltz())	{
					if (m_nMeasure || m_nBeat)
						theApp.m_pDrums->NoteOff(37, 0);
					if (m_nCounting == 1)
						theApp.m_pDrums->NoteOn(37, 100);
				}
			}
			theApp.m_dwMidiClock++;
		}
		if (!m_bCounting)	{
			Song::Style nOldStyle = theSong.m_nStyle;
			if (!theSong.TimingMessage(m_nBeat, m_nClock))	{
				if (!theApp.m_bBuildingMidiFile)	{
					theApp.ActivePage()->PostMessage(WM_COMMAND, IDC_STARTSTOP, BN_CLICKED);
					theApp.ActivePage()->PostMessage(WM_COMMAND, IDC_PLAYBACK,  BN_CLICKED);
				}
				return FALSE;
			}
			if (m_nClock == 0)	{
				if (nOldStyle != theSong.m_nStyle)
					m_pCSheet->DisplayStyle();
				m_pCSheet->Beat(m_nBeat);
			}
		}
		m_nClock++;
	}
	return TRUE;
}

void CPlay::StopMsg()
{
	if (theSong.IsPlaying() || m_bCounting)	{
		theSong.StopPlaying();
		MidiOut::StopMessage();
		m_bCounting = FALSE;
		m_nCountDown = 0;
		theApp.m_strBeat = "";
		theApp.m_strChord = "";
		m_pCSheet->DisplayBeat();
		m_pCSheet->DisplayChord();
		m_nTempoCtr = 0;
		m_nTempo = 0;
		m_dwPrevTime = 0;
		if (m_bIsInitialized)	{
			m_Beat.SetWindowText(theApp.m_strBeat);
			m_Chord.SetWindowText(theApp.m_strChord);
			m_StartButton.SetIcon(theSong.IsWaltz() ? theApp.m_hThreeFourIcon : theApp.m_hFourFourIcon);
			UpdateData(FALSE);
			PostMessage(WM_COMMAND, ID_PUSHSTYLEBUTTON);
		}
	}
	EnableAllStyles();
}

void CPlay::SetStyleFromButtons()
{
	switch (theSong.StyleFromButton(m_nStyleButton))	{
		case Song::fourfour:
			OnSwing();
			break;
		case Song::halftime:
			OnTwobeat();
			break;
		case Song::waltz:
			OnWaltz();
			break;
		case Song::bossa:
			OnBossa();
			break;
		case Song::jazzwaltz:
			OnJazzwaltz();
			break;
		case Song::ballad:
			OnBallad();
			break;
		case Song::rock:
			OnRock();
			break;
		case Song::rockballad:
			OnRockballad();
			break;
		case Song::dixie:
			OnDixie();
			break;
		case Song::country:
			OnCountry();
			break;
		case Song::easybossa:
			OnEasybossa();
			break;
		case Song::shuffle:
			OnShuffle();
			break;
		default:
			break;
	}
}

void CPlay::SetCountOffIcon()
{
	m_CountOff.SetIcon(m_nCounting == 0 ? theApp.m_hNoCountIcon :
					   m_nCounting == 1 ? theApp.m_hRimShotIcon :
					   theApp.m_hCountoffIcon);
}

/////////////////////////////////////////////////////////////////////////////
// CPlay message handlers

BOOL CPlay::OnInitDialog() 
{
	CPage::OnInitDialog();

	m_PianoButton.SetIcon(theApp.m_hPianoIcon);
	m_DrumsButton.SetIcon(theApp.m_hDrumsIcon);
	m_BassButton.SetIcon(theApp.m_hBassIcon);
	m_GuitarButton.SetIcon(theApp.m_hGuitarIcon);

	m_PianoButton.SetCheck(theOptions.m_nPianoPlaying);
	m_DrumsButton.SetCheck(theOptions.m_nDrumsPlaying);
	m_BassButton.SetCheck(theOptions.m_nBassPlaying);
	m_GuitarButton.SetCheck(theOptions.m_nGuitarPlaying);

	m_RecordButton.SetIcon(theApp.m_hRecordIcon);
	m_RecordButton.SetCheck(FALSE);
	
	m_nCounting = theOptions.m_nCountoff;
	SetCountOffIcon();

	m_nStyleButton = theOptions.m_nStyle;

	UpdateData(FALSE);

	SetStyleFromButtons();

	EnableAllStyles();
	ResetFocus();

	return FALSE;
}

void CPlay::PlaySong(int nFirstMeasure, int nLastMeasure) 
{
	m_nTempoCtr = theSong.IsWaltz() ? 5 : 3; 
	theSong.SetPlayback(nFirstMeasure, nLastMeasure);
	OnStartstop();
}

void CPlay::OnStartstop()
{
	static UINT ms = 0;

	if (m_nTempoCtr == 0 && m_nTimer == 0)	{
		theSong.BuildDummySong();
		SetStyleFromButtons();
	}
	int counts = theSong.IsWaltz() ? 6 : 4;
	if (!theSong.IsPlaying() && !m_bCounting && m_nTempoCtr < counts)	{
		if (!theSong.IsPlayback())
			ComputeExternalTempo();
		if (++m_nTempoCtr == counts)	{
			if (!theSong.IsPlayback())	{
				if (m_nTempo < 64)	{
					m_nTempoCtr = 0;
					m_dwPrevTime = 0;
					m_bCounting = FALSE;
					m_nTempo = 0;
					UpdateData(FALSE);
					return;
				}
				if (m_nTempo > MaxTempo)	{
					m_nTempo = MaxTempo;
					UpdateData(FALSE);
				}
			}

			StartMsg();

			ms = tmpo2ms(m_nTempo);
			pCPlay = this;
			firstbeat = 0;

			timeGetDevCaps(&m_TimeCaps, sizeof m_TimeCaps);
			timeBeginPeriod(m_TimeCaps.wPeriodMin);
			m_nTimer = timeSetEvent(ms, m_TimeCaps.wPeriodMin, TimerCallback, 0, TIME_PERIODIC);
		}
	}
	else if (!theSong.IsPlayback())	
		theSong.PadLastMeasure();
	else if (m_nTimer != 0)	{
		timeKillEvent(m_nTimer);
		m_nTimer = 0;
		timeEndPeriod(m_TimeCaps.wPeriodMin);
		ms = 0;
		StopMsg();
		if (m_bIsActive)	{
			if (!m_bRecording)
				theSong.BuildDummySong();
			theApp.SetMouse(m_StartButton);
		}
	}
}

void CPlay::OnPianobutton() 
{
	theOptions.m_nPianoPlaying = m_PianoButton.GetCheck();
	ResetFocus();
}

void CPlay::OnDrumsbutton() 
{
	theOptions.m_nDrumsPlaying = m_DrumsButton.GetCheck();
	ResetFocus();
}

void CPlay::OnBassbutton() 
{
	theOptions.m_nBassPlaying = m_BassButton.GetCheck();
	ResetFocus();
}

void CPlay::OnGuitarbutton() 
{
	theOptions.m_nGuitarPlaying = m_GuitarButton.GetCheck();
	ResetFocus();
}

void CPlay::DoFourFour()
{
	if (!theSong.IsPlaying())
		m_StartButton.SetIcon(theApp.m_hFourFourIcon);
	UpdateData(TRUE);
	ResetFocus();
}
void CPlay::OnRockballad() 
{
	theSong.SetStyle(Song::rockballad);
	DoFourFour();
}

void CPlay::OnBallad() 
{
	theSong.SetStyle(Song::ballad);
	DoFourFour();
}
void CPlay::OnRock() 
{
	theSong.SetStyle(Song::rock);
	DoFourFour();
}

void CPlay::OnBossa() 
{
	theSong.SetStyle(Song::bossa);
	DoFourFour();
}

void CPlay::OnEasybossa() 
{
	theSong.SetStyle(Song::easybossa);
	DoFourFour();
}

void CPlay::OnCountry() 
{
	theSong.SetStyle(Song::country);
	DoFourFour();
}

void CPlay::OnDixie() 
{
	theSong.SetStyle(Song::dixie);
	DoFourFour();
}

void CPlay::OnSwing() 
{
	theSong.SetStyle(Song::fourfour);
	DoFourFour();
}

void CPlay::OnTwobeat() 
{
	theSong.SetStyle(Song::halftime);
	DoFourFour();
}

void CPlay::OnShuffle()
{
	theSong.SetStyle(Song::shuffle);
	DoFourFour();
}

void CPlay::DoWaltz()
{
	if (!theSong.IsPlaying())
		m_StartButton.SetIcon(theApp.m_hThreeFourIcon);
	UpdateData(TRUE);
	ResetFocus();
}

void CPlay::OnWaltz() 
{
	theSong.SetStyle(Song::waltz);
	DoWaltz();
}

void CPlay::OnJazzwaltz() 
{
	theSong.SetStyle(Song::jazzwaltz);
	DoWaltz();
}

void CPlay::OnCount() 
{
	if (++m_nCounting == 3)
		m_nCounting = 0;
	theOptions.m_nCountoff = m_nCounting;
	SetCountOffIcon();
	ResetFocus();
}

void CPlay::OnRecord() 
{
	ResetFocus();
}

// ----- disable 3/4 styles while song playing in 4/4
void CPlay::DisableThree()
{
	if (m_bIsInitialized)	{
		GetDlgItem(IDC_WALTZ)->EnableWindow(FALSE);
		GetDlgItem(IDC_JAZZWALTZ)->EnableWindow(FALSE);
	}
}

// ----- disable 4/4 styles while song playing in 3/4
void CPlay::DisableFour()
{
	if (m_bIsInitialized)	{
		GetDlgItem(IDC_SWING)->EnableWindow(FALSE);
		GetDlgItem(IDC_TWOBEAT)->EnableWindow(FALSE);
		GetDlgItem(IDC_BOSSA)->EnableWindow(FALSE);
		GetDlgItem(IDC_DIXIE)->EnableWindow(FALSE);
		GetDlgItem(IDC_BALLAD)->EnableWindow(FALSE);
		GetDlgItem(IDC_ROCK)->EnableWindow(FALSE);
		GetDlgItem(IDC_ROCKBALLAD)->EnableWindow(FALSE);
		GetDlgItem(IDC_COUNTRY)->EnableWindow(FALSE);
		GetDlgItem(IDC_EASYBOSSA)->EnableWindow(FALSE);
		GetDlgItem(IDC_SHUFFLE)->EnableWindow(FALSE);
	}
}

// ---- enable all styles while no song is playing
void CPlay::EnableAllStyles()
{
	if (m_bIsInitialized)	{
		GetDlgItem(IDC_WALTZ)->EnableWindow();
		GetDlgItem(IDC_SWING)->EnableWindow();
		GetDlgItem(IDC_TWOBEAT)->EnableWindow();
		GetDlgItem(IDC_BOSSA)->EnableWindow();
		GetDlgItem(IDC_ROCK)->EnableWindow();
		GetDlgItem(IDC_COUNTRY)->EnableWindow();
		GetDlgItem(IDC_SHUFFLE)->EnableWindow();
		GetDlgItem(IDC_DIXIE)->EnableWindow();
		GetDlgItem(IDC_EASYBOSSA)->EnableWindow();
		GetDlgItem(IDC_JAZZWALTZ)->EnableWindow();
		GetDlgItem(IDC_BALLAD)->EnableWindow();
		GetDlgItem(IDC_ROCKBALLAD)->EnableWindow();
	}
}

BOOL CPlay::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if (wParam == ID_PUSHSTYLEBUTTON)	{
		m_nStyleButton = theSong.ButtonFromStyle(theSong.m_nStyle);
		UpdateData(FALSE);
		return TRUE;
	}
	return CPage::OnCommand(wParam, lParam);
}

