// ------- songundo.h

#ifndef SONGUNDO_H
#define SONGUNDO_H

#include "undo.h"
#include "song.h"
#include "midicord.h"

// --------------------------------------------------------------------------
class UndoInsertEvents : public UndoInsertNode<Song, Song::Event>	{
public:
	UndoInsertEvents(Song::Event* pEvent, int nCount, BOOL bStophere) : 
				UndoInsertNode<Song, Song::Event>(theSong, pEvent, nCount, bStophere)
		{	}
};
// --------------------------------------------------------------------------
class UndoDeleteEvents : public UndoDeleteNode<Song, Song::Event>	{
public:
	UndoDeleteEvents(Song::Event* pEvent, int nCount, BOOL bStophere) : 
				UndoDeleteNode<Song, Song::Event>(theSong, pEvent, nCount, bStophere)
		{	}
};
// --------------------------------------------------------------------------
class UndoReplaceEvent : public UndoReplaceNode<Song::Event>	{
public:
	UndoReplaceEvent(Song::Event* pEvent, BOOL bStophere) : 
				UndoReplaceNode<Song::Event>(pEvent, bStophere)
		{	}
};
class UndoReplaceKey : public UndoReplaceNode<CChord>	{
public:
	UndoReplaceKey(CChord& rChord) : 
				UndoReplaceNode<CChord>(&rChord)
		{	}
};
class UndoReplaceTempo : public UndoReplaceNode<int>	{
public:
	UndoReplaceTempo(int nTempo) : 
				UndoReplaceNode<int>(&nTempo)
		{	}
};
class UndoReplaceTitle : public UndoReplaceNode<CString>	{
public:
	UndoReplaceTitle(CString& strTitle) : 
				UndoReplaceNode<CString>(&strTitle)
		{	}
	virtual void Undo()
	{
		theSong.ReplaceTitle(m_Data);
		UndoReplaceNode<CString>::Undo();
	}
};
class UndoReplaceTranspose : public UndoReplaceNode<short int>	{
public:
	UndoReplaceTranspose(short int nDelta) : 
				UndoReplaceNode<short int>(&nDelta, FALSE)
		{	}
	virtual void Undo()
	{
		theSong.ReplaceTranspose(m_Data);
	}
};
// --------------------------------------------------------------------------
class UndoSongData : public UndoData	{
public:
	UndoSongData() { }
	void AddInsertMeasureUndo(Song::Event* pEvent, int nCount, BOOL bStophere = TRUE)
	{
		AddUndoNode(new UndoInsertEvents(pEvent, nCount, bStophere));
	}
	void AddDeleteMeasureUndo(Song::Event* pEvent, int nCount, BOOL bStophere = TRUE)
	{
		AddUndoNode(new UndoDeleteEvents(pEvent, nCount, bStophere));
	}
	void AddReplaceEventUndo(Song::Event* pEvent, BOOL bStophere)
	{
		AddUndoNode(new UndoReplaceEvent(pEvent, bStophere));
	}
	void AddReplaceKeyUndo(CChord& rKey)
	{
		AddUndoNode(new UndoReplaceKey(rKey));
	}
	void AddReplaceTempoUndo(int rTempo)
	{
		AddUndoNode(new UndoReplaceTempo(rTempo));
	}
	void AddReplaceTitleUndo(CString& strTitle)
	{
		AddUndoNode(new UndoReplaceTitle(strTitle));
	}
	void AddReplaceTransposeUndo(short int delta)
	{
		AddUndoNode(new UndoReplaceTranspose(delta));
	}
};

#endif

