// Sets.cpp : implementation file
//

#include "stdafx.h"
#include <io.h>
#include "midifitz.h"
#include "Sheet.h"
#include "song.h"
#include "Sets.h"
#include "DlgSetName.h"
#include "Options.h"
#include "LeadSheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSets property page

IMPLEMENT_DYNCREATE(CSets, CPage)

CSets::CSets() : CPage(CSets::IDD, "Sets Page")
{
	//{{AFX_DATA_INIT(CSets)
	m_Key = _T("");
	m_Tempo = _T("");
	//}}AFX_DATA_INIT
	m_psp.dwFlags |= PSP_HASHELP;
}

CSets::~CSets()
{
}

void CSets::DoDataExchange(CDataExchange* pDX)
{
	CPage::DoDataExchange(pDX);
#ifdef DEMO_VERSION
	DDX_Control(pDX, IDC_ADDSONG, m_AddSong);
	DDX_Control(pDX, IDC_NEWSET, m_NewSet);
	DDX_Control(pDX, IDC_DELSONG, m_DelSong);
	DDX_Control(pDX, IDC_DELSET, m_DelSet);
#endif
	//{{AFX_DATA_MAP(CSets)
	DDX_Control(pDX, IDC_STYLEDISPLAY, m_Style);
	DDX_Control(pDX, IDC_BEATDISPLAY, m_Beat);
	DDX_Control(pDX, IDC_CHORDDISPLAY, m_Chord);
	DDX_Control(pDX, IDC_UP, m_Up);
	DDX_Control(pDX, IDC_DOWN, m_Down);
	DDX_Control(pDX, IDC_CUE, m_Cue);
	DDX_Control(pDX, IDC_PLAYBACK, m_Playback);
	DDX_Control(pDX, IDC_SONGS, m_SongList);
	DDX_Control(pDX, IDC_SETS, m_SetList);
	DDX_Control(pDX, IDC_PRINT, m_Print);
	DDX_Text(pDX, IDC_KEYDISPLAY, m_Key);
	DDX_Text(pDX, IDC_TEMPODISPLAY, m_Tempo);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CSets, CPage)
#ifndef DEMO_VERSION
	ON_BN_CLICKED(IDC_NEWSET, OnNewset)
	ON_BN_CLICKED(IDC_DELSET, OnDelset)
	ON_BN_CLICKED(IDC_ADDSONG, OnAddsong)
	ON_BN_CLICKED(IDC_DELSONG, OnDelsong)
	ON_BN_CLICKED(IDC_DOWN, OnDown)
	ON_BN_CLICKED(IDC_UP, OnUp)
#endif
	//{{AFX_MSG_MAP(CSets)
	ON_LBN_SELCHANGE(IDC_SETS, OnSelchangeSets)
	ON_LBN_SELCHANGE(IDC_SONGS, OnSelchangeSongs)
	ON_BN_CLICKED(IDC_PLAYBACK, OnPlayback)
	ON_BN_CLICKED(IDC_CUE, OnCue)
	ON_BN_CLICKED(IDC_PRINT, OnPrint)
	ON_LBN_DBLCLK(IDC_SONGS, OnDblclkSongs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CSets::BuildSetList()
{
	m_SetList.ResetContent();
	struct _finddata_t fileinfo;
	CString strPath = theOptions.m_strSongPath + "*.set";
	long hFile = _findfirst(strPath.GetBuffer(0), &fileinfo);

	while (hFile != -1)	{
		CString strName(fileinfo.name);
		m_SetList.AddString(strName.Left(strName.GetLength()-4));
		if (_findnext(hFile, &fileinfo) == -1)
			hFile = -1;
	}
}

void CSets::BuildSongList(int nSong)
{
	m_SongList.ResetContent();
	int nSel = m_SetList.GetCurSel();
	int nSongCt = 0;
	if (nSel != -1)	{
		CString strSetName;
		m_SetList.GetText(nSel, strSetName);
		strSetName += ".set";
		strSetName = theOptions.m_strSongPath + strSetName;
		try	{
			CStdioFile setFile(strSetName.GetBuffer(0), CFile::modeRead);
			CString strSongTitle;
			while (setFile.ReadString(strSongTitle))	{
				m_SongList.AddString(strSongTitle);
				nSongCt++;
			}
		}
		catch (CFileException)	{
			AfxMessageBox("Set file is missing or unreadable", MB_ICONSTOP);
		}
	}
	m_SongList.SetCurSel(nSongCt ? nSong : -1);
	OnSelchangeSongs();
}

void CSets::WriteSongList()
{
	int nSel = m_SetList.GetCurSel();
	if (nSel != -1)	{
		CString strSetName;
		m_SetList.GetText(nSel, strSetName);
		strSetName += ".set";
		strSetName = theOptions.m_strSongPath + strSetName;
		try	{
			CStdioFile setFile(strSetName.GetBuffer(0), CFile::modeCreate | CFile::modeWrite);
			int nCt = m_SongList.GetCount();
			CString strSongName;
			for (int nSong = 0; nSong < nCt; nSong++)	{
				m_SongList.GetText(nSong, strSongName);
				strSongName += "\n";
				setFile.WriteString(strSongName.GetBuffer(0));
			}
			setFile.Flush();
		}
		catch (CFileException)	{
			AfxMessageBox("Cannot build set file", MB_ICONSTOP);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSets message handlers

BOOL CSets::OnInitDialog() 
{
	CPage::OnInitDialog();

	m_Up.SetIcon(theApp.m_hUpIcon);
	m_Down.SetIcon(theApp.m_hDownIcon);
	m_Playback.SetIcon(theApp.m_hPlayIcon);
	m_Cue.SetIcon(theApp.m_hCueIcon);
	m_Cue.EnableWindow(FALSE);
	m_Print.SetIcon(theApp.m_hPrintIcon);
#ifdef DEMO_VERSION
	m_AddSong.EnableWindow(FALSE);
	m_NewSet.EnableWindow(FALSE);
	m_DelSong.EnableWindow(FALSE);
	m_DelSet.EnableWindow(FALSE);
	m_Up.EnableWindow(FALSE);
	m_Down.EnableWindow(FALSE);
#endif
	return TRUE;
}

void CSets::DisplayChord()
{
	if (m_bIsActive)
		m_Chord.SetWindowText(theApp.m_strChord); 
}
void CSets::DisplayBeat()
{
	if (m_bIsActive)
		m_Beat.SetWindowText(theApp.m_strBeat); 
}
void CSets::DisplayStyle()
{
	if (m_bIsActive)
		m_Style.SetWindowText(theSong.StyleName(theSong.m_nStyle));
}
void CSets::InitializeSongPanel()
{
	m_Chord.SetWindowText(""); 
	m_Beat.SetWindowText(""); 
	m_Tempo.Format("%d", theSong.m_nTempo);
	m_Style.SetWindowText(theSong.StyleName(theSong.m_nStyle));
	theSong.BuildChordDisplay(m_Key, theSong.m_Key);
}

BOOL CSets::OnSetActive()
{
	BuildSetList();
	m_SetList.SetCurSel(theOptions.m_nSet);
	BuildSongList(theOptions.m_nSong);

	OnSelchangeSongs();
	InitializeSongPanel();
	if (theSong.ChordCount() != 0)	{
		m_Playback.EnableWindow();
		if (theSong.IsPlaying())
			m_Playback.SetIcon(theApp.m_hStopIcon);
		else
			m_Playback.SetIcon(theApp.m_hPlayIcon);
	}
	else
		m_Playback.EnableWindow(FALSE);
	return CPage::OnSetActive();
}

BOOL CSets::OnKillActive() 
{
	return CPage::OnKillActive();
}

BOOL CSets::SaveSettings()
{
	if (CPage::SaveSettings())	{
		theOptions.m_nSet  = m_SetList.GetCurSel();
		theOptions.m_nSong = m_SongList.GetCurSel();
		return TRUE;
	}
	return FALSE;
}

#ifndef DEMO_VERSION
void CSets::OnNewset() 
{
	CDlgSetName dlg;
	if (dlg.DoModal() == IDOK)	{
		CString strName(theOptions.m_strSongPath + dlg.m_strSetName);
		strName += ".set";
		if (_access(strName.GetBuffer(0), 0) == 0)
			AfxMessageBox("Set already exists", MB_ICONSTOP);
		else	{
			m_SetList.SetCurSel(m_SetList.AddString(dlg.m_strSetName));
			CFile setFile(strName.GetBuffer(0), CFile::modeCreate);
			setFile.Close();
			setFile.Flush();
			m_SongList.ResetContent();
			OnAddsong();
		}
	}
	FocusOnList();
}
void CSets::OnDelset() 
{
	int nSel = m_SetList.GetCurSel();
	if (nSel != -1)	{
		CString strSetName;
		m_SetList.GetText(nSel, strSetName);
		strSetName = theOptions.m_strSongPath + strSetName + ".set";
		if (AfxMessageBox("Delete the set?", MB_YESNO | MB_ICONQUESTION) == IDYES)	{
			try	{
				CFile::Remove(strSetName.GetBuffer(0));
				BuildSetList();
				if (nSel == m_SetList.GetCount())
					--nSel;
				m_SetList.SetCurSel(nSel);
				BuildSongList(0);
			}
			catch (CFileException)	{
				AfxMessageBox("Can't delete the set", MB_ICONSTOP);
			}
		}
	}
	FocusOnList();
}

void CSets::OnAddsong() 
{
	if (m_SetList.GetCurSel() == -1)
		AfxMessageBox("Select a set first", MB_ICONSTOP);
	else	{
		CDlgAddSong dlg;
		if (dlg.DoModal() == IDOK)	{
			int nSel = m_SongList.GetCurSel();
			m_SongList.InsertString(nSel, dlg.m_strTitle);
			WriteSongList();
		}
		FocusOnList();
	}
}

void CSets::OnDelsong() 
{
	int nSel = m_SongList.GetCurSel();
	if (nSel != -1)	{
		if (AfxMessageBox("Delete the song from the set?", MB_YESNO | MB_ICONQUESTION) == IDYES)	{
			m_SongList.DeleteString(nSel);		
			if (nSel == m_SongList.GetCount())
				--nSel;
			m_SongList.SetCurSel(nSel);
			WriteSongList();
		}
	}
	FocusOnList();
}

void CSets::OnDown() 
{
	int nSel = m_SongList.GetCurSel();
	if (nSel != -1 && nSel < m_SongList.GetCount() - 1)	{
		CString strSongName;
		m_SongList.GetText(nSel, strSongName);
		m_SongList.DeleteString(nSel++);		
		m_SongList.InsertString(nSel, strSongName);
		m_SongList.SetCurSel(nSel);
		WriteSongList();
	}
	m_SongList.SetFocus();
}

void CSets::OnUp() 
{
	int nSel = m_SongList.GetCurSel();
	if (nSel > 0)	{
		CString strSongName;
		m_SongList.GetText(nSel, strSongName);
		m_SongList.DeleteString(nSel);		
		m_SongList.InsertString(--nSel, strSongName);
		m_SongList.SetCurSel(nSel);
		WriteSongList();
	}
	m_SongList.SetFocus();
}

#endif

void CSets::FocusOnList()
{
	m_SongList.SetFocus();
	theApp.SetMouse(m_Playback);
}

void CSets::OnSelchangeSets() 
{
	BuildSongList(0);
}

void CSets::OnSelchangeSongs() 
{
	int nSel = m_SongList.GetCurSel();
	if (nSel != -1)	{
		CString strSongName;
		m_SongList.GetText(nSel, strSongName);
		if (theSong.LoadSong(strSongName))	{
			InitializeSongPanel();
			m_Playback.EnableWindow();
			UpdateData(FALSE);
		}
	}
}

void CSets::OnPlayback() 
{
	if (theSong.IsPlaying())
		m_pCSheet->StopSong();
	else
		m_pCSheet->PlaySong();
}

void CSets::OnCue() 
{
	theSong.Cue();
	m_Cue.EnableWindow(FALSE);
	FocusOnList();
}

void CSets::StartPlaying()
{
	if (m_bIsActive)	{
		m_Playback.SetIcon(theApp.m_hStopPlayIcon);
		m_Playback.SetFocus();
		theApp.SetMouse(m_Playback);
	}
}

void CSets::StopPlaying()
{
	if (m_bIsActive)	{
		m_Playback.SetIcon(theApp.m_hPlayIcon);
		m_Cue.EnableWindow(FALSE);
		int nSel = m_SongList.GetCurSel();
		if (nSel != -1)	{
			if (nSel == m_SongList.GetCount() - 1)
				m_SongList.SetCurSel(0);
			else
				m_SongList.SetCurSel(nSel+1);
			OnSelchangeSongs();
		}
		FocusOnList();
	}
}

BOOL CSets::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if (wParam == ID_ENABLECUE)	{
		m_Cue.EnableWindow();
		m_Cue.SetFocus();
		theApp.SetMouse(m_Cue);
		return TRUE;
	}
	return CPage::OnCommand(wParam, lParam);
}


void CSets::OnDblclkSongs() 
{
	OnPlayback();
}

void CSets::OnPrint() 
{
	if (m_SongList.GetCount() == 0)
		return;

	PRINTDLG printDlg = {
		sizeof(PRINTDLG),
		0,
		0,
		0,
		0,
		PD_RETURNDC | PD_USEDEVMODECOPIES | PD_NOSELECTION | PD_NOPAGENUMS
	};
	if (PrintDlg(&printDlg))	{

		UpdateWindow();

		CDC* pPrinterDC = CDC::FromHandle(printDlg.hDC);

		if (pPrinterDC->GetDeviceCaps(TECHNOLOGY) != DT_RASPRINTER)	{
			AfxMessageBox("Graphics printer required", MB_ICONSTOP);
			return;
		}

		CWaitCursor wait;

		// --- this window is not displayed. It is used to build the print pages.
		CLeadSheet* pCLeadSheet = new CLeadSheet();
		pCLeadSheet->Initialize(this);

		// ---- get rid of vertical scroll style, which displays for some reason.
		pCLeadSheet->ModifyStyle(WS_VSCROLL, 0);

		DOCINFO DocInfo = {
			sizeof(DOCINFO),
			theSong.m_strTitle,
			0
		};

		if (pPrinterDC->StartDoc(&DocInfo) != -1)	{
			int nSongs = m_SongList.GetCount();
			int* pPageNos = new int[nSongs];

			int nPageno = 1, pagey = 0;
			pPrinterDC->StartPage();
			int nSel;
			for (nSel = 0; nSel < nSongs; nSel++)	{
				m_SongList.SetCurSel(nSel);
				OnSelchangeSongs();
				pCLeadSheet->BuildSong();
				pCLeadSheet->PrintSong(pPrinterDC, nPageno, pPageNos[nSel], pagey);
			}
			pCLeadSheet->BottomOfPage(pPrinterDC, nPageno);

			// ---- print the table of contents		

			// ------ 1/2" margins
			int lmargin = pPrinterDC->GetDeviceCaps(LOGPIXELSX) / 2;
			int tmargin = pPrinterDC->GetDeviceCaps(LOGPIXELSY) / 2;

			CString strDots("...........................................................");
			CSize size = pPrinterDC->GetTextExtent(strDots);

			const int maxlines = 99;
			int y;
			int linect = maxlines;

			CString strSetName;
			m_SetList.GetText(m_SetList.GetCurSel(), strSetName);

			for (nSel = 0; nSel < nSongs; nSel++)	{
				if (linect > 40)	{
					if (linect != 99)
						pPrinterDC->EndPage();
					pPrinterDC->StartPage();
					pPrinterDC->TextOut(lmargin, tmargin, strSetName);
					linect = 0;
					y = size.cy * 2;
				}
				linect++;
				CString strSongName;
				m_SongList.GetText(nSel, strSongName);
				strSongName += strDots.Left(strDots.GetLength() - strSongName.GetLength());
				CString strPageNo;
				strPageNo.Format("%d", pPageNos[nSel]);
				strSongName += strPageNo;
				pPrinterDC->TextOut(lmargin, tmargin+y, strSongName);
				y += size.cy;
			}

			pPrinterDC->EndPage();

			delete [] pPageNos;

			pPrinterDC->EndDoc();
		}
		delete pCLeadSheet;
		m_SongList.SetCurSel(0);
		OnSelchangeSongs();
	}
}

