// Settings.cpp : implementation file
//

#include "stdafx.h"
#include "options.h"
#include "Settings.h"
#include "piano.h"
#include "drums.h"
#include "bass.h"
#include "guitar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSettings property page

IMPLEMENT_DYNCREATE(CSettings, CPage)

CSettings::CSettings() : CPage(CSettings::IDD, "Settings Page")
{
	//{{AFX_DATA_INIT(CSettings)
	m_bAdjustBass = FALSE;
	//}}AFX_DATA_INIT
	m_psp.dwFlags |= PSP_HASHELP;
}

CSettings::~CSettings()
{
}

void CSettings::DoDataExchange(CDataExchange* pDX)
{
	CPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSettings)
	DDX_Control(pDX, IDC_PIANOVOLUME, m_PianoVolume);
	DDX_Control(pDX, IDC_PIANOPAN, m_PianoPan);
	DDX_Control(pDX, IDC_DRUMSVOLUME, m_DrumsVolume);
	DDX_Control(pDX, IDC_GUITARVOLUME, m_GuitarVolume);
	DDX_Control(pDX, IDC_DRUMSPAN, m_DrumsPan);
	DDX_Control(pDX, IDC_BASSVOLUME, m_BassVolume);
	DDX_Control(pDX, IDC_BASSPAN, m_BassPan);
	DDX_Control(pDX, IDC_GUITARPAN, m_GuitarPan);
	DDX_Check(pDX, IDC_ADJUSTBASS, m_bAdjustBass);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSettings, CPage)
	//{{AFX_MSG_MAP(CSettings)
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_RESET, OnSettingsReset)
	ON_BN_CLICKED(IDC_ADJUSTBASS, OnAdjustbass)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSettings message handlers

const int panticfreq = 13;
const int volticfreq = 13;

void CSettings::InitializeControls()
{
	if (m_bIsInitialized)	{
		m_PianoPan.SetPos(theOptions.m_nPianoPan);
		m_PianoVolume.SetPos(127-theOptions.m_nPianoVolume);
		m_DrumsPan.SetPos(theOptions.m_nDrumsPan);
		m_DrumsVolume.SetPos(127-theOptions.m_nDrumsVolume);
		m_BassPan.SetPos(theOptions.m_nBassPan);
		m_BassVolume.SetPos(127-theOptions.m_nBassVolume);
		m_GuitarPan.SetPos(theOptions.m_nGuitarPan);
		m_GuitarVolume.SetPos(127-theOptions.m_nGuitarVolume);
	}
}

BOOL CSettings::OnInitDialog() 
{
	m_bAdjustBass = theOptions.m_bAdjustBass;
	CPage::OnInitDialog();

	m_PianoPan.SetRange(0, 127);
	m_PianoPan.SetTicFreq(panticfreq);

	m_PianoVolume.SetRange(0, 127);
	m_PianoVolume.SetTicFreq(volticfreq);

	m_DrumsPan.SetRange(0, 127);
	m_DrumsPan.SetTicFreq(panticfreq);

	m_DrumsVolume.SetRange(0, 127);
	m_DrumsVolume.SetTicFreq(volticfreq);

	m_BassPan.SetRange(0, 127);
	m_BassPan.SetTicFreq(panticfreq);

	m_BassVolume.SetRange(0, 127);
	m_BassVolume.SetTicFreq(volticfreq);

	m_GuitarPan.SetRange(0, 127);
	m_GuitarPan.SetTicFreq(panticfreq);

	m_GuitarVolume.SetRange(0, 127);
	m_GuitarVolume.SetTicFreq(volticfreq);

	InitializeControls();

/*
	theApp.m_pPiano->SetPan(theOptions.m_nPianoPan);
	theApp.m_pPiano->SetVolume(theOptions.m_nPianoVolume);

	theApp.m_pDrums->SetPan(theOptions.m_nDrumsPan);
	theApp.m_pDrums->SetVolume(theOptions.m_nDrumsVolume);

	theApp.m_pBass->SetPan(theOptions.m_nBassPan);
	theApp.m_pBass->SetVolume(theOptions.m_nBassVolume);
*/
	return TRUE;
}

BOOL CSettings::SaveSettings()
{
	if (CPage::SaveSettings())	{
		UpdateData();
		theOptions.m_nPianoPan = m_PianoPan.GetPos();
		theOptions.m_nPianoVolume = 127-m_PianoVolume.GetPos();

		theOptions.m_nDrumsPan = m_DrumsPan.GetPos();
		theOptions.m_nDrumsVolume = 127-m_DrumsVolume.GetPos();

		theOptions.m_nBassPan = m_BassPan.GetPos();
		theOptions.m_nBassVolume = 127-m_BassVolume.GetPos();

		theOptions.m_nGuitarPan = m_GuitarPan.GetPos();
		theOptions.m_nGuitarVolume = 127-m_GuitarVolume.GetPos();

		theOptions.m_bAdjustBass = m_bAdjustBass;

		return TRUE;
	}
	return FALSE;
}

void CSettings::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
	midiOutDevice* pDevice;
	switch (pScrollBar->GetDlgCtrlID())	{
		case IDC_PIANOPAN:
			pDevice = theApp.m_pPiano;
			break;
		case IDC_DRUMSPAN:
			pDevice = theApp.m_pDrums;
			break;
		case IDC_BASSPAN:
			pDevice = theApp.m_pBass;
			break;
		case IDC_GUITARPAN:
			pDevice = theApp.m_pGuitar;
			break;
		default:
			return;
	}
	pDevice->SetPan(((CSliderCtrl*)pScrollBar)->GetPos());
}

void CSettings::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
	midiOutDevice* pDevice;
	switch (pScrollBar->GetDlgCtrlID())	{
		case IDC_PIANOVOLUME:
			pDevice = theApp.m_pPiano;
			break;
		case IDC_DRUMSVOLUME:
			pDevice = theApp.m_pDrums;
			break;
		case IDC_BASSVOLUME:
			pDevice = theApp.m_pBass;
			break;
		case IDC_GUITARVOLUME:
			pDevice = theApp.m_pGuitar;
			break;
		default:
			return;
	}
	pDevice->SetVolume(127-((CSliderCtrl*)pScrollBar)->GetPos());
}

void CSettings::OnSettingsReset() 
{
	m_PianoPan.SetPos(64);
	theApp.m_pPiano->SetPan(64);
	m_PianoVolume.SetPos(127-96);
	theApp.m_pPiano->SetVolume(96);
	m_DrumsPan.SetPos(64);
	theApp.m_pDrums->SetPan(64);
	m_DrumsVolume.SetPos(127-96);
	theApp.m_pDrums->SetVolume(96);
	m_BassPan.SetPos(64);
	theApp.m_pBass->SetPan(64);
	m_BassVolume.SetPos(127-96);
	theApp.m_pBass->SetVolume(96);
	m_GuitarPan.SetPos(64);
	theApp.m_pGuitar->SetPan(64);
	m_GuitarVolume.SetPos(127-96);
	theApp.m_pGuitar->SetVolume(96);
}

void CSettings::OnAdjustbass() 
{
	UpdateData();
	theOptions.m_bAdjustBass = m_bAdjustBass;
}
