// Sheet.cpp : implementation file
//

#include "stdafx.h"
#include <mmsystem.h>
#include "options.h"
#include "Sheet.h"
#include "piano.h"
#include "keyboard.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(CSheet, CPropertySheet)
	//{{AFX_MSG_MAP(CSheet)
	ON_WM_SYSCOMMAND()
	ON_MESSAGE(MM_MIM_DATA, OnMIDIMessage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CSheet::CSheet(LPCTSTR pszCaption)
	:CPropertySheet(pszCaption, 0, 0)
{
	m_psh.dwFlags |= PSH_NOAPPLYNOW; 
	m_psh.dwFlags |= PSH_HASHELP;
	AddPage(&m_CPlay);
	AddPage(&m_CSettings);
	AddPage(&m_CMidiPage);
	AddPage(&m_CSongPage);
	AddPage(&m_CSets);
	AddPage(&m_CAbout);
	m_bExitEnabled = TRUE;
	m_pExitWnd = 0;
	SetActivePage(theOptions.m_nActivePage);
}

CSheet::~CSheet()
{
}

void CSheet::Initialize()
{
	ModifyStyleEx(0, WS_EX_CONTEXTHELP);

    CWnd* pWnd = GetDlgItem(IDCANCEL);
    if (pWnd != 0)  {
        pWnd->ShowWindow(SW_HIDE);
        pWnd->EnableWindow(FALSE);
    }

    m_pExitWnd = GetDlgItem(IDOK);
    if (m_pExitWnd != 0)	{
        m_pExitWnd->SetWindowText("Exit");

		CRect rc;
		GetClientRect(&rc);
		
		CRect brc;
		m_pExitWnd->GetWindowRect(&brc);

		CRect nrc;
		nrc.left = (rc.right - brc.Width() * 2) - 12;
		nrc.top = (rc.bottom - brc.Height()) - 7;
		nrc.right = nrc.left + brc.Width(); 
		nrc.bottom = nrc.top + brc.Height();

		m_pExitWnd->MoveWindow(nrc);
	}

	theApp.m_pMidiIn->RegisterWindow(m_hWnd);
	theApp.SetDevices();
}

BOOL CSheet::SaveSettings()
{
	m_CPlay.SaveSettings();
	m_CSettings.SaveSettings();
	m_CMidiPage.SaveSettings();
	m_CSongPage.SaveSettings();
	m_CSets.SaveSettings();
	theOptions.m_nActivePage = GetActiveIndex();
	return TRUE;
}

void CSheet::StopPlaying()
{
	CMenu* pCMenu = GetSystemMenu(FALSE);
	ASSERT(pCMenu != 0);
	pCMenu->EnableMenuItem(SC_CLOSE, MF_ENABLED);
	m_bExitEnabled = TRUE;
    m_pExitWnd->EnableWindow(TRUE);
	m_CSongPage.StopPlaying();
	m_CSets.StopPlaying();
}

void CSheet::StartPlaying()
{
	CMenu* pCMenu = GetSystemMenu(FALSE);
	ASSERT(pCMenu != 0);
	pCMenu->EnableMenuItem(SC_CLOSE, MF_GRAYED);
	m_bExitEnabled = FALSE;
    m_pExitWnd->EnableWindow(FALSE);
	m_CSongPage.StartPlaying();
	m_CSets.StartPlaying();
}

#ifndef DEMO_VERSION
BOOL CSheet::CanExit()
{
	if (m_bExitEnabled)
		if (m_CSongPage.SaveSong(TRUE))	{
			SaveSettings();
			return TRUE;
		}
	return FALSE;
}
#endif

/////////////////////////////////////////////////////////////////////////////
// CSheet message handlers
#ifndef DEMO_VERSION
void CSheet::OnSysCommand(UINT nID, LPARAM lParam) 
{
	if (nID != SC_CLOSE || CanExit())
		CPropertySheet::OnSysCommand(nID, lParam);
}
#endif

LONG CSheet::OnMIDIMessage(WPARAM, LPARAM msg)
{
	// --- extract MIDI message components
	short int velocity = ((msg >> 16) & 0xff);
	short unsigned note = ((msg >> 8) & 0xff);
	short unsigned status = msg & 0xf0;
	short unsigned channel = msg & 0x0f;
	if (channel == theApp.m_pKeyboard->GetChannel() - 1)	{
		BOOL bPassThrough = theOptions.m_nPianoPlaying == 1;
		switch (status)	{
			case 0xb0:		// MIDI status = controller change
				if (bPassThrough)
					theApp.m_pPiano->ControllerChange(note, velocity);
				break;
			case 0x80:		// MIDI status = note off
				if (!theSong.IsPlayback())
					theApp.m_pKeyboard->NoteOff(note);
				if (bPassThrough)
					theApp.m_pPiano->NoteOff(note, velocity);
				m_CMidiPage.NoteOff(note);
				velocity = 0;
			case 0x90:		// MIDI status = note on
				if (!theSong.IsPlayback())
					velocity ? theApp.m_pKeyboard->NoteOn(note) : theApp.m_pKeyboard->NoteOff(note);
				if (bPassThrough)
					theApp.m_pPiano->NoteOn(note, velocity);
				velocity ? m_CMidiPage.NoteOn(note) : m_CMidiPage.NoteOff(note);
				break;
			default:
				break;
		}
	}
	return 0;
}


