// ---- song.cpp

#include "stdafx.h"
#include <io.h>
#include "MidiFitz.h"
#include "song.h"
#include "midicord.h"
#include "songundo.h"
#include "piano.h"
#include "drums.h"
#include "bass.h"
#include "options.h"
#include "page.h"

Song theSong;

Song::Event Song::m_aEvents[maxchords];
CString Song::m_strDefaultTitle(UNTITLED);

IMPLEMENT_SERIAL(Song, CObject, 0)

// --- maps song styles to style buttons on the dialogs
Song::Style Song::m_aButtonMap[] = {
	Song::fourfour, 
	Song::halftime, 
	Song::rock,
	Song::country,
	Song::waltz, 
	Song::bossa, 
	Song::rockballad,
	Song::easybossa,
	Song::jazzwaltz, 
	Song::ballad,
	Song::dixie,
	Song::shuffle

};

int Song::ButtonFromStyle(Song::Style nStyle) const
{
	for (int i = 0; i < sizeof(m_aButtonMap) / sizeof(Style); i++)
		if (m_aButtonMap[i] == nStyle)
			return i;
	return 0;
}

// --- maps style names and notations to the song styles
Song::StyleText Song::m_Styles[] = {
	{ "",            ""				},
	{ "Swing",       "in 4"			},
	{ "In Two",      "in 2"			},
	{ "Rock",        "rock"			},
	{ "Country",     "country"		},
	{ "Waltz",       "3/4"			},
	{ "Bossa",       "bossa-1"		},
	{ "Rock Ballad", "rock ballad"  },
	{ "Easy Bossa",  "bossa-2"		},
	{ "Jazz 3/4",    "jazz 3/4"		},
	{ "Ballad",      "ballad"		},
	{ "Dixie",       "dixie"		},
	{ "Shuffle",     "shuffle"		},

};

Song::Song()
{
	m_bPlayback = m_bWasPlayback = FALSE;
	m_bRecording = FALSE;
	m_bPlaying = FALSE;
	m_pUndoSongData = new UndoSongData;
	InitializeSong();
}
Song::~Song()
{
	delete m_pUndoSongData;
}
void Song::BuildDummySong()
{
	InitializeSong();
	m_aEvents[0].m_nStyleChange = m_nStyle = m_nNewStyle = fourfour;
	SetEvent(0, style);
	ResetChanged();			// removes the SetEvent undo event
	m_nChordCount = 4 * 32;
	m_Chord.root = 4;
	m_Chord.attr = CChord::major;
	m_Key = m_Chord;
	ASSERT(m_nChordCount < maxchords);
	for (unsigned int i = 0; i < m_nChordCount; i++)
		m_aEvents[i].m_Chord = m_Chord;
	theOptions.m_strLastSong.Empty();
}

void Song::ResetChanged()
{ 	
	ASSERT(m_pUndoSongData != 0);
	m_pUndoSongData->DeleteAllUndoActions();
}

void Song::SetStyle(Style nStyle)
{
	m_nNewStyle = nStyle;
	if (!m_bPlaying)	{
		m_nStyle = nStyle;
		m_nBeatsPerMeasure = IsWaltz() ? 3 : 4;
	}
}
void Song::SetStyle(int nMeasure, Style nStyle)
{
	if (CurrentStyle(nMeasure) != nStyle)	{
		unsigned int nChord = nMeasure * m_nBeatsPerMeasure;
		ASSERT(nChord < maxchords);
		SetEvent(nChord, style);
		m_aEvents[nChord].m_nStyleChange = nStyle;
		if (nMeasure == 0)
			SetStyle(nStyle);
		// --- look ahead to see if this style change is the same as a later one.
		while (nChord < m_nChordCount - m_nBeatsPerMeasure)	{
			nChord += m_nBeatsPerMeasure;
			ASSERT(nChord < maxchords);
			if ((m_aEvents[nChord].m_nEventType & style) != 0)	{
				if (m_aEvents[nChord].m_nStyleChange != nStyle)
					break;
				ClearEvent(nChord, style, FALSE);
				m_aEvents[nChord].m_nStyleChange = nostyle;
			}
		}
	}
}
Song::Style Song::CurrentStyle(int nMeasure) const
{
	int nChord = nMeasure * m_nBeatsPerMeasure;
	ASSERT(nChord < maxchords);
	while (nChord >= 0)	{
		if ((m_aEvents[nChord].m_nEventType & style) != 0)
			return m_aEvents[nChord].m_nStyleChange;
		--nChord;
	}
	return fourfour;
}
void Song::InitializeSong()
{
	ResetChanged();
	m_strTitle = UNTITLED;
	m_nTempo = 120;
	m_nBeatsPerMeasure = 4;
	m_nChordCount = 0;
	m_nCurrentChord = 0;
	m_nCurrentMeasure = 0;
	m_nFirstChord = 0;
	m_nLastChord = 0;
	m_nNextChord = 0;
	m_nBeat = 0;
	m_nClock = 0;
	m_nRepeatTo = 0;
	memset(m_aEvents, 0, sizeof(m_aEvents));
	m_Key = CChord();
	m_bRepeat = FALSE;
	m_bCued = FALSE;
	m_bLastMeasure = FALSE;
	m_nMeasureCount = 0;
	m_nPlayingEnding = 0;
	m_nRepeatMeasure = 0;
}

void Song::ResetChordOffset()
{
	m_nNextChord = m_nFirstChord;
}
void Song::PostChord(const CChord& rChord)
{
	if (!m_bPlaying)
		theApp.ChordEvent(rChord);
	m_Chord = rChord;
	// --- if this chord was struck in the first half of a beat while recording, 
	//     post the chord to the previous beat in the chord table
	if (m_bPlaying && !m_bPlayback && m_nClock < 12 && m_nChordCount)	{
		ASSERT(m_nChordCount <= maxchords);
		m_aEvents[m_nChordCount-1].m_Chord = m_Chord;
	}
}
void Song::BuildChordDisplay(CString& cstr, const midiChord& rChord, BOOL bAbbrev)
{
	if (rChord.root == 0)	{
		cstr = "";
		return;
	}
    static char *notes[] = {
		"A", "Bb","B", "C", "Db","D",
   		"Eb","E", "F", "F#","G", "Ab"
	};
	cstr = notes[(rChord.root&0xf)-1];
	if (rChord.diminished)
		cstr += bAbbrev ? "o" : "dim";	// \xb0
	else if (rChord.augmented)
		cstr += bAbbrev ? "+" : "aug";
	else	{
		if (rChord.minor)
			cstr += bAbbrev ? "\x96" : "mi";
		if (rChord.seventh)
			cstr += "7";
		else if (rChord.major7)
			cstr += bAbbrev ? "w" : "Maj7";
	}
	if (rChord.flat5)
		cstr += "b5";
	int nBassNote = (rChord.root>>4)&0xf;
	if (nBassNote)	{
		cstr += "/";
		cstr += notes[nBassNote-1];
	}
}
void Song::StartPlaying(BOOL bRecording)
{
	m_bLastMeasure = FALSE;
	m_bPlaying = TRUE;
	m_bRecording = bRecording && !m_bPlayback;
	m_nBeatsPerMeasure = IsWaltz() ? 3 : 4;
	m_nMeasureCount = m_nChordCount / m_nBeatsPerMeasure;
	m_bRepeat = FALSE;
	if (!m_bPlayback)
		m_bCued = FALSE;
	m_nRepeatTo = 0;
	m_nPlayingEnding = 0;
	m_nRepeatMeasure = 0;
	m_nCurrentMeasure = 0;
	m_nCurrentChord = 0;
	if (m_bRecording)	{
		memset(m_aEvents, 0, sizeof(m_aEvents));
		m_aEvents[0].m_nStyleChange = m_nStyle = m_nNewStyle;
		m_aEvents[0].m_nEventType = style;
		m_Key = CChord();
	}
	else if (m_bPlayback)
		m_nStyle = m_nNewStyle = CurrentStyle(m_nFirstChord / m_nBeatsPerMeasure);
	theApp.StartPlaying();
}
void Song::SetPlayback(int nFirstMeasure, int nLastMeasure)
{
	m_bCued = FALSE;
	if (nFirstMeasure == -1)
		m_nFirstChord = 0;
	else	{
		// starting from other than at the beginning
		m_nFirstChord = nFirstMeasure * m_nBeatsPerMeasure;
		// see if an end repeat is in the block without its begin repeat
		for (unsigned int nEvent = m_nFirstChord; nEvent < m_nChordCount; nEvent += m_nBeatsPerMeasure)	{
			ASSERT(nEvent < maxchords);
			EventType evType = m_aEvents[nEvent].m_nEventType;
			if (evType & beginrepeat)
				break;
			if (evType & endrepeat)	{
				m_bCued = TRUE;
				break;
			}
		}
	}
	if (nLastMeasure == -1)
		m_nLastChord = m_nChordCount;
	else
		m_nLastChord = (nLastMeasure + 1) * m_nBeatsPerMeasure;
	m_bPlayback = m_bWasPlayback = TRUE;
}
BOOL Song::TimingMessage(int beat, int clock)
{
	m_nClock = clock;
	m_nBeat = beat;

	if (clock == 0)	{
		// --- on the beat
		if (beat == 0 && m_nNewStyle != m_nStyle)	{
			// --- manual style change
			m_nStyle = m_nNewStyle;
			m_nBeatsPerMeasure = IsWaltz() ? 3 : 4;
			if (m_bRecording && !m_bPlayback && m_nChordCount < maxchords)	{
				// --- recording, put new style at end of program
				ASSERT(m_nChordCount < maxchords);
				m_aEvents[m_nChordCount].m_nStyleChange = m_nStyle;
				m_aEvents[m_nChordCount].m_nEventType = style;
			}
		}
		if (m_bPlayback)	{
			// --- in playback mode
			if (m_nNextChord >= m_nLastChord)
				// --- end of chord table = end of song
				return FALSE;	// let caller decide to stop playing

			if (beat == 0)	{
				// --- at the beginning of a measure to play
				EventType evType = m_aEvents[m_nNextChord].m_nEventType;

				if (evType & ending)	{
					// --- the measure has an ending
					int nEndingChord = m_nNextChord;
					ASSERT(m_nNextChord < maxchords);
					int nEnding = m_aEvents[m_nNextChord].m_nEndingNumber;
					// --- scan to the next unplayed measure
					//     following the current ending being played
					while (nEnding <= m_nPlayingEnding)	{
						// --- scan to the end repeat if any
						BOOL bFoundEndRepeat = TRUE;
						do {
							m_nNextChord += m_nBeatsPerMeasure;
							if (m_nNextChord >= m_nLastChord + 1)	{
								bFoundEndRepeat = FALSE;
								break;
							}
							evType = m_aEvents[m_nNextChord].m_nEventType;
						} while ((evType & endrepeat) == 0);

						// --- one measure past the end repeat
						m_nNextChord += m_nBeatsPerMeasure;
						ASSERT(m_nNextChord < maxchords);

						if (bFoundEndRepeat && m_nPlayingEnding != 0 && !(m_aEvents[m_nNextChord].m_nEventType & ending))	{
							//     playing an ending,
							//     and found an end repeat with no following ending
							m_nNextChord = nEndingChord;
							m_nPlayingEnding = m_aEvents[m_nNextChord].m_nEndingNumber;
							evType = m_aEvents[m_nNextChord].m_nEventType;
							goto breakout;
						}

						if (m_nNextChord >= m_nLastChord + 1)	{
							// --- went past the end of the song, no end repeat was found
							m_nNextChord = m_nLastChord;
							m_nPlayingEnding = 0;
							m_nRepeatMeasure = 0;
							m_bCued = FALSE;
							return FALSE;
						}

						// --- set the next ending number, if any
						nEnding = m_aEvents[m_nNextChord].m_nEndingNumber;
						evType = m_aEvents[m_nNextChord].m_nEventType;

					}

					m_nPlayingEnding = nEnding;
					m_nRepeatMeasure = m_nNextChord;

				}

breakout:
				ASSERT(m_nNextChord < maxchords);
				if (m_aEvents[m_nNextChord].m_nEventType & style)	{
					if (m_aEvents[m_nNextChord].m_nStyleChange != nostyle)	{
						// --- style change event
						m_nStyle = m_nNewStyle = m_aEvents[m_nNextChord].m_nStyleChange;
						m_nBeatsPerMeasure = IsWaltz() ? 3 : 4;
					}
				}


				m_nCurrentMeasure = m_nNextChord / m_nBeatsPerMeasure;
				m_bLastMeasure = (m_nCurrentMeasure == m_nMeasureCount - 1);

				// --- beat 0 records begin and end repeat events

				if (evType & beginrepeat)	{
					// --- remember the offset to beat 0 of the begin repeat event
					m_nRepeatTo = m_nNextChord;
					if (m_nRepeatMeasure < m_nNextChord)	{
						m_nPlayingEnding = 0;
						m_nRepeatMeasure = 0;
					}
					if (!m_bCued && !theApp.m_bBuildingMidiFile)	{
						// ---- scan for endrepeat with no following ending and enable Cue if so
						int nEvent = m_nNextChord;
						while (nEvent <= m_nLastChord)	{
							ASSERT(nEvent < maxchords);
							if (m_aEvents[nEvent].m_nEventType & endrepeat)	{
								if (nEvent == m_nLastChord || (m_aEvents[nEvent+m_nBeatsPerMeasure].m_nEventType & ending) == 0)	{
									theApp.ActivePage()->PostMessage(WM_COMMAND, ID_ENABLECUE);
									break;
								}
							}
							nEvent += m_nBeatsPerMeasure;
						}
					}
				}
				if (evType & endrepeat)	{
					if (m_nPlayingEnding || !theApp.m_bBuildingMidiFile)	{
						// --- post the event to be sensed after the last beat of this measure
						m_bRepeat = TRUE;
						if (!m_bCued)
							m_bLastMeasure = FALSE;
					}
				}
			}
			// --- use the next chord from the chord table
			m_nCurrentChord = m_nNextChord;
			ASSERT(m_nNextChord < maxchords);
			m_Chord = m_aEvents[m_nNextChord++].m_Chord;

			if (beat == m_nBeatsPerMeasure - 1 && m_bRepeat)	{
				// --- last beat of a measure where an end repeat event was sensed
				m_bRepeat = FALSE;
				if (!m_bCued || (m_nNextChord < m_nLastChord && (m_aEvents[m_nNextChord].m_nEventType & ending)))
					// --- if not cued or the repeat is followed by an ending, 
					//     post the begin repeat measure offset as the next chord to play
					m_nNextChord = m_nRepeatTo;
				else
					m_bCued = FALSE;
			}
		}
		else	{
			m_nCurrentMeasure = m_nChordCount / m_nBeatsPerMeasure;
			if (m_bRecording)	{
				// --- recording, append the current chord to the program
				ASSERT(m_nChordCount < maxchords);
				m_aEvents[m_nChordCount].m_Chord = m_Chord;
				if (m_nChordCount < maxchords)
					m_nChordCount++;
				else
					m_nChordCount = 0;
			}
			else	{
				m_nChordCount++;
				if (m_nChordCount == 96)	// magic number
					m_nChordCount = 0;
			}
		}
	}
	theApp.TimingMessage(m_nCurrentMeasure, beat, clock);
	return TRUE;
}

BOOL Song::Validate(unsigned int& nEvent, CString& rstrErrMsg)
{
	if (m_nChordCount == 0)
		return FALSE;
	int nPEnding = 0;
	int nBeginRepeat = -1;
	int nEndRepeat = -1;
	for (nEvent = 0; nEvent < m_nChordCount; nEvent += m_nBeatsPerMeasure)	{
		ASSERT(nEvent < maxchords);
		EventType evType = m_aEvents[nEvent].m_nEventType;
		if (evType & beginrepeat)	{
			nBeginRepeat = nEvent;
			nPEnding = 0;
		}
		if (evType & ending)	{
			int nEnding = m_aEvents[nEvent].m_nEndingNumber;
			if (nEnding == 1 && nBeginRepeat == -1)	{
				rstrErrMsg = "Ending with no begin repeat";
				return FALSE;
			}
			if (nEnding != nPEnding + 1)	{
				rstrErrMsg = "Ending sequence error";
				return FALSE;
			}
			if (nEnding > 1 && (m_aEvents[nEvent-m_nBeatsPerMeasure].m_nEventType & endrepeat) == 0)	{
				rstrErrMsg = "Endings 2,3 should follow measure with end repeat";
				return FALSE;
			}
			nPEnding = nEnding;
		}
		if (evType & endrepeat)	{
			nEndRepeat = nEvent;
			if (nBeginRepeat == -1)	{
				rstrErrMsg = "End repeat with no begin repeat";
				return FALSE;
			}
		}
	}
	if (nPEnding == 1)	{
		nEvent = nPEnding;
		rstrErrMsg = "Only one ending";
		return FALSE;
	}
	if (nBeginRepeat > nEndRepeat)	{
		nEvent = nBeginRepeat;
		rstrErrMsg = "Begin repeat with no end repeat";
		return FALSE;
	}
	return TRUE;
}

void Song::PadLastMeasure()
{
	if (m_nChordCount || !m_bRecording)	{
		if (!m_bRecording)	{
			m_aEvents[0].m_Chord = m_Chord;
			m_nChordCount = 1;
			m_nNextChord = m_nBeat + 1;
		}
		else
			m_nNextChord = m_nChordCount;
		// --- pad the last measure with the last chord
		while ((m_nChordCount % m_nBeatsPerMeasure) != 0)	{
			ASSERT(m_nChordCount < maxchords);
			m_aEvents[m_nChordCount].m_Chord = m_aEvents[m_nChordCount-1].m_Chord;
			m_nChordCount++;
		}
		m_bPlayback = TRUE;		// recording, but forces the playback of the last measure
		m_bRecording = FALSE;
		m_bLastMeasure = TRUE;
		m_nLastChord = m_nChordCount;
	}
}
void Song::StopPlaying()
{
	m_Chord = CChord();
	theApp.StopPlaying();
	m_bPlayback = FALSE;
	m_bPlaying = FALSE;
}

void Song::TransposeNote(short int& note, short int delta)
{
	note += delta;
	if (note < 1)
		note += 12;
	if (note > 12)
		note -= 12;
}

void Song::TransposeChord(CChord& rChord, short int delta)
{
	short int bass = (rChord.root >> 4) & 0xf;
	short int root = (rChord.root) & 0xf;
	if (bass)
		TransposeNote(bass, delta);
	TransposeNote(root, delta);
	rChord.root = (bass << 4) | root;
}

void Song::Transpose(const CChord& NewKey)
{
	short int delta = NewKey.root - m_Key.root;
	for (unsigned int i = 0; i < m_nChordCount; i++)	{
		ASSERT(i < maxchords);
		TransposeChord(m_aEvents[i].m_Chord, delta);
	}
	SetKey(NewKey);
	m_pUndoSongData->AddReplaceTransposeUndo(delta);
}
// ---- called from undo to undo transposition
void Song::ReplaceTranspose(short int nDelta)
{
	for (unsigned int i = 0; i < m_nChordCount; i++)	{
		ASSERT(i < maxchords);
		TransposeChord(m_aEvents[i].m_Chord, -nDelta);
	}
}
void Song::Serialize(CArchive& archive)
{
	CObject::Serialize(archive);
#ifndef DEMO_VERSION
	if(archive.IsStoring())	{
		// --- store the MidiFitz signature
		archive << CString("MidiFitz Song File");
		// --- store the patches for this song
		archive << theApp.m_pPiano->GetPatch();
		archive << theApp.m_pDrums->GetPatch();
		archive << theApp.m_pBass->GetPatch();

		// --- store the data for this song
		m_Key.attr &= (CChord::major | CChord::minor);
		archive << m_Key;
		archive << (int)m_nStyle;
		archive << m_nTempo;
		archive << m_nChordCount;
		for (unsigned int i = 0; i < m_nChordCount; i++)	{
			ASSERT(i < maxchords);
			archive << m_aEvents[i].m_Chord;
			archive << (int)m_aEvents[i].m_nEventType;
			archive << (int)m_aEvents[i].m_nStyleChange;
			archive << (int)m_aEvents[i].m_nEndingNumber;
		}
		archive.Flush();
		m_pUndoSongData->DeleteAllUndoActions();
	}
	else
#endif
	{
		CString strSignature;
		archive >> strSignature;
		if (strSignature != "MidiFitz Song File")	{
			AfxMessageBox("Not a MidiFitz song file", MB_ICONSTOP);
			return;
		}

		InitializeSong();
		// --- retrieve the patches for this song
		int nPatch;
		archive >> nPatch;
		theApp.m_pPiano->SetPatch(nPatch);
		archive >> nPatch;
		theApp.m_pDrums->SetPatch(nPatch);
		archive >> nPatch;
		theApp.m_pBass->SetPatch(nPatch);

		// --- have the app tell the pages to initialize their controls for the newly loaded song
		theApp.InitializeControls();

		// --- retrieve the title for this song (implied by the file name)

		CFile* pCFile = archive.GetFile();
		m_strTitle = pCFile->GetFileTitle();
		int ndx = m_strTitle.Find('.');
		if (ndx != -1)
			m_strTitle = m_strTitle.Left(ndx);

		// --- retrieve the musical data for this song

		archive >> m_Key;
		int nStyle;
		archive >> nStyle;
		m_nStyle = (Style) nStyle;
		archive >> m_nTempo;
		archive >> m_nChordCount;
		for (unsigned int i = 0; i < m_nChordCount; i++)	{
			ASSERT(i < maxchords);
			archive >> m_aEvents[i].m_Chord;
			int nType, nStyle;
			archive >> nType;
			archive >> nStyle;
			m_aEvents[i].m_nEventType = (EventType)nType;
			m_aEvents[i].m_nStyleChange = (Style)nStyle;
			archive >> m_aEvents[i].m_nEndingNumber;
		}
		m_nBeatsPerMeasure = IsWaltz() ? 3 : 4;
		m_nStyle = m_nNewStyle = CurrentStyle(0);
	}
}
void Song::SetTitle(const CString& strTitle)
{
	ReplaceTitle(strTitle);
	m_pUndoSongData->AddReplaceTitleUndo(m_strTitle);
	m_strTitle = strTitle;
}

// ------- called from undo, too
void Song::ReplaceTitle(CString strTitle)
{
	if (strTitle != UNTITLED)	{
		CString strOldFilename = theOptions.m_strSongPath + m_strTitle + ".sng";
		if (_access(strOldFilename, 0) == 0)	{
			CString strNewFilename = theOptions.m_strSongPath + strTitle + ".sng";
			CFile::Rename(strOldFilename, strNewFilename);
		}
	}
}

void Song::SetKey(const CChord& Key)
{
	m_pUndoSongData->AddReplaceKeyUndo(m_Key);
	m_Key = Key;
}
void Song::SetTempo(int nTempo)
{
	m_pUndoSongData->AddReplaceTempoUndo(m_nTempo);
	m_nTempo = nTempo;
}
void Song::ReplaceChord(int nChordNo, const CChord& rChord, BOOL bStophere)
{
	ASSERT(nChordNo < maxchords);
	m_pUndoSongData->AddReplaceEventUndo(m_aEvents + nChordNo, bStophere);
	m_aEvents[nChordNo].m_Chord = rChord;
}
void Song::SetEvent(int nEventNo, EventType nType, BOOL bStophere)
{
	ASSERT(nEventNo < maxchords);
	m_pUndoSongData->AddReplaceEventUndo(m_aEvents + nEventNo, bStophere);
	m_aEvents[nEventNo].m_nEventType = (EventType)((int)m_aEvents[nEventNo].m_nEventType | (int)nType);
}
void Song::ClearEvent(int nEventNo, EventType nType, BOOL bStophere)
{
	ASSERT(nEventNo < maxchords);
	m_pUndoSongData->AddReplaceEventUndo(m_aEvents + nEventNo, bStophere);
	m_aEvents[nEventNo].m_nEventType = (EventType)((int)m_aEvents[nEventNo].m_nEventType & ~(int)nType);
}
void Song::DeleteChord(unsigned int nChordNo)
{
	if (nChordNo > 0)	{
		ASSERT(nChordNo < maxchords);
		CChord oldChord = m_aEvents[nChordNo].m_Chord;
		CChord newChord = m_aEvents[nChordNo-1].m_Chord;
		if (oldChord != newChord)	{
			BOOL bStophere = TRUE;
			do	{
				m_pUndoSongData->AddReplaceEventUndo(m_aEvents + nChordNo, bStophere);
				bStophere = FALSE;
				ASSERT(nChordNo < maxchords);
				m_aEvents[nChordNo].m_Chord = newChord;
				nChordNo++;
			} while (nChordNo+1 < m_nChordCount && ((nChordNo+1) % m_nBeatsPerMeasure) && m_aEvents[nChordNo+1].m_Chord == oldChord);
		}
	}
}
// ------ clipboard interface
void Song::DeleteMeasures(int nMeasureNo, int nCount, BOOL bStophere)
{
	int nFirstChord = nMeasureNo * m_nBeatsPerMeasure;
	int nChords = nCount * m_nBeatsPerMeasure;
	ASSERT(nFirstChord < maxchords);
	m_pUndoSongData->AddDeleteMeasureUndo(m_aEvents + nFirstChord, nChords, bStophere);
	Delete(m_aEvents + nFirstChord, nChords);
}
void Song::InsertMeasures(int nMeasureNo, int nCount, Event* pEvent, BOOL bStophere)
{
	unsigned int nInsertChord = nMeasureNo * m_nBeatsPerMeasure;
	Style nStyle = CurrentStyle(nInsertChord);
	int nChords = nCount * m_nBeatsPerMeasure;
	ASSERT(nInsertChord < maxchords);
	Insert(m_aEvents+nInsertChord, nChords, pEvent);
	m_pUndoSongData->AddInsertMeasureUndo(m_aEvents+nInsertChord, nChords, bStophere);
	 
	while (nInsertChord < m_nChordCount)	{
		ASSERT(nInsertChord < maxchords);
		if ((m_aEvents[nInsertChord].m_nEventType & style) != 0)	{
			m_pUndoSongData->AddReplaceEventUndo(m_aEvents + nInsertChord, FALSE);
			if (m_aEvents[nInsertChord].m_nStyleChange != nStyle)
				nStyle = m_aEvents[nInsertChord].m_nStyleChange;
			else	{
				m_aEvents[nInsertChord].m_nEventType = (EventType)((int)m_aEvents[nInsertChord].m_nEventType & ~(int)style);
				m_aEvents[nInsertChord].m_nStyleChange = nostyle;
			}
		}
		nInsertChord += m_nBeatsPerMeasure;
	}
}
// ------- undo interface
void Song::Insert(Event* pInsertAt, int nCount, Event* pEvent)
{
	if (m_nChordCount + nCount <= maxchords)	{
		int nEvent = pInsertAt - m_aEvents;
		int i;
		for (i = m_nChordCount - 1; i >= nEvent; --i)	{
			ASSERT(i < maxchords);
			ASSERT(i + nCount < maxchords);
			m_aEvents[i + nCount] = m_aEvents[i];
		}
		for (i = 0; i < nCount; i++)
			*pInsertAt++ = *pEvent++;
		m_nChordCount += nCount;
	}
}
void Song::Delete(Event* pDeleteAt, int nCount)
{
	int nEvent = pDeleteAt - m_aEvents;
	int nLeft = m_nChordCount - nEvent - nCount;
	for (int i = 0; i < nLeft; i++)
		*(pDeleteAt + i) = *(pDeleteAt + i + nCount);
	m_nChordCount -= nCount;
}

void Song::UndoLastAction()
{
	m_pUndoSongData->UndoLastAction();
}
BOOL Song::IsUndoDataStored() const
{
	return m_pUndoSongData->IsUndoDataStored();
}

BOOL Song::LoadSong(const CString& strTitle)
{
	CFile theFile;
	CString strFilename = theOptions.m_strSongPath + strTitle + ".sng";
	BOOL bRtn = theFile.Open(strFilename, CFile::modeRead | CFile::shareDenyWrite);
	if (!bRtn)
		AfxMessageBox("Cannot open " + strFilename, MB_ICONSTOP);
	else	{
		CArchive archive(&theFile, CArchive::load);
		Serialize(archive);
		theOptions.m_strLastSong = strTitle;
	}
	return bRtn;
}

#ifndef DEMO_VERSION
void Song::SaveSong()
{
	CFile theFile;
	CString strFilename = theOptions.m_strSongPath + m_strTitle + ".sng";
	if (!theFile.Open(strFilename, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite))	{
		AfxMessageBox("Cannot open " + strFilename, MB_ICONSTOP);
		return;
	}
	CArchive archive(&theFile, CArchive::store);
	Serialize(archive);
	theFile.Flush();
	theOptions.m_strLastSong = m_strTitle;
}
#endif

#ifdef _DEBUG

void Song::DumpSong() const
{
	CStdioFile dumper(theOptions.m_strSongPath + m_strTitle + ".txt", CFile::modeCreate | CFile::modeWrite);
	char line[200];
	sprintf(line, "\nm_nChordCount: %d", m_nChordCount);
	dumper.WriteString(line);
	for (unsigned int i = 0; i < m_nChordCount; i++)	{
		ASSERT(i < maxchords);
		CChord& rch = m_aEvents[i].m_Chord;
		Event& rev = m_aEvents[i];
		sprintf(line, "\n%03d: Chord: root=%02d, bass=%02d, attr=%02x, ", i, rch.root&0xf, (rch.root>>4)&0xf, rch.attr);
		dumper.WriteString(line);
		sprintf(line, "Event: m_nEventType=%02d, m_nStyleChange=%02d, m_nEndingNumber=%03d",
			rev.m_nEventType,
			rev.m_nStyleChange,
			rev.m_nEndingNumber);
		dumper.WriteString(line);
	}
}

#endif
