// --------- undo.h
#ifndef UNDO_H
#define UNDO_H

#define bool BOOL
#define false FALSE
#define true TRUE

class UndoNode	{
	bool m_bAnchor;		// undo action stream terminator
public:
	UndoNode(bool bStophere = true) : m_bAnchor(bStophere)
		{ }
	virtual ~UndoNode()	{ }
	virtual void Undo() = 0;
	bool Anchor() const
		{ return m_bAnchor; }
};

//------------------------------------------------------------------------------
template <class D, class T>
class UndoItem : public UndoNode	{
protected:
	D&  m_rDoc;
	T*  m_pPosition;	// document position
	int m_nCount;		// item count
public:
    UndoItem(D& rDoc, T* pPosition = 0, int nCount = 0, bool bStophere = true) : 
		UndoNode(bStophere), m_rDoc(rDoc), m_pPosition(pPosition), m_nCount(nCount)
		{ }
    virtual ~UndoItem() { }
};
//------------------------------------------------------------------------------
template <class D, class T>
class UndoInsertNode : public UndoItem<D, T>	{
public:
	UndoInsertNode(D& rDoc, T* pPosition, int nCount, bool bStophere) : 
		UndoItem<D,T>(rDoc, pPosition, nCount, bStophere)
		{ }
	virtual ~UndoInsertNode() { }
	virtual void Undo()
	{
		m_rDoc.Delete(m_pPosition, m_nCount);
	}
};
//------------------------------------------------------------------------------
template <class D, class T>
class UndoDeleteNode : public UndoItem<D, T>	{
protected:
	T* m_pData;
public:
	UndoDeleteNode(D& rDoc, T* pPosition, int nCount, bool bStophere) : 
		UndoItem<D,T>(rDoc, pPosition, nCount, bStophere)
	{
		m_pData = new T[nCount];
		for (int i = 0; i < nCount; i++)
			*(m_pData + i) = *pPosition++;
	}
	virtual ~UndoDeleteNode()
	{
		delete [] m_pData;
	}
	virtual void Undo()
	{
		m_rDoc.Insert(m_pPosition, m_nCount, m_pData);
		delete [] m_pData;
		m_pData = 0;
	}
};
template <class T>
class UndoReplaceNode : public UndoNode	{
protected:
	T m_Data;
	T* m_pPosition;	// object position in document
public:
	UndoReplaceNode(T* pPosition, bool bStophere = true) : UndoNode(bStophere), m_pPosition(pPosition)
		{ m_Data = *pPosition; }
	virtual void Undo()
		{ *m_pPosition = m_Data; }
};
//------------------------------------------------------------------------------
const int maxUndos = 300;

class UndoData	{
	UndoNode* m_pUndoNode[maxUndos];
	int m_nCurrentUndo;
	int m_nUndoCount;
	bool m_bUndoing;
public:
	UndoData();
	~UndoData();
	void AddUndoNode(UndoNode* pUndoNode);
	void UndoLastAction();
    bool IsUndoDataStored() const
		{ return m_nUndoCount != 0; }
	bool IsUndoing() const
		{ return m_bUndoing; }
	void DeleteAllUndoActions();
};
#endif

